// $Id: Read_Input.cxx 607 2007-01-04 19:52:38Z Larry $
//
// Read_Input.cxx - Source module for DRAWxtl V5.3
// Coded using the FLTK 1.1.6 widget set
//
//     Larry W. Finger, Martin Kroeker and Brian Toby
//
//
// This module contains the following routines:
//
//  Blank_Strip - strips leading spaces from a string
//  check_atom_name - return 1 if input names match, 0 otherwise
//  expand_atom - generates all atoms of a given type within the unit cell
//  fillcube - read a 512 byte record of a dn6 Fouier map
//  findsys - determines crystal system from symmetry information
//  find_lattice_type - interpret a set of x',y',z' symmetry operators to find lattice type
//  get_label - gets 1-4 character atom label from input string
//  getsym - obtain symmetry information from a SHELX-style SYMM card
//  import_cif - get structural information from a CIF (Crystallographic Information File)
//  import_fdat - get structural information from a CSD- or FDAT-format file
//  import_gsas - get structural information from a GSAS-format file
//  import_schakal - get structural information from a SCHAKAL-format file
//  import_shelx - get structural information from a SHELX-format file
//  import_wien - get structural information from a WIEN2k struct file
//  import_discus - get structural information from a DISCUS file
//  import_pcr - get structural information from a FULLPROF-format file
//  Init_DRAWxtl - initialize all global variables
//  position_cif - position the CIF import file at a specific token
//  P_to_C - convert ellipsoid probability to scaling parameter
//  read_dn6 - read O Format (dn6) Fourier file
//  read_fcf - read CIF-standard Fo/Fc file and compute density
//  read_grd - read GSAS (grd) Fourier file
//  read_stf - read JANA (stf) Fourier file
//  read_flp - read FullProf (flp) Fourier file
//  read_w2k - read WIEN2k calculated charge density file
//  read_exc - read EXCITING calculated charge density or ELF file
//  read_vasp - read VASP calculated charge density file
//  read_inp - read and process input ('.str') file
//  set_tf_status - sets the status information regarding temperature factors
//  skip_blocks - skip over data blocks in a CIF
//  Token_Strip - strips tokens from the beginning of a string
//  Transform_POV_Color - change input color info to POV form (if necessary)
//  Transform_VRML_Color - change input color into rgb triple needed by VRML and openGL
//  trim_string - trim strings - remove trailing spaces and control characters
//  Unique_Atom - checks that atom position is unique
//  vec_dif - find if two vectors are equal
//
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <stdlib.h>
#if defined(WIN32)
  #define snprintf _snprintf
#endif
#include <ctype.h>
#include "drawxtl.h"
#include "draw_ext.h"
#include "DRAWxtlViewUI.h"

#include "DRAWxtl_proto.h"

int Block_CIF = 0;
int g_Quick;

/* ************************************************************** */
/* ************************************************************** */

void Blank_Strip (char input[100])

/* routine to suppress any leading spaces in the input string */
{
  int i, j;

  for (i = 0; i < 100; i++)
    if (input[i] != ' ')
      break;
  for (j = 0; j < 100; j++) {
    input[j] = input[i];
    if (input[i++] == '\0')
      break;
  }
}

/* ************************************************************** */
/* ************************************************************** */

int check_atom_name(char * name1, char * name2)
{
// determine if two atom names are the same. Return 1 (TRUE) if they are

    if (name1[0] == name2[0] && name1[1] == name2[1] &&
        name1[2] == name2[2] && name1[3] == name2[3])
        return 1;
    else
        return 0;
}

/* ************************************************************** */
/* ************************************************************** */

float convert_pos(char * string)
{
// routine to convert an atom string to a float. If only digits, a simple sscanf
//  is done. If the position is of the form n/m, each part is decoded and the
// results of the division is returned.

    float result;
    int i, n, m;
    char * pdest;
    char sub_string1[10], sub_string2[10];

    if (!(pdest = strstr(string,"/"))) {
        sscanf(string,"%f",&result);             // no / in the input
    } else {
        i = pdest - string +1;                   // number of chars to copy
        strncpy(sub_string1, string, i);
        sub_string1[i - 1] = '\0';
        strcpy(sub_string2, pdest + 1);          // copy part after /
        sscanf(sub_string1,"%d", &n);
        sscanf(sub_string2,"%d", &m);
        result = (float)n/(float)m;
    }
    return result;
}

/* ************************************************************** */
/* ************************************************************** */

int end_flip(int value)
{
// routine to flip the endian nature of the input integer
// This routine was copied from a Web article:
//   http://www.codeproject.com/cpp/endianness.asp by Juan Carlos Cobas

   return (((value&0x000000FF)<<24)+((value&0x0000FF00)<<8)+
   ((value&0x00FF0000)>>8)+((value&0xFF000000)>>24));
}

/* ************************************************************** */
/* ************************************************************** */

float end_flip_real(float value)
{
// routine to flip the endian nature of the input float 
// This routine was copied from a Web article:
//   http://www.codeproject.com/cpp/endianness.asp by Juan Carlos Cobas

   union u {float vi; unsigned char c[4];}; 
   union v {float ni; unsigned char d[4];};
   union u un; 
   union v vn; 
   un.vi = value;
   vn.d[0]=un.c[3]; 
   vn.d[1]=un.c[2]; 
   vn.d[2]=un.c[1]; 
   vn.d[3]=un.c[0]; 
   return (vn.ni); 

}

/* ************************************************************** */
/* ************************************************************** */

void expand_atom (int natom)

/* routine to expand an atom position and obtain all equipoints within
   the unit cell */
/* natom - number of atom in list to expand */
{
  float xp[3], xpp[3];                 /* used in generating position */
  int i, j, k, l;                 /* loop variables */

  ncell = 0;                     /* clear list */
  for (i = 0; i < drvui->ng; ++i) {             /* symmetry operations */
      for (j = 0; j <= 2; ++j) {
          xp[j] = drvui->ts[i][j];
          for (k = 0; k <= 2; ++k)
              xp[j] = xp[j] + drvui->ss[i][j][k] * drvui->atom_xyz[natom][k];
      }
      for (k = 0; k < drvui->nlat; ++k) {         /*lattice points */
          for (l = 0; l <= 2; ++l)
              xpp[l] = xp[l] + drvui->lat_pos[k][l];
          add_to_list (xpp, i);
          if (!drvui->acentric) {
             for (l = 0; l <= 2; ++l)
                 xpp[l] = -xpp[l];
             add_to_list (xpp, i);
          }
      }
  }
}

/* ************************************************************** */
/* ************************************************************** */

int fillcube(uchar * cube, FILE * mapin)
{
/* routine to read the next record of a dn6 map.
 *
 * The first record is a header. After that, each record contains
 * an 8 x 8 x 8 piece of the map scaled so that each map point
 * occupies a single byte.
 */

    int i, val;
    for (i=0; i<512; i++) {
        if ((val = fgetc(mapin)) == EOF)
            return 0;
        cube[i] = (uchar)val;
    }
    return 1;
}

/* ************************************************************** */
/* ************************************************************** */

void findsys(void) 
/* routine determine crystal system from symmetry information */
{
  int i,j,k;
  int n,kk=0,nn,symlist[23];
  int yzx=0,zxy=0,ymxmz=0,mxmyz=0,myxmz=0;

  if (drvui->ng==1) { /* only x,y,z => triclinic */
    drvui->sys = 1;
    return;
  }

  if (drvui->ng==2 && fabs(drvui->lat_con[4]-90.)<0.00001) { /* only two symops => monoclinic */
    drvui->sys = 2;
    return;
  }

  drvui->sys = 3; /* orthorhombic unless we find specific operators below */

  for (i = 0; i < drvui->ng; ++i) {
    for (j = 0; j <= 2; ++j) {
      if(fabs(drvui->ts[i][j]-0.33333333) <0.00001) drvui->sys=5;
      if(fabs(drvui->ts[i][j]-0.16666667) <0.00001) drvui->sys=5;
      if(fabs(drvui->ts[i][j]-0.66666667) <0.00001) drvui->sys=5;
      if(fabs(drvui->ts[i][j]-0.83333333) <0.00001) drvui->sys=5;
    }
    if (drvui->sys==5) return;
  }

  if (fabs(drvui->lat_con[5] - 120.0) < 0.00001
      && fabs(drvui->lat_con[4] - 90.0) <0.0001) {
    drvui->sys = 5;
    return;
  }

#if 0
  if (fabs(drvui->lat_con[5] - 54.27) < 0.1
      &&fabs(drvui->lat_con[4] - 54.27) < 0.1
      && fabs(drvui->lat_con[3] - 54.27) <0.1) {
    drvui->sys = 5;
    return;
  }
#endif

  n=0;
  for (i = 0; i < drvui->ng; ++i) {
    for (j = 0; j <= 2; ++j) {
       if (drvui->ts[i][j] != 0.0) break;
       kk=0;
       for (k = 0; k <= 2; ++k)
         if (drvui->ss[i][j][k] != 0) kk++;
    }
    if (kk >1) {
      drvui->sys = 5;
      return;
    }
    symlist[n++]=i;
  }

  for (nn=0; nn < n; nn++) {
    i=symlist[nn];
    if (drvui->ss[i][0][1]==1 && 
        drvui->ss[i][1][2]==1 &&
        drvui->ss[i][2][0]==1) yzx=1;
    if (drvui->ss[i][0][2]==1 && 
        drvui->ss[i][1][0]==1 &&
        drvui->ss[i][2][1]==1) zxy=1;
    if (drvui->ss[i][0][1]==-1 && 
        drvui->ss[i][1][2]==-1 &&
        drvui->ss[i][2][0]==-1) yzx=1;
    if (drvui->ss[i][0][2]==-1 && 
        drvui->ss[i][1][0]==-1 &&
        drvui->ss[i][2][1]==-1) zxy=1;
    if (drvui->ss[i][0][1]==1 && 
        drvui->ss[i][1][0]==-1 &&
        drvui->ss[i][2][2]==-1) ymxmz=1;
    if (drvui->ss[i][0][0]==-1 && 
        drvui->ss[i][1][1]==-1 &&
        drvui->ss[i][2][2]==1) mxmyz=1;
    if (drvui->ss[i][0][1]==-1 && 
        drvui->ss[i][1][0]==1 &&
        drvui->ss[i][2][2]==-1) myxmz=1;
  }

  if (ymxmz+mxmyz+myxmz==3) drvui->sys=4; /*tetragonal*/
  if (yzx+zxy==2) drvui->sys=6; /*cubic*/

  if (drvui->sys==3 && fabs(drvui->lat_con[4] -90.0) > 0.0001 ) drvui->sys=2;
}

/* ************************************************************** */
/* ************************************************************** */

void get_label (char input[], char *c1, char *c2, char *c3, char *c4,
        int strip)

// Routine to get 1 to 4 character atom ID from input line, and shift input
//   line to left, if strip true, skip over first token
{

  int i, j;

  i = 0;
  if (strip)
    while (input[i] != ' ')
      input[i++] = ' ';            // skip over command
  while (input[i] == ' ')
    i++;                           // skip whitespace after command
  *c1 = input[i];                  // return first atom label character
  input[i++] = ' ';
  *c2 = input[i];                  // return 2nd atom label char
  if (input[i] != ' ')
    input[i++] = ' ';              // if 2nd not blank, rub it out
  *c3 = input[i];                  // return 3rd character
  if (input[i] != ' ')
    input[i++] = ' ';              // if 3rd not blank, rub it out
  *c4 = input[i];                  // return 4th character
  if (input[i] != ' ')
    input[i++] = ' ';              // if 4th not blank, rub it out
  while (input[i] == ' ')
    i++;                          // skip white space after atom label
  for (j = 0; i < (int)strlen(input); i++) {
    input[j++] = input[i];        // copy rest of command
  }
  input[j] = 0;                   // put NULL at end
}                                 // end of get_label

/* ************************************************************** */
/* ************************************************************** */

void getsym(char* text, int num, int kk)

/* helper routine to parse SHELX or CIF symmetry operator strings */
{ 
    int i,j;
    char *txt;
    char *n;
    char *d;

    txt = (char*)zalloc(24*sizeof(char));
//    if (!g_Quick) fprintf(drvui->flout, "Entering getsym with text = %s, num = %d\n",text,num);
 
    j=0;

    if (drvui->modulated >= 1) {
      for (i=0;i<(int)strlen(text)-1;i++) {
        if (text[i]=='X' || text[i]=='x') {
          if (text[i+1]=='2') text[i]= 'Y';
          if (text[i+1]=='3') text[i]= 'Z';
          if (text[i+1]=='4') text[i]= 'Q';
          if (text[i+1]=='5') text[i]= 'R';
          if (text[i+1]=='6') text[i]= 'S';
          if (text[i]!='X'&& text[i]!='x') 
            text[++i]=' ';
        }
      }
    }

    for (i=0;i< (int)strlen(text);i++)
      if (text[i] != ' ' && text[i] != '\n')
        txt[j++]=toupper(text[i]);

    txt[j]='\0';

    if (kk <3) 
      drvui->ts[num][kk] = 0.;
    else
      drvui->ts_m[num][kk-3] = 0.;

    n = strstr(txt,"/");
    if (n != NULL) {
      n++;
      d = n-3;
      if (d < txt || atof(d)==0.) d=n-2;
      if (kk<3)
        drvui->ts[num][kk] = (float)(atof(d) / atof(n));
      else
        drvui->ts_m[num][kk-3] = (float)(atof(d) / atof(n));
    }

    n= strstr(txt,".");
    if (n != NULL) {
      if (kk < 3) {
        drvui->ts[num][kk] = (float)atof(n);
        if (n != txt) {
            if (txt[&txt-&n+1] == '-' )
                drvui->ts[num][kk] *= -1;
        }
      } else {
        drvui->ts_m[num][kk-3] = (float)atof(n);
        if (n != txt) {
            if (txt[&txt-&n+1] == '-' )
                drvui->ts_m[num][kk-3] *= -1;
        }
      }
    }

    if (kk < 3) {
      drvui->ss[num][kk][0]=0;
      drvui->ss[num][kk][1]=0;
      drvui->ss[num][kk][2]=0;
    } else {
      drvui->ss_m[num][kk-3][0]=0;
      drvui->ss_m[num][kk-3][1]=0;
      drvui->ss_m[num][kk-3][2]=0;
    }

    for (i=0; i<(int)strlen(txt); i++) {
        if (txt[i]=='X') {
            drvui->ss[num][kk][0]=1;
            if (i > 0) {
                if (txt[i-1]=='-') drvui->ss[num][kk][0]=-1;
            }
        }
        if (txt[i]=='Y') {
            drvui->ss[num][kk][1]=1;
            if (i > 0) {
                if (txt[i-1]=='-') drvui->ss[num][kk][1]=-1;
            }
        }     
        if (txt[i]=='Z') {
            drvui->ss[num][kk][2]=1;
            if (i > 0) {
                if (txt[i-1]=='-') drvui->ss[num][kk][2]=-1;
            }
        }
        if (txt[i]=='Q') {
            drvui->ss_m[num][kk-3][0]=1;
            if (i > 0) {
                if (txt[i-1]=='-') drvui->ss_m[num][kk-3][0]=-1;
            }
        }
        if (txt[i]=='R') {
             drvui->ss_m[num][kk-3][1]=1;
             if (i > 0) {
                 if (txt[i-1]=='-') drvui->ss_m[num][kk-3][1]=-1;
             }
        }     
        if (txt[i]=='S') {
            drvui->ss_m[num][kk-3][2]=1;
            if (i >0) {
                if (txt[i-1]=='-') drvui->ss_m[num][kk-3][2]=-1;
            }
        }
    }
    free(txt);
/*
    if (!g_Quick) {
       fprintf(drvui->flout, "ss: %d %d %d / %d %d %d / %d %d %d\n", drvui->ss[num][0][0],
            drvui->ss[num][0][1], drvui->ss[num][0][2], drvui->ss[num][1][0],
            drvui->ss[num][1][1], drvui->ss[num][1][2], drvui->ss[num][2][0],
            drvui->ss[num][2][1], drvui->ss[num][2][2]);
       fprintf(drvui->flout, "ts: %f %f %f\n", drvui->ts[num][0], drvui->ts[num][1], drvui->ts[num][2]);
       fprintf(drvui->flout, "ss_m: %d %d %d / %d %d %d / %d %d %d\n", drvui->ss_m[num][0][0],
            drvui->ss_m[num][0][1], drvui->ss_m[num][0][2], drvui->ss_m[num][1][0],
            drvui->ss_m[num][1][1], drvui->ss_m[num][1][2], drvui->ss_m[num][2][0],
            drvui->ss_m[num][2][1], drvui->ss_m[num][2][2]);
       fprintf(drvui->flout, "ts_m: %f %f %f\n", drvui->ts_m[num][0], drvui->ts_m[num][1], drvui->ts_m[num][2]);
    }
*/
}

/* ************************************************************** */
/* ************************************************************** */

int get_next_token( char *p, int max_len, FILE * fpin)
{
/* routine to return the next token in a stream. A token is the string
 * characters to the next white space consisting of a space, a tab, or
 * a newline character. Carriage returns are ignored. The returned value
 * is 1 (true) if a token is found and 0 (false) if the end of file is
 * reached */

    char *pp = p;
    int inchar;
    int started = 0;
    int openquote = -1;
    int len = max_len - 2;

    for (;;) {        /* start an infinite loop */
        if((inchar = getc(fpin)) == EOF)
            return 0;             /* here for end-of-file */
        if (inchar == '\t' || inchar == ' ') {
            if (started && openquote != 1) {
                *pp = '\0';       /* add null terminator */
                return 1;
            }
        } else if (inchar == '\n') {
            if (started) {
                *pp = '\0';
                return 1;
            }
        } else if (inchar == '#' && !started) { // comment - skip to end of line
                for (;;) {
                  if((inchar = getc(fpin)) == EOF)
                    return 0;             /* here for end-of-file */
                  if (inchar=='\n')
                    break;
                }
        } else if (inchar != '\r') {
            *pp++ = (char)inchar;
            started = 1;
            if (inchar == (char)39)
              openquote *= -1;
            if (!--len)
                return 0;
        }
    }
}

/* ************************************************************** */
/* ************************************************************** */

int position_cif(int numblocks, FILE * impin, char * search_string, char * string, int error)
{
/* routine to search through a CIF until the token in 'search_string' is found 
   If the string is not found, the routine returns 1. If 'error' is non-zero, file 'impin' is closed.
   If 'search_string' is found, the routine returns 0
 */

    rewind (impin);
    memset(string, 0, 100);
    if (numblocks > 1) skip_blocks(numblocks-1, impin);
    while (strncmp(string, search_string, strlen(search_string))) {
        if (!get_next_token(string, 100, impin)) {
            if (error) (void)fclose(impin);
            return 1;
        }
    }
    return 0;
}

/* ************************************************************** */
/* ************************************************************** */

void find_lattice_type( void)
{
/* routine to scan symmetry elements, find the lattice type, and fill in the coorcdinates
   of the lattice points */

    int i;

    drvui->nbr=1;

    for (i = 1; i <= drvui->ng; ++i) {
        if (drvui->ss[i-1][0][0] == 1 && drvui->ss[i-1][1][1] == 1 &&
            drvui->ss[i-1][2][2] == 1) {  /* rotational part of symmetry is x,y,z - centering? */
            if (drvui->ts[i-1][0]== 0. && drvui->ts[i-1][1] == 0.5 &&
                drvui->ts[i-1][2] == 0.5) { /* found x,1/2+y,1/2+z */
                if (drvui->nbr == 3 || drvui->nbr == 4)
                    drvui->nbr = 5;  /* x,1/2+y,1/2+z && B || C ==> F */   
                else
                    drvui->nbr = 2;  /* x,1/2+y,1/2+z ==> A */
            }
            if (drvui->ts[i-1][0]== 0.5 && drvui->ts[i-1][1] == 0. &&
                drvui->ts[i-1][2] == 0.5) { /* found 1/2+x,y,1/2+z */
                if (drvui->nbr == 2 || drvui->nbr == 4)
                    drvui->nbr = 5;  /* 1/2+x,y,1/2+z && A || C ==> F */   
                else
                    drvui->nbr = 3;  /* 1/2+x,y,1/2+z ==> B */
            }
            if (drvui->ts[i-1][0]== 0.5 && drvui->ts[i-1][1] == 0.5 &&
                drvui->ts[i-1][2] == 0.) { /* found 1/2+x,1/2+y,z */
                if (drvui->nbr == 2 || drvui->nbr == 3)
                    drvui->nbr = 5;  /* 1/2+x,1/2+y,z && A || B ==> F */   
                else
                    drvui->nbr = 4;  /* 1/2+x,1/2+y,z ==> C */
            }
            if (drvui->nbr==1 && drvui->ts[i-1][0]==0.5 && drvui->ts[i-1][1]==0.5 &&
                drvui->ts[i-1][2]==0.5)
                   drvui->nbr=6;  // 0.5+X,0.5+Y,0.5+Z ==> I
			
            if (drvui->ts[i-1][0]<0.5 && 
                drvui->ts[i-1][0]>0.25 )
                drvui->nbr=7;  // 0.33+X ==> R
        }

    }

    switch (drvui->nbr) {
    case 1:
        drvui->nlat = 1;
        break;
    case 2:
        drvui->nlat = 2;
        drvui->lat_pos[1][0] = 0.0;
        drvui->lat_pos[1][1] = 0.5f;
        drvui->lat_pos[1][2] = 0.5f;
        break;
    case 3:
        drvui->nlat = 2;
        drvui->lat_pos[1][0] = 0.5f;
        drvui->lat_pos[1][1] = 0.0;
        drvui->lat_pos[1][2] = 0.5f;
        break;
    case 4:
        drvui->nlat = 2;
        drvui->lat_pos[1][0] = 0.5f;
        drvui->lat_pos[1][1] = 0.5f;
        drvui->lat_pos[1][2] = 0.0;
        break;
    case 5:
        drvui->nlat = 4;
        drvui->lat_pos[1][0] = 0.0;
        drvui->lat_pos[1][1] = 0.5f;
        drvui->lat_pos[1][2] = 0.5f;
        drvui->lat_pos[2][0] = 0.5f;
        drvui->lat_pos[2][1] = 0.0;
        drvui->lat_pos[2][2] = 0.5f;
        drvui->lat_pos[3][0] = 0.5f;
        drvui->lat_pos[3][1] = 0.5f;
        drvui->lat_pos[3][2] = 0.0;
        break;
    case 6:
        drvui->nlat = 2;
        drvui->lat_pos[1][0] = 0.5f;
        drvui->lat_pos[1][1] = 0.5f;
        drvui->lat_pos[1][2] = 0.5f;
        break;
    case 7:
        drvui->nlat = 3;
        drvui->lat_pos[1][0] = (1.f / 3.f);
        drvui->lat_pos[1][1] = (2.f / 3.f);
        drvui->lat_pos[1][2] = (2.f / 3.f);
        drvui->lat_pos[2][0] = (2.f / 3.f);
        drvui->lat_pos[2][1] = (1.f / 3.f);
        drvui->lat_pos[2][2] = (1.f / 3.f);
    }
}


/* ************************************************************** */
/* ************************************************************** */

void import_cif (char input[100], int in_line, int Quick, int * Block, int frame_no)

/* routine to extract structural information from a CIF */
{
  FILE *impin;
  char string[100], filename[256], tstring[100];
  int U_pos[6], X_pos[3], Id_pos=0, O_pos=0, Label_pos = 0;
  int i, j, sg;
  int n_items, items;
  static int numblocks = 0;
  static char save_filename[256] = "";
  long skip_id;

  g_Quick = Quick;
  memset(U_pos, 0, sizeof(U_pos));
  memset(X_pos, 0, sizeof(X_pos));
  if (in_line) {
      Error_Box("Cannot read inline CIF data. Please use the import instruction");
      return;
  }
//  (void)sscanf (input, "%s %s", filename, filename);     /* get file name */

  strcpy(filename,input);

  if (!(impin = fopen (filename, "r"))) {
      if (!Quick) 
        fprintf(drvui->flout,"Unable to open CIF file %s\n",filename);
      Error_Box("Cannot Open CIF Import File, Run aborted.");
      return;
  }

  if (strcmp(filename, save_filename)) {
      numblocks = 0;
      strcpy(save_filename, filename);
  }
  
  memset(string,0,100);
  memset(tstring,0,100);

  if (*Block)
      numblocks = *Block;

  if (!numblocks) {
      char dataname[2040] = "";
      for (;;) {
          if (!get_next_token(string, 100, impin))
              break;               /* out on EOF */
          if (!strncmp(string,"data_",5)) {
              char tmp[40],tmp2[10];
              sscanf(string,"%s",tmp);
              sprintf(tmp2," %d. ", ++numblocks);
              strcat(dataname,tmp2);
              strcat(dataname,tmp);
              strcat(dataname,"\n");
          }
      }
      if (numblocks > 1) {
          sprintf(tstring,"This CIF has %d data blocks with labels\n%s.\nPlease enter the number"
              " of the one to use:", numblocks, dataname);
          const char *which = fl_input(tstring ,"1");
          if (!which) return; // Cancel button was pressed 
          strcpy(string,which);	  
          (void)sscanf(string,"%d",&numblocks);
          *Block = numblocks;                    /* return to caller */
      } else {
          *Block = 1;
      }
  }
  if (position_cif(numblocks, impin, "_cell_length_a", string, 1)) {
      Error_Box("Error finding cell (a) in CIF Import File, Run aborted.");
      return;
  }
  get_next_token(string,100,impin);
  (void)sscanf (string, "%f", &drvui->lat_con[0]);
  if (position_cif(numblocks, impin, "_cell_length_b", string, 1)) {
      Error_Box("Error finding cell (b) in CIF Import File, Run aborted.");
      return;
  }
  get_next_token(string,100,impin);
  (void)sscanf (string, "%f", &drvui->lat_con[1]);
  if (position_cif(numblocks, impin, "_cell_length_c", string, 1)) {
      Error_Box("Error finding cell (c) in CIF Import File, Run aborted.");
      return;
  }
  get_next_token(string,100,impin);
  (void)sscanf (string, "%f", &drvui->lat_con[2]);
  if (position_cif(numblocks, impin, "_cell_angle_al", string, 1)) {
      Error_Box("Error finding cell (alpha) in CIF Import File, Run aborted.");
      return;
  }
  get_next_token(string,100,impin);
  (void)sscanf (string, "%f", &drvui->lat_con[3]);
  if (position_cif(numblocks, impin, "_cell_angle_be", string, 1)) {
      Error_Box("Error finding cell (beta) in CIF Import File, Run aborted.");
      return;
  }
  get_next_token(string,100,impin);
  (void)sscanf (string, "%f", &drvui->lat_con[4]);
  if (position_cif(numblocks, impin, "_cell_angle_ga", string, 1)) {
      Error_Box("Error finding cell (gamma) in CIF Import File, Run aborted.");
      return;
  }
  get_next_token(string,100,impin);
  (void)sscanf (string, "%f", &drvui->lat_con[5]);
  if (!Quick) {
    fprintf (drvui->flout,
         "******      Cell     %8.5f %8.5f %8.5f %8.3f %8.3f %8.3f\n",
         drvui->lat_con[0], drvui->lat_con[1], drvui->lat_con[2], drvui->lat_con[3], drvui->lat_con[4],
         drvui->lat_con[5]);
    if (drvui->lat_con[0] == drvui->lat_con[1]
        && drvui->lat_con[1] == drvui->lat_con[2]
        && drvui->lat_con[3] == drvui->lat_con[4]
        && drvui->lat_con[4] == drvui->lat_con[5]
        && drvui->lat_con[3] != 0.0 && fabs(drvui->lat_con[3] - 90.0f) > 0.01f) {
      fprintf( drvui->flout,"***** Warning - rhombohedral settings are not supported.\n");
      fprintf( drvui->flout,"***** Please transform to standard hexagonal setting.\n");
      Error_Box("Rhombohedral cells are not supported\n"
                "Please transform to standard hexagonal setting first\n");
    }
  }
  i = -1;
  if (position_cif(numblocks, impin, "_space_group_ssg_name", string, 0)) {
      i = 0;                     /* indicate not found */
      if (!position_cif(numblocks, impin, "_space_group_symop_ssg_id", string, 0))
        i = -1;
  }
  j = 1;
  sg = 0;

  if (i==0) {
  if (position_cif(numblocks, impin, "_symmetry_space", string, 0)) 
      j = 0;                     /* indicate not found */
  }

  if (i == -1 && drvui->modulated != -1) {
    if (!Quick) {
      fprintf (drvui->flout, "******      this CIF describes a modulated structure -- \n");
      fprintf (drvui->flout, "******      use the 'average' keyword to display average structure only\n");
    }
    drvui->modulated = 1;
    j = 1;
  }

  if (j != 0) {
    fgets(string,100,impin);
    tstring[0] = 's';
    tstring[1] = 'p';
    tstring[2] = 'g';
    tstring[3] = 'r';
    tstring[4] = ' ';
    if (string[0] != '?' && strchr(string,(char)39) ) {
      for (i = 0; i < 60; i++)
        if (string[i] == (char) 39)
          break;
      if (strchr(string,':') ) { // WJJ notation has reference sg between colons
        for (i++;i<60;i++)
          if (string[i] == ':')
            break;
      }
      for (i++, j = 5; i < 60; i++)
        if (string[i] == (char) 39 || string[i] == (char) 40 || string[i] == ':')
          break;
        else
          tstring[j++] = string[i];
        tstring[j] = '\0';
        for (i = 0; i < 60; i++) {
          if (tstring[i] == 'H')
            tstring[i] = ' ';             /* remove any H from rhomb. spgrps */
          if (tstring[i] == 'S')
            tstring[i] = ' ';             /* remove any S from monoclinic spgrps */
        }

        if  (tstring[6] != ' ') { /* no space after lattice symbol, need to dissect line first */
          int k,k3,k4,l;
          char spstring[60];
     /* separate class symbol from other symmetry elements */
          spstring[0] = 's';
          spstring[1] = 'p';
          spstring[2] = 'g';
          spstring[3] = 'r';
          spstring[4] = ' ';
          spstring[5] = tstring[5];
          spstring[6] = ' ';
          j = 6;
          k = 0;
          k3 = 0;
          k4 = 0;
          l = (int) strlen (tstring);
          for (i = 6; i <= l; i++) {
            switch (tstring[i]) {
            case ' ':
              break;
            case '/':
              spstring[++j] = tstring[i];
              break;
            default:
              j++;
    /* do we have to collate this symbol (for screw axes or mirror planes) ? */
              if (tstring[i] > '0' && tstring[i] <= '9') { /* it is a number */
                if (tstring[0] == 'R' ||             /* no screw axes in R */
                l == 4 ||                 /* no screw axes in four letter names */
                k != 0 ||                 /* have just appended to a number */
                k3 != 0 ||                 /* cannot have further screw axes after 3_x */
                (k4 != 0 && spstring[j - 1] == '3')) {    /* cannot have 3_x after a 4_x */
                  spstring[j] = ' ';             /* so separate it */
                  k = 0;                 /* and reset the collation flag */
                  j++;
                } else if (spstring[j - 1] != '-' && (spstring[j - 1] <= string[i]
                            || spstring[j - 1] >= 'a')) {
     /* neither a minus sign nor a higher number precedes it */
                  spstring[j] = ' ';             /* so separate it */
                  k = 0;                 /* and reset the collation flag */
                  j++;
                } else {
                  k++;                     /* set flag for collation */
                  if (spstring[j - 1] == '3')
                    k3 = 1;                 /* if 3_x screw, prohibit further screw axes */
                  if (spstring[j - 1] == '4')
                    k4 = 1;                 /* if 4_x screw, prohibit 3_x screw axes */
                }
              } else {                     /* not a number */
                if (spstring[j - 1] != '/') {         /*but not a mirror plane either */
                  spstring[j] = ' ';             /* separate it */
                  j++;
                }
              } 
              spstring[j] = tstring[i];             /* now add the symbol */
              break;
            }                         /*switch */
          }                         /*for i */
          spstring[j + 1] = '\0';

      /* Space group 90, P 4 21 2, is not correctly identified by this code. 
         It accounts for only 14 of the 175000 entries in the CSD (and not all 
         of them have coordinates), so this quick and dirty fix is all it can
         expect from us. Does this spacegroup really exist ? :-) */
         if (!strcmp (tstring, "P4212"))
           strcpy (spstring, "P 4 21 2");

         if(!Quick)fprintf (drvui->flout, "******  %s\n", spstring);
         symop (spstring);
      } else {
        if (!Quick)
          fprintf (drvui->flout, "******  %s\n", tstring);
        symop (tstring);
      }
      sg = 1;
    } else {
      if (!Quick)
        fprintf (drvui->flout, "******      this CIF does not contain a space group name\n");
    }
  } else {
    if (!Quick)
      fprintf (drvui->flout, "******      this CIF does not contain a space group record\n");
  }
    
  if (sg == 0 && drvui->sys == 0) {
    j = 1;
    if (position_cif(numblocks, impin, "_symmetry_equiv_pos_site_id", string, 0))
      skip_id=0;       // no id token in the symop table
    else
      skip_id=ftell(impin); // save position of id token
    
    if (skip_id == 0) { // try modern spelling
    if (position_cif(numblocks, impin, "_space_group_symop_id", string, 0))
      skip_id=0;       // no id token in the symop table
    else
      skip_id=ftell(impin); // save position of id token
    }
    if (position_cif(numblocks, impin, "_symmetry_equiv_pos_as_xyz", string, 0))
      j = 0;                     /* indicate not found, but no error */
    if (j == 0) { /* try modern spelling */
      if (position_cif(numblocks, impin, "_space_group_symop_operation_xyz", string, 0))
      j = 0;
    }                     /* indicate not found, but no error */
    if ( j == 0) {
      if (!Quick)
        fprintf (drvui->flout, "******      this CIF contains neither a space group nor a list of symmetry operations\n");
    } else {
      char *p,*pp;
      int k;
      drvui->acentric=1;
    /* initialize symmetry arrays */
      for (i = 0; i <= 23; ++i) {
        for (j = 0; j <= 2; ++j) {
          drvui->ts[i][j] = 0.0;
          for (k = 0; k <= 2; ++k)
            drvui->ss[i][j][k] = 0;
        }
      }
      for (i = 0; i <= 2; ++i) {
        drvui->lat_pos[0][i] = 0.0;
        drvui->ss[0][i][i] = 1;
        for (j = 0; j <= 3; ++j)
          drvui->spg[i][j] = ' ';
      }
      drvui->ng=0;

/* find out our current position (i.e. that of the _pos_as_xyz token) 
   in the CIF file. if it is smaller than that of the _id token, 
   we need not strip id numbers from the symmetry operation table */

      if (ftell(impin) < skip_id) 
        skip_id = 0;

      fgets(string,100,impin);

      while (strlen(string)>2) {
        fgets(string,100,impin);
        for (j=0;j<(int)strlen(string);j++)
            if (string[j]==(unsigned char)39) string[j]=' ';  /* 39 == single quote */
       if (skip_id >0) 
         Token_Strip(string,2); 
       Blank_Strip (string);
        if (strlen(string)<3) 
            break;
        if (strstr(string,"-x, -y, -z")) {
            drvui->acentric=0;
            break;
        }
        drvui->ng++;
        if (drvui->ng >23 ) {
          Error_Box ("Please supply a spacegroup name - either in the CIF file itself\n"
                     "or in a 'spgr' command preceding the 'import' line in the str file");
          break;
        }
        p = strstr(string,",");
        *p='\0';
        getsym(string,drvui->ng-1,0);
        *p=' ';
        p++;
        pp = strstr(p,",");
        *pp='\0';
        getsym(p,drvui->ng-1,1);
        pp++;
        getsym(pp,drvui->ng-1,2);
      }
      findsys();
      find_lattice_type();
    }
  }

  if (drvui->modulated >= 1) { // parse all symmetry lines to preserve superspace symmetry
    j = 1;
    if (position_cif(numblocks, impin, "_space_group_symop_ssg_id", string, 0))
      skip_id=0;       // no id token in the symop table
    else
      skip_id=ftell(impin); // save position of id token
    if (position_cif(numblocks, impin, "_space_group_symop_ssg_operation_algebraic", string, 0))
      j = 0;                     /* indicate not found, but no error */
    if ( j == 0) {
      if (!Quick)
        fprintf (drvui->flout, "******      this CIF does not contain superspace symmetry operations\n");
    } else {
      char *p,*pp;
      int k;
      drvui->acentric=1;
    /* initialize symmetry arrays */
      for (i = 0; i <= 23; ++i) {
        for (j = 0; j <= 2; ++j) {
          drvui->ts[i][j] = 0.0;
          drvui->ts_m[i][j]=0.0;
          for (k = 0; k <= 2; ++k) {
            drvui->ss[i][j][k] = 0;
            drvui->ss_m[i][j][k]=0;
          }
        }
      }
      for (i = 0; i <= 2; ++i) {
        drvui->lat_pos[0][i] = 0.0;
        drvui->ss[0][i][i] = 1;
        drvui->ss_m[0][i][i] = 1;
        for (j = 0; j <= 3; ++j)
          drvui->spg[i][j] = ' ';
      }
      for (i = 0;i < 3; ++i) {
        drvui->cell_vec[0][i] = 0.0;
        drvui->cell_vec[1][i] = 0.0;
        drvui->cell_vec[2][i] = 0.0;
      }
      drvui->ng=0;

/* find out our current position (i.e. that of the _symop_ token) 
   in the CIF file. if it is smaller than that of the _id token, 
   we need not strip id numbers from the symmetry operation table */

      if (ftell(impin) < skip_id) 
        skip_id = 0;              

      fgets(string,100,impin);

      while (strlen(string)>2) {
        int kk;
        char temp_string[100];
        fgets(string,100,impin);
        if (skip_id >0 )
          Token_Strip(string, 2);
        for (j=0;j<(int)strlen(string);j++) 
          if (string[j]==(unsigned char)39) string[j]=' ';
//        trim_string(string,100);
//        if (!strncmp(string,"White",5))
//          break;
        Blank_Strip (string);
        if (strlen(string)<3 || string[0]=='_') 
          break;
        if (!drvui->ng) {
          if (strstr(string,"x5"))     /* get the order of the modulation */
            drvui->modulated = 2;
          if (strstr(string,"x6"))
            drvui->modulated = 3;
        }
        if (drvui->modulated < 3)       /* fill in parts of ss_m not included in input line */
            drvui->ss_m[drvui->ng][2][2] = 1; 
        if (drvui->modulated < 2)
            drvui->ss_m[drvui->ng][1][1] = 1; 
        drvui->ng++;
        strcpy(temp_string, string);
        strcat(temp_string,",");
        p = temp_string;
        for (kk=0; kk<6; kk++) {
          pp = strstr(p,",");
          *pp='\0';
          getsym(p,drvui->ng-1,kk);
          p=++pp;
          if (strlen(p) < 2) kk=6;
        }
      }
      findsys();
      find_lattice_type();
    }
  } 

/* skip through file to atom_site */
  if (position_cif(numblocks, impin, "_atom_site_", string, 1)) {
      Error_Box("Error finding _atom_site_ in CIF Import File, Run aborted.");
      return;
  }

 /* first _atom_site_ line in string need to find position of label, x, y, and z */
  j = 0;
  while (!strncmp (string, "_atom_site_", 11)) {
    if (!strncmp (string, "_atom_site_label", 16))
      Label_pos = j;
    if (!strncmp (string, "_atom_site_fract_x", 18))
      X_pos[0] = j;
    if (!strncmp (string, "_atom_site_fract_y", 18))
      X_pos[1] = j;
    if (!strncmp (string, "_atom_site_fract_z", 18))
      X_pos[2] = j;
    if (!strncmp (string, "_atom_site_occupancy", 20))
      O_pos = j;
    j++;
    if (!get_next_token (string, 100, impin)) {         /* search for next item */
        Error_Box("Error reading CIF Import File, Run aborted.");
        (void)fclose(impin);
        return;
    }
  }
  n_items = j;
  for (;;) {
    if (!strncmp (string, "loop_", 5) || !strncmp(string, "data_", 5) || !strncmp(string,"_",1))
      break;
    for (items=0; items<n_items; items++) {
      if (items == Label_pos) {
        j = 0;
        drvui->atom_n[natom] = 0;
        strcpy(drvui->atom_l[natom],"    ");
        for (i=0; i<(int)strlen(string); i++) {
          if (string[i] >= '0' && string[i] <= '9') {
            drvui->atom_n[natom] = 10 * drvui->atom_n[natom] + (int) string[i] - 48;
          } else {
            drvui->atom_l[natom][j++] = string[i];
            if (j > 3 )
              j=3;
          }
        }
      } else if (items == X_pos[0]) {
        (void)sscanf (string, "%f", &drvui->atom_xyz[natom][0]);
      } else if (items == X_pos[1]) {
        (void)sscanf (string, "%f", &drvui->atom_xyz[natom][1]);
      } else if (items == X_pos[2]) {
        (void)sscanf (string, "%f", &drvui->atom_xyz[natom][2]);
      } else if (items == O_pos) {
        (void)sscanf (string, "%f", &drvui->occupancy[natom]);
      }
      if (!get_next_token (string, 100, impin) || !strlen(string)) {
//        Error_Box("End of File while reading atoms in CIF Import File. loop_ added.");
        strcpy(string,"loop_");
      }
    }
    if (!Quick) {
      fprintf (drvui->flout,
           "******      Atom %c%c%c%c%3d %8.5f %8.5f %8.5f\n", drvui->atom_l[natom][0],
           drvui->atom_l[natom][1], drvui->atom_l[natom][2], drvui->atom_l[natom][3],
           drvui->atom_n[natom], drvui->atom_xyz[natom][0], drvui->atom_xyz[natom][1],
           drvui->atom_xyz[natom][2]);
      fflush(drvui->flout);
    }
    drvui->atom_fn[natom] = frame_no;
    drvui->sv_atom_n[natom] = drvui->atom_n[natom];
    drvui->atom_ismod[natom] = 0;
    drvui->occ_ismod[natom] = 0;
    drvui->min_occ[natom] = 0.;
    if (++natom >= MAX_ATOM) {
        Error_Box("Too many atoms in asymmetric unit - increase MAX_ATOM.");
        (void)fclose(impin);
        return;
    }
  }
  j = 1;
/* skip through file to atom_site_aniso */
  if (position_cif(numblocks, impin, "_atom_site_aniso", string, 0)) {
      j = 0; // indicate not found, but no error
  }

  if ( j == 1) { // only do if there are anisotropic temperature factors
/* first _atom_site_aniso line in string
   need to find position of label, U11, etc */
    j = 0;
    while (!strncmp (string, "_atom_site_aniso", 16)) {
      if (!strncmp (string, "_atom_site_aniso_label", 22))
        Label_pos = j;
      if (!strncmp (string, "_atom_site_aniso_U_11", 21))
        U_pos[0] = j;
      if (!strncmp (string, "_atom_site_aniso_U_22", 21))
        U_pos[1] = j;
      if (!strncmp (string, "_atom_site_aniso_U_33", 21))
        U_pos[2] = j;
      if (!strncmp (string, "_atom_site_aniso_U_12", 21))
        U_pos[3] = j;
      if (!strncmp (string, "_atom_site_aniso_U_13", 21))
        U_pos[4] = j;
      if (!strncmp (string, "_atom_site_aniso_U_23", 21))
        U_pos[5] = j;
      j++;
      if (!get_next_token (string, 100, impin)) {         /* search for next item */
//        Error_Box("End of File while reading Uij in CIF Import File, Run aborted.");
          (void)fclose(impin);
          return;
      }
    }
    n_items = j;
    for (;;) {
      if (!strncmp (string, "_", 1) || strstr(string,"loop_") )
        break;
      for (items=0; items<=n_items-1; items++) {
        if (Label_pos == items) {
          drvui->ellips_n[drvui->n_ellips] = 0;
          strcpy(drvui->ellips_l[drvui->n_ellips], "    ");    /* initialize ellipsoid name */
          j=0;
          for (i=0; i<(int)strlen(string); i++) {
            if (string[i] >= '0' && string[i] <= '9') {
              drvui->ellips_n[drvui->n_ellips] = 10 * drvui->ellips_n[drvui->n_ellips]
                + (int) string[i] - 48;
            } else {
              drvui->ellips_l[drvui->n_ellips][j++] = string[i];
              if (j > 3)
                j = 3;
            }
          }
        }
        drvui->save_el_number[drvui->n_ellips] = drvui->ellips_n[drvui->n_ellips];
        drvui->do_ellipsoids = 1;
        if (U_pos[0] == items)
          (void)sscanf (string, "%f", &drvui->ellips[drvui->n_ellips][0]);
        if (U_pos[1] == items) 
          (void)sscanf (string, "%f", &drvui->ellips[drvui->n_ellips][1]);
        if (U_pos[2] == items)
          (void)sscanf (string, "%f", &drvui->ellips[drvui->n_ellips][2]);
        if (U_pos[3] == items)
          (void)sscanf (string, "%f", &drvui->ellips[drvui->n_ellips][3]);
        if (U_pos[4] == items)
          (void)sscanf (string, "%f", &drvui->ellips[drvui->n_ellips][4]);
        if (U_pos[5] == items)
          (void)sscanf (string, "%f", &drvui->ellips[drvui->n_ellips][5]);
        if (drvui->auto_ellipse==1) {
          drvui->ell_type[drvui->n_ellips] = 1001;         // Uij with ellipcolor
          memset(drvui->ellips_col[drvui->n_ellips],0,40);
          strcpy(drvui->ellips_col[drvui->n_ellips],"Gray30");
        } else {
          drvui->ell_type[drvui->n_ellips] = 1;            // Uij
        }
        if (items <n_items-1) get_next_token(string, 100, impin);
      }
      if(!Quick)fprintf (drvui->flout,
           "******      Uij  %c%c%c%c%3d %8.5f %8.5f %8.5f %8.5f %8.5f %8.5f\n",
           drvui->ellips_l[drvui->n_ellips][0], drvui->ellips_l[drvui->n_ellips][1],
           drvui->ellips_l[drvui->n_ellips][2], drvui->ellips_l[drvui->n_ellips][3],
           drvui->ellips_n[drvui->n_ellips], drvui->ellips[drvui->n_ellips][0], drvui->ellips[drvui->n_ellips][1], 
           drvui->ellips[drvui->n_ellips][2], drvui->ellips[drvui->n_ellips][3], drvui->ellips[drvui->n_ellips][4],
           drvui->ellips[drvui->n_ellips][5]);
      drvui->ellips_ismod[drvui->n_ellips] = 0;
      if (++drvui->n_ellips >= MAX_EL) {
          Error_Box("Too many ellipsoid types specified - increase MAX_EL.");
          return;
      }
      if (!get_next_token (string, 100, impin)) {
//        Error_Box("Error reading CIF Import File, Run aborted.");
          (void)fclose(impin);
          return;
      }
    }
    drvui->auto_ellipse = 0;
  }

  if (drvui->modulated >= 1) {
    int axis=0;
    int elem=0;
    int theatom=0;
    drvui->no_mod_vectors = 0;

/* unit cell wave vectors */

/* skip through file to cell_wave_vector_seq_id line */

  if (position_cif(numblocks, impin, "_cell_wave_vector_seq_id", string, 1)) {
      Error_Box("Error finding _cell_wave_vector in CIF Import File, Run aborted.");
      return;
  }
  j = 0;
  if (!Quick)
    fprintf(drvui->flout, "******\n******  Cell Modulation Vector(s)\n****** No.             "
            "Components\n******\n");
  while (!strncmp (string, "_cell_wave_vector", 17)) {
    if (!strncmp (string, "_cell_wave_vector_x", 19))
      X_pos[0] = j;
    if (!strncmp (string, "_cell_wave_vector_y", 19))
      X_pos[1] = j;
    if (!strncmp (string, "_cell_wave_vector_z", 19))
      X_pos[2] = j;
    j++;
    if (!get_next_token (string, 100, impin)) {         /* search for next item */
        Error_Box("Error reading CIF Import File, Run aborted.");
        (void)fclose(impin);
        return;
    }
  }
  n_items = j;
  drvui->no_cell_vec = 0;
  for (;;) {
    if (!strncmp (string, "loop_", 5) || !strncmp(string, "data_", 5) || !strncmp(string,"_",1))
      break;
    for (items=0; items<n_items; items++) {
      if (items == X_pos[0]) {
        (void)sscanf (string, "%f", &drvui->cell_vec[drvui->no_cell_vec][0]);
      } else if (items == X_pos[1]) {
        (void)sscanf (string, "%f", &drvui->cell_vec[drvui->no_cell_vec][1]);
      } else if (items == X_pos[2]) {
        (void)sscanf (string, "%f", &drvui->cell_vec[drvui->no_cell_vec][2]);
      }
      if (!get_next_token (string, 100, impin) || !strlen(string)) {
        strcpy(string,"loop_");
      }
    }
    if (!Quick) {
      fprintf (drvui->flout,
           "****** %2d     %8.5f %8.5f %8.5f\n", drvui->no_cell_vec+1,
           drvui->cell_vec[drvui->no_cell_vec][0], drvui->cell_vec[drvui->no_cell_vec][1],
           drvui->cell_vec[drvui->no_cell_vec][2]);
      fflush(drvui->flout);
    }
    drvui->no_cell_vec++;
  }
  if (drvui->no_cell_vec && !Quick)
    fprintf(drvui->flout, "******\n");

  /* modulation wave vector(s) */

  /* skip through file to atom_site_Fourier */
    if (position_cif(numblocks, impin, "_atom_site_Fourier_", string, 1)) {
        Error_Box("Error reading modulation wave vector in CIF Import File, Run aborted.");
        return;
    }
  /* first _atom_site_ line in string need to find position of label, x, y, and z */
    j = 0;
    X_pos[0]=X_pos[1]=X_pos[2]=Label_pos=-1;
    if (!Quick)
      fprintf(drvui->flout, "******\n******  Atom Site Modulation Vector(s)\n******  No.     Components         Cell Wave Components\n******\n");
    while (!strncmp (string, "_atom_site_Fourier_", 19)) {
      if (!strncmp (string, "_atom_site_Fourier_wave_vector_seq_id", 37))
        Label_pos = j;
      if (!strncmp (string, "_atom_site_Fourier_wave_vector_x", 32))
        X_pos[0] = j;
      if (!strncmp (string, "_atom_site_Fourier_wave_vector_y", 32))
        X_pos[1] = j;
      if (!strncmp (string, "_atom_site_Fourier_wave_vector_z", 32))
        X_pos[2] = j;
      j++;
      if (!get_next_token (string, 100, impin)) {         /* search for next item */
        Error_Box("Error reading CIF Import File, Run aborted.");
        (void)fclose(impin);
        return;
      }
    }
    n_items = j;
    for (;;) {
      int no_z, max_z = 5;
      int no_y, max_y = 5;
      int no_x, max_x = 5;

      if (drvui->no_cell_vec < 3) max_z = 0;
      if (drvui->no_cell_vec < 2) max_y = 0;
      if (!strncmp (string, "loop_", 5) || !strncmp(string, "data_", 5)) 
        break;
      j = drvui->no_mod_vectors;
      drvui->modvector[j][0]=0.;
      drvui->modvector[j][1]=0.;
      drvui->modvector[j][2]=0.;
      for (items=0; items<=n_items-1; items++) {
        if (X_pos[0] == items)
          (void)sscanf (string, "%f", &drvui->modvector[j][0]);
        if (X_pos[1] == items) 
          (void)sscanf (string, "%f", &drvui->modvector[j][1]);
        if (X_pos[2] == items) 
          (void)sscanf (string, "%f", &drvui->modvector[j][2]);
        if (items <n_items) get_next_token(string, 100, impin);
      }
      for (no_z=-max_z; no_z<=max_z; no_z++) {
        for (no_y=-max_y; no_y<=max_y; no_y++) {
          for (no_x=-max_x; no_x<=max_x; no_x++) {
            if(vec_dif(no_x, drvui->cell_vec[0], no_y, drvui->cell_vec[1], no_z, drvui->cell_vec[2],
                       drvui->modvector[j])) {
              drvui->vector_mult[j][0] = no_x;    /* have components of modvector */
              drvui->vector_mult[j][1] = no_y;
              drvui->vector_mult[j][2] = no_z;
              goto done;
            }
          }
        }
      }
      Error_Box("Unable to decombine modulation wave vector into cell vector parts.");
done:
      if(!Quick) {
         fprintf (drvui->flout,"****** %2d %8.5f %8.5f %8.5f  %3d %3d %3d\n",
                  j+1,drvui->modvector[j][0],drvui->modvector[j][1],drvui->modvector[j][2],
                  drvui->vector_mult[j][0], drvui->vector_mult[j][1], drvui->vector_mult[j][2]);
      }
      drvui->no_mod_vectors++;
    }

  /* displacement modulation - fourier series */

  /* skip through file to atom_site_displace_Fourier */
    (void) position_cif(numblocks, impin, "_atom_site_displace_Fourier_", string, 0);
  /* first _atom_site_ line in string need to find position of label and parameters */
    j = 0;
    while (!strncmp (string, "_atom_site_displace_Fourier_", 28)) {
      if (!strncmp (string, "_atom_site_displace_Fourier_atom_site_label", 43))
        Label_pos = j;
      if (!strncmp (string, "_atom_site_displace_Fourier_wave_vector_seq_id", 46))
        Id_pos = j;
      if (!strncmp (string, "_atom_site_displace_Fourier_axis", 32))
        X_pos[0] = j;
      if (!strncmp (string, "_atom_site_displace_Fourier_param_cos", 37))
        X_pos[1] = j;
      if (!strncmp (string, "_atom_site_displace_Fourier_param_sin", 37))
        X_pos[2] = j;
      j++;
      if (!get_next_token (string, 100, impin)) {         /* search for next item */
        Error_Box("Error reading CIF Import File, Run aborted.");
        (void)fclose(impin); // we saw the header, so there should be data
        return;
      }
    }
    if (j >0) {
      if (!Quick)
          fprintf(drvui->flout, "******\n****** Atom Site Fourier Displacement Items\n****** \n"
                  "****** Atom  axis ID      cos      sin\n******\n");
      n_items = j;
      drvui->no_site_displace = 0;
      for (;;) {
        if (!strncmp (string, "_", 1) || strstr(string,"loop_") || !strlen(string) )
          break;
        for (items=0; items<=n_items-1; items++) {
          if (Label_pos == items) {
            int modnum = 0;
            char modl[5];
            strcpy(modl, "    ");    /* initialize name */
            j=0;
            for (i=0; i<(int)strlen(string); i++) {
              if (string[i] >= '0' && string[i] <= '9') {
                modnum = 10 * modnum
                         + (int) string[i] - 48;
              } else {
                modl[j++] = string[i];
                if (j > 3)
                  j = 3;
              }
            }
          for (j=0;j< natom;j++) 
            if (check_atom_name(modl,drvui->atom_l[j]) && modnum == drvui->atom_n[j] ) {
	      theatom=j;
              drvui->atom_ismod[j] |= 1;
              drvui->occ_ismod[j] = 0;
              break;
            }
          }
          if (X_pos[0] == items)  {
	    axis = 0;
	    if (!strncmp(string,"y",1)) 
              axis =1;
	    else if (!strncmp(string,"z",1))
              axis =2;
	  }
          drvui->atom_modpar_axis[drvui->no_site_displace] = axis;
          drvui->atom_modpar_atom[drvui->no_site_displace] = theatom;
          if (Id_pos == items)  
	    (void) sscanf(string,"%d",&drvui->atom_modpar_id[drvui->no_site_displace]);
          if (X_pos[1] == items)  
	    (void) sscanf(string,"%f",&drvui->atom_modpar[drvui->no_site_displace][0]);
          if (X_pos[2] == items)  
	    (void) sscanf(string,"%f",&drvui->atom_modpar[drvui->no_site_displace][1]);
        
          if (items <n_items) 
            if (!get_next_token(string, 100, impin))
              strcpy(string,"_loop");

        }
        if (!Quick) {
            char caxis[3][2] = {"X","Y","Z"};
            char axis2[2];
            strcpy(axis2,caxis[drvui->atom_modpar_axis[drvui->no_site_displace]]);
            fprintf(drvui->flout,"****** %4s%2d   %s%3d   %8.5f %8.5f\n", drvui->atom_l[theatom],
                    drvui->atom_n[theatom], axis2,
                    drvui->atom_modpar_id[drvui->no_site_displace],
                    drvui->atom_modpar[drvui->no_site_displace][0],
                    drvui->atom_modpar[drvui->no_site_displace][1]);
        }
        drvui->no_site_displace++;
      }
    }

  /* displacement modulation - sawtooth function */

  /* skip through file to atom_site_displace_special */
    (void) position_cif(numblocks, impin, "_atom_site_displace_special_", string, 0);
  /* first _atom_site_ line in string need to find position of label and parameters */
    j = 0;
    Label_pos=U_pos[0]=U_pos[1]=U_pos[2]=U_pos[3]=U_pos[4]=0;
    while (!strncmp (string, "_atom_site_displace_special_", 28)) {
      if (!strncmp (string, "_atom_site_displace_special_func_atom_site_label", 48))
        Label_pos = j;
      if (!strncmp (string, "_atom_site_displace_special_func_sawtooth_ax", 44))
        U_pos[0] = j;
      if (!strncmp (string, "_atom_site_displace_special_func_sawtooth_ay", 44))
        U_pos[1] = j;
      if (!strncmp (string, "_atom_site_displace_special_func_sawtooth_az", 44))
        U_pos[2] = j;
      if (!strncmp (string, "_atom_site_displace_special_func_sawtooth_c", 43))
        U_pos[3] = j;
      if (!strncmp (string, "_atom_site_displace_special_func_sawtooth_w", 43))
        U_pos[4] = j;
      j++;
      if (!get_next_token (string, 100, impin)) {         /* search for next item */
        Error_Box("Error reading CIF Import File, Run aborted.");
        (void)fclose(impin); // we saw the header, so there should be data
        return;
      }
    }
    if (j >0) {
      if (!Quick)
          fprintf(drvui->flout, "******\n****** Atom Site Sawtooth Displacement Items\n****** \n"
                  "****** Atom         ax       ay       az     center   width\n******\n");
      n_items = j;
      for (;;) {
        if (!strncmp (string, "_", 1) || strstr(string,"loop_") || !strlen(string) )
          break;
        for (items=0; items<=n_items-1; items++) {
          if (Label_pos == items) {
            int modnum = 0;
            char modl[5];
            strcpy(modl, "    ");    /* initialize name */
            j=0;
            for (i=0; i<(int)strlen(string); i++) {
              if (string[i] >= '0' && string[i] <= '9') {
                modnum = 10 * modnum
                         + (int) string[i] - 48;
              } else {
                modl[j++] = string[i];
                if (j > 3)
                  j = 3;
              }
            }
          for (j=0;j< natom;j++) 
            if (check_atom_name(modl,drvui->atom_l[j]) && modnum == drvui->atom_n[j] ) {
	      theatom=j;
              drvui->atom_ismod[j] |= 2;
              drvui->occ_ismod[j] = 0;
              break;
            }
          }
          if (U_pos[0] == items)  
	    (void) sscanf(string,"%f",&drvui->atom_mod_sawtooth[theatom][0]);
          if (U_pos[1] == items)  
	    (void) sscanf(string,"%f",&drvui->atom_mod_sawtooth[theatom][1]);
          if (U_pos[2] == items)  
	    (void) sscanf(string,"%f",&drvui->atom_mod_sawtooth[theatom][2]);
          if (U_pos[3] == items)  
	    (void) sscanf(string,"%f",&drvui->atom_mod_sawtooth[theatom][3]);
          if (U_pos[4] == items)  
	    (void) sscanf(string,"%f",&drvui->atom_mod_sawtooth[theatom][4]);
        
          if (items <n_items) 
            if (!get_next_token(string, 100, impin))
              strcpy(string,"_loop");

        }
        if (!Quick) {
            fprintf(drvui->flout,"****** %4s%2d   %8.5f %8.5f %8.5f %8.5f %8.5f\n", drvui->atom_l[theatom],
                    drvui->atom_n[theatom], 
                    drvui->atom_mod_sawtooth[theatom][0],
                    drvui->atom_mod_sawtooth[theatom][1],
                    drvui->atom_mod_sawtooth[theatom][2],
                    drvui->atom_mod_sawtooth[theatom][3],
                    drvui->atom_mod_sawtooth[theatom][4]);
        }
      }
    }

  /* occupancy modulation - fourier series */

  /* skip through file to atom_site_occ_Fourier */
    (void) position_cif(numblocks, impin, "_atom_site_occ_Fourier_", string, 0);
  /* first _atom_site_ line in string need to find position of label, x, y, and z */
    j = 0;
    while (!strncmp (string, "_atom_site_occ_Fourier_", 23)) {
      if (!strncmp (string, "_atom_site_occ_Fourier_atom_site_label", 38))
        Label_pos = j;
      if (!strncmp (string, "_atom_site_occ_Fourier_wave_vector_seq_id", 41))
        Id_pos = j;
      if (!strncmp (string, "_atom_site_occ_Fourier_param_cos", 32))
        X_pos[1] = j;
      if (!strncmp (string, "_atom_site_occ_Fourier_param_sin", 32))
        X_pos[2] = j;
      j++;
      if (!get_next_token (string, 100, impin)) {         /* search for next item */
        Error_Box("Error reading CIF Import File, Run aborted.");
        (void)fclose(impin);
        return;
      }
    }
    if (j>0) {
      if (!Quick)
          fprintf(drvui->flout, "******\n****** Atom Site Fourier Occupancy Items\n******\n"
                  "******  Atom  axis    cos      sin\n******\n");
      n_items = j;
      drvui->no_site_occ = 0;
      for (;;) {
        if (!strncmp (string, "_", 1) || strstr(string,"loop_") || !strlen(string) )
          break;
        for (items=0; items<=n_items-1; items++) {
          if (Label_pos == items) {
            int modnum = 0;
            char modl[5];
            strcpy(modl, "    ");    /* initialize name */
            j=0;
            for (i=0; i<(int)strlen(string); i++) {
              if (string[i] >= '0' && string[i] <= '9') {
                modnum = 10 * modnum
                         + (int) string[i] - 48;
              } else {
                modl[j++] = string[i];
                if (j > 3)
                  j = 3;
              }
            }
            for (j=0;j< natom;j++) 
              if (check_atom_name(modl,drvui->atom_l[j]) && modnum == drvui->atom_n[j] ) {
	        theatom=j;
                drvui->occ_ismod[j] |= 1;
                break;
              }
          }
          drvui->atom_occpar_atom[drvui->no_site_occ] = theatom;

          if (Id_pos == items)  
            (void) sscanf(string,"%d",&drvui->atom_occpar_id[drvui->no_site_occ]);

          if (X_pos[1] == items)  
            (void) sscanf(string,"%f",&drvui->atom_occpar[drvui->no_site_occ][0]);

          if (X_pos[2] == items)  
            (void) sscanf(string,"%f",&drvui->atom_occpar[drvui->no_site_occ][1]);

          if (items <n_items) 
            if (!get_next_token(string, 100, impin))strcpy(string,"_loop");

        }
        if (!Quick) {
            fprintf(drvui->flout,"****** %4s%d %5d %8.5f %8.5f\n", drvui->atom_l[theatom],
                    drvui->atom_n[theatom], drvui->atom_occpar_id[drvui->no_site_occ],
                    drvui->atom_occpar[drvui->no_site_occ][0],
                    drvui->atom_occpar[drvui->no_site_occ][1]);
        }
        drvui->no_site_occ++;
      }
    }

  /* occupancy modulation : crenel function */

  /* skip through file to atom_site_occ_special */
    (void) position_cif(numblocks, impin, "_atom_site_occ_special_", string, 0);
  /* first _atom_site_ line in string need to find position of label, x, y, and z */
    j = 0;
    while (!strncmp (string, "_atom_site_occ_special_", 23)) {
      if (!strncmp (string, "_atom_site_occ_special_func_atom_site_label", 43))
        Label_pos = j;
      if (!strncmp (string, "_atom_site_occ_special_func_crenel_c", 36))
        X_pos[1] = j;
      if (!strncmp (string, "_atom_site_occ_special_func_crenel_w", 36))
        X_pos[2] = j;
      j++;
      if (!get_next_token (string, 100, impin)) {         /* search for next item */
        Error_Box("Error reading CIF Import File, Run aborted.");
        (void)fclose(impin);
        return;
      }
    }
    if (j >0 ) {
      n_items = j;
      for (;;) {
        if (!strncmp (string, "_", 1) || !strncmp(string,"loop_",5) || !strlen(string) )
          break;
        for (items=0; items<=n_items-1; items++) {
          if (Label_pos == items) {
            int modnum = 0;
            char modl[5];
            strcpy(modl, "    ");    /* initialize name */
            j=0;
            for (i=0; i<(int)strlen(string); i++) {
              if (string[i] >= '0' && string[i] <= '9') {
                modnum = 10 * modnum
                         + (int) string[i] - 48;
              } else {
                modl[j++] = string[i];
                if (j > 3)
                  j = 3;
              }
            }
            for (j=0;j< natom;j++) 
              if (check_atom_name(modl,drvui->atom_l[j]) && modnum == drvui->atom_n[j] ) {
	        theatom=j;
                drvui->occ_ismod[j] |= 2;
                break;
              }
            }

            if (X_pos[1] == items)  
              (void) sscanf(string,"%f",&drvui->atom_occ_crenel[theatom][0]);
            if (X_pos[2] == items)  
	      (void) sscanf(string,"%f",&drvui->atom_occ_crenel[theatom][1]);
            if (items <n_items) 
              if (!get_next_token(string, 100, impin))strcpy(string,"_loop");

        }
      }
      for (j=0;j<natom;j++) 
        if (drvui->occ_ismod[j] == 2) { // convert crenel parameters to min/max values 
          if (!Quick) {
            fprintf(drvui->flout,"*** Occupational modulation parameters for %s%d:\n",
                    drvui->atom_l[j],drvui->atom_n[j]);
            fprintf(drvui->flout,"*** \t crenel offset %7.5f width %7.5f\n",
                    drvui->atom_occ_crenel[j][0],drvui->atom_occ_crenel[j][1]);
          }
          float w=drvui->atom_occ_crenel[j][1]/2.0f;
          drvui->atom_occ_crenel[j][1] = drvui->atom_occ_crenel[j][0] +w;
          drvui->atom_occ_crenel[j][0] -= w;
      }
    }
  
  /* modulation of thermal parameters - fourier series */

  /* skip through file to atom_site_U_Fourier */
    (void) position_cif(numblocks, impin, "_atom_site_U_Fourier_", string, 0);
  /* first _atom_site_ line in string need to find position of label and parameters */
    j = 0;
    while (!strncmp (string, "_atom_site_U_Fourier_", 21)) {
      if (!strncmp (string, "_atom_site_U_Fourier_atom_site_label", 36))
        Label_pos = j;
      if (!strncmp (string, "_atom_site_U_Fourier_tens_elem", 30))
        X_pos[0] = j;
      if (!strncmp (string, "_atom_site_U_Fourier_wave_vector_seq_id", 39))
        Id_pos = j;
      if (!strncmp (string, "_atom_site_U_Fourier_param_cos", 30))
        X_pos[1] = j;
      if (!strncmp (string, "_atom_site_U_Fourier_param_sin", 30))
        X_pos[2] = j;
      j++;
      if (!get_next_token (string, 100, impin)) {         /* search for next item */
        Error_Box("Error reading CIF Import File, Run aborted.");
        (void)fclose(impin); // we saw the header, so there should be data
        return;
      }
    }
    if (j >0) {
      n_items = j;
      drvui->no_site_U_terms = 0;
      for (;;) {
        if (!strncmp (string, "_", 1) || strstr(string,"loop_") || !strlen(string) )
          break;
        for (items=0; items<=n_items-1; items++) {
          if (Label_pos == items) {
            int modnum = 0;
            char modl[5];
            strcpy(modl, "    ");    /* initialize name */
            j=0;
            for (i=0; i<(int)strlen(string); i++) {
              if (string[i] >= '0' && string[i] <= '9') {
                modnum = 10 * modnum
                         + (int) string[i] - 48;
              } else {
                modl[j++] = string[i];
                if (j > 3)
                  j = 3;
              }
            }
            for (j=1;j< drvui->n_ellips;j++) 
              if (check_atom_name(modl,drvui->ellips_l[j]) && modnum == drvui->ellips_n[j] ) {
              drvui->ellips_ismod[j]=1;
              break;
            }

            for (j=0;j< natom;j++) 
              if (check_atom_name(modl,drvui->atom_l[j]) && modnum == drvui->atom_n[j] ) {
	        theatom=j;
              break;
            }
            drvui->ellips_modpar_atom[drvui->no_site_U_terms] = theatom;
          }
          if (X_pos[0] == items)  {
            elem = 0; // U11
            if (!strncmp(string,"U22",3)) 
              elem =1;
	    else if (!strncmp(string,"U33",3))
              elem =2;
	    else if (!strncmp(string,"U12",3)) 
              elem =3;
	    else if (!strncmp(string,"U13",3)) 
              elem =4;
	    else if (!strncmp(string,"U23",3))
              elem =5;
            drvui->ellips_modpar_term[drvui->no_site_U_terms] = elem;
	  }
          if (Id_pos == items)  
	    (void) sscanf(string,"%d",&drvui->ellips_modpar_id[drvui->no_site_U_terms]);
          if (X_pos[1] == items)  
	    (void) sscanf(string,"%f",&drvui->ellips_modpar[drvui->no_site_U_terms][0]);
          if (X_pos[2] == items)  
	    (void) sscanf(string,"%f",&drvui->ellips_modpar[drvui->no_site_U_terms][1]);
        
          if (items <n_items) 
            if (!get_next_token(string, 100, impin))
              strcpy(string,"_loop");

        }
        drvui->no_site_U_terms++;
      }
      if (!Quick) {
        char Terms[6][4] = {"U11","U22","U33","U12","U13","U23"};
        char out_term[4];
        int id, term;
        fprintf(drvui->flout, "******\n****** Uij Fourier Modulation terms\n******\n"
                "****** Atom    ID  Term      cos      sin\n******\n");
        for (j=0;j<drvui->no_site_U_terms;j++) {
            theatom = drvui->ellips_modpar_atom[j];
            id = drvui->ellips_modpar_id[j];
            term = drvui->ellips_modpar_term[j];
            strcpy(out_term, Terms[term]);
            fprintf(drvui->flout,"****** %4s%d   %2d   %3s %10.5f%9.5f\n",drvui->atom_l[theatom],
                                 drvui->atom_n[theatom], id, out_term, drvui->ellips_modpar[j][0],
                                 drvui->ellips_modpar[j][1]);
        }
        fprintf(drvui->flout, "******\n");
      }
    }
  }

/* get subsystem information for composite crystals */

  make_bmat (drvui->sys, drvui->lat_con, drvui->b_mat,
             drvui->ginv, drvui->rec_lat_con);  /* create the lattice metric */
  if (!position_cif(numblocks, impin, "_cell_subsystems_number", string, 0)) { /* no error if not found */
      float e[6] = {0,0,0,0,0,0}; /* the (3 + d)D reciprocal basis vectors */
      float mat[6][6];          /* the matrix describing the relations between e and ep */
      int kk;
      float ast = drvui->rec_lat_con[0];
      float bst = drvui->rec_lat_con[1];
      float cst = drvui->rec_lat_con[2];
      float csal = drvui->rec_lat_con[3];
      float csbe = drvui->rec_lat_con[4];
      float csga = drvui->rec_lat_con[5];
      float snalp, snbep, sngap, vol;
      float ap, bp, cp, csalp, csbep, csgap;
      float a, b, c, alpha, beta, gamma;
      int l, m;

      get_next_token(string, 100, impin);   /* this CIF describes a composite crystal */
      sscanf(string, "%d", &drvui->no_subsys);
      if (!Quick)
          fprintf(drvui->flout, "******\n****** CIF describes a composite crystal "
                  "with %d subsystems\n******\n", drvui->no_subsys);
      fgets(string, 100, impin);
      for (;;) {
          for (j=0; j<6; j++)
              for (kk=0; kk<6; kk++)
                  mat[j][kk] = 0.0f;
          fgets(string, 100, impin);
          if (!strstr(string, "_cell_subsystem"))
              break;                    /* skip through all the W matrix lines - FIXME? */
      }
      for (j=0; j<3; j++)
          e[j] = drvui->rec_lat_con[j];       /* first three elements of e are a*, b* and c* */
      for (j=0; j<drvui->modulated; j++)      /* remainder are e4*, e5*, and e6* */
          for (kk=0; kk<3; kk++)
              e[j+3] += drvui->cell_vec[j][kk] * drvui->rec_lat_con[kk];
      for (j=0; j<drvui->no_subsys; j++) {
          for (i=0; i<3+drvui->modulated; i++) {
              for (kk=0; kk<3+drvui->modulated; kk++) {
                  get_next_token(string, 100, impin);
                  sscanf(string, "%f", &mat[i][kk]);   /* read the appropriate matrix element */
              }
          }
          fgets(string, 100, impin);    /* skip the line with ? */
          for (l=0; l<3; l++)
              for (m=0; m<3; m++)
                  drvui->subsys_fact[j][l][m] = mat[l][m] + mat[l][3] * drvui->cell_vec[0][m]
                     + mat[l][4] * drvui->cell_vec[1][m] + mat[l][5] * drvui->cell_vec[2][m];
/* this is really ugly code, but I don't know how to make it pretty */
          ap = (float) sqrt(ast * ast * drvui->subsys_fact[j][0][0] * drvui->subsys_fact[j][0][0]
             + bst * bst * drvui->subsys_fact[j][0][1] * drvui->subsys_fact[j][0][1]
             + cst * cst * drvui->subsys_fact[j][0][2] * drvui->subsys_fact[j][0][2]
             + 2.0f * ast * bst * csga * drvui->subsys_fact[j][0][0] * drvui->subsys_fact[j][0][1]
             + 2.0f * ast * cst * csbe * drvui->subsys_fact[j][0][0] * drvui->subsys_fact[j][0][2]
             + 2.0f * bst * cst * csal * drvui->subsys_fact[j][0][1] * drvui->subsys_fact[j][0][2]);
          bp = (float) sqrt(ast * ast * drvui->subsys_fact[j][1][0] * drvui->subsys_fact[j][1][0]
             + bst * bst * drvui->subsys_fact[j][1][1] * drvui->subsys_fact[j][1][1]
             + cst * cst * drvui->subsys_fact[j][1][2] * drvui->subsys_fact[j][1][2]
             + 2.0f * ast * bst * csga * drvui->subsys_fact[j][1][0] * drvui->subsys_fact[j][1][1]
             + 2.0f * ast * cst * csbe * drvui->subsys_fact[j][1][0] * drvui->subsys_fact[j][1][2]
             + 2.0f * bst * cst * csal * drvui->subsys_fact[j][1][1] * drvui->subsys_fact[j][1][2]);
          cp = (float) sqrt(ast * ast * drvui->subsys_fact[j][2][0] * drvui->subsys_fact[j][2][0]
             + bst * bst * drvui->subsys_fact[j][2][1] * drvui->subsys_fact[j][2][1]
             + cst * cst * drvui->subsys_fact[j][2][2] * drvui->subsys_fact[j][2][2]
             + 2.0f * ast * bst * csga * drvui->subsys_fact[j][2][0] * drvui->subsys_fact[j][2][1]
             + 2.0f * ast * cst * csbe * drvui->subsys_fact[j][2][0] * drvui->subsys_fact[j][2][2]
             + 2.0f * bst * cst * csal * drvui->subsys_fact[j][2][1] * drvui->subsys_fact[j][2][2]);
          csgap = (drvui->subsys_fact[j][0][0] * drvui->subsys_fact[j][1][0] * ap * ap
                + drvui->subsys_fact[j][0][1] * drvui->subsys_fact[j][1][1] * bp * bp
                + drvui->subsys_fact[j][0][2] * drvui->subsys_fact[j][1][2] * cp * cp
                + ast * bst * csga * (drvui->subsys_fact[j][0][0] * drvui->subsys_fact[j][1][1]
                +                     drvui->subsys_fact[j][0][1] * drvui->subsys_fact[j][1][0])
                + ast * cst * csbe * (drvui->subsys_fact[j][0][0] * drvui->subsys_fact[j][1][2]
                +                     drvui->subsys_fact[j][0][2] * drvui->subsys_fact[j][1][0])
                + bst * cst * csal * (drvui->subsys_fact[j][0][1] * drvui->subsys_fact[j][1][2]
                +                     drvui->subsys_fact[j][0][2] * drvui->subsys_fact[j][1][1]))
                /(ap * bp);
          sngap = (float) sqrt(1.0 - csgap * csgap);
          csbep = (drvui->subsys_fact[j][0][0] * drvui->subsys_fact[j][2][0] * ap * ap
                + drvui->subsys_fact[j][0][1] * drvui->subsys_fact[j][2][1] * bp * bp
                + drvui->subsys_fact[j][0][2] * drvui->subsys_fact[j][2][2] * cp * cp
                + ast * bst * csga * (drvui->subsys_fact[j][0][0] * drvui->subsys_fact[j][2][1]
                +                     drvui->subsys_fact[j][0][1] * drvui->subsys_fact[j][2][0])
                + ast * cst * csbe * (drvui->subsys_fact[j][0][0] * drvui->subsys_fact[j][2][2]
                +                     drvui->subsys_fact[j][0][2] * drvui->subsys_fact[j][2][0])
                + bst * cst * csal * (drvui->subsys_fact[j][0][1] * drvui->subsys_fact[j][2][2]
                +                     drvui->subsys_fact[j][0][2] * drvui->subsys_fact[j][2][1]))
                /(ap * cp);
          snbep = (float) sqrt(1.0 - csbep * csbep);
          csalp = (drvui->subsys_fact[j][1][0] * drvui->subsys_fact[j][2][0] * ap * ap
                + drvui->subsys_fact[j][1][1] * drvui->subsys_fact[j][2][1] * bp * bp
                + drvui->subsys_fact[j][1][2] * drvui->subsys_fact[j][2][2] * cp * cp
                + ast * bst * csga * (drvui->subsys_fact[j][1][0] * drvui->subsys_fact[j][2][1]
                +                     drvui->subsys_fact[j][1][1] * drvui->subsys_fact[j][2][0])
                + ast * cst * csbe * (drvui->subsys_fact[j][1][0] * drvui->subsys_fact[j][2][2]
                +                     drvui->subsys_fact[j][1][2] * drvui->subsys_fact[j][2][0])
                + bst * cst * csal * (drvui->subsys_fact[j][1][1] * drvui->subsys_fact[j][2][2]
                +                     drvui->subsys_fact[j][1][2] * drvui->subsys_fact[j][2][1]))
                /(bp * cp);
          snalp = (float) sqrt(1.0 - csalp * csalp);
          vol = ap * bp * cp * (float) sqrt(1.0 - csalp * csalp - csbep * csbep - csgap * csgap
              - csalp * csbep * csgap);
          a = bp * cp * snalp / vol;
          b = ap * cp * snbep / vol;
          c = ap * bp * sngap / vol;
          alpha = (float)(180.0 / PI * acos((csbep * csgap - csalp) / (snbep * sngap)));
          beta  = (float)(180.0 / PI * acos((csalp * csgap - csbep) / (snalp * sngap)));
          gamma = (float)(180.0 / PI * acos((csalp * csbep - csgap) / (snalp * snbep)));
          drvui->subsys_vol[j] = vol;     /* save reciprocal volume */
              
          if (!Quick)
              fprintf(drvui->flout, "****** The cell for subsystem %d is:\n"
                   "******  a = %8.5f, b = %8.5f, c = %8.5f, Vol = %9.3f\n"
                   "******  alpha = %8.3f, beta = %8.3f,"
                   " gamma = %8.3f\n******\n", j+1 , a, b, c, 1.0f/vol, alpha, beta, gamma);
      }
  }

  (void)fclose (impin);
}

/* ************************************************************** */
/* ************************************************************** */

void import_fdat (char input[100], int in_line, int Quick, int frame_no)

/* routine to extract structural information from a CSD 'FDAT' data file */
{
  FILE *impin;
  char string[200];                 /* temporary string */
  char filename[256];                 /* string for filename */
  int i, j;                     /* temporary */
  int k, l, k3, k4;                 /*flags for space group symbol parsing */
  char spstring[100];                 /* space group symbol */
  char lat_s[6][7];                 /*temporary storage for lattice constants */
  int ldec[6];                     /*temporary storage for decimal places of lattice constants */
  char number[4];
  char ncons[4], nrfacs[4], nrems[4], ndiss[4], nerrs[4], noprs[4],
    nrads[4], nats[4];
  char sats[4], nbnds[4];
  int ncards, nrfac = 0, nrem = 0, ndis = 0, nerr = 0, nopr = 0, nrad =
    0, nat = 0, sat = 0, ncon = 0, cellflag;
  int intflag, coordflag, centflag, errflag = 0;
  int nskip;                     /* lines to skip between lattice constants and atoms */
  int natomin;

  memset (spstring, 0, 100);
  memset (string, 0, 100);
  memset (nrfacs, 0, 4);
  memset (nrems, 0, 4);
  memset (ndiss, 0, 4);
  memset (ncons, 0, 4);
  memset (number, 0, 4);
  memset (nerrs, 0, 4);
  memset (noprs, 0, 4);
  memset (nrads, 0, 4);
  memset (nats, 0, 4);
  memset (sats, 0, 4);
  memset (nbnds, 0, 4);
  for (i = 0; i < 6; i++)
    memset (lat_s[i], 0, 7);
  if (in_line) {
    impin = drvui->fpin;
  } else {
    (void)sscanf (input, "%s %s", filename, filename); /* get file name */
    if (!(impin = fopen (filename, "r"))) {
        Error_Box("Cannot Open CSD/FDAT Import File, Run aborted.");
        return;
    }
  }

  if(!Quick)fprintf (drvui->flout, "****** Now interpreting CSD-style data\n");

  if (!fgets (input, 100, impin)) {         /* read a line */
      Error_Box("Error reading CSD/FDAT file.");
      return;
  }

/* the first line should contain the REFCODE of the database entry */
/* and information about the quality of the data and the length of */
/* the following sections                                          */

  if (strncmp (input, "#", 1) == 0) {

    (void)sscanf (input, "%23c%3d%3c%3c%3c%3c%3c%3c%3c%3c%3c%3c%1d%1d%1d%1d%1d",
        string, &ncards, nrfacs, nrems, ndiss, nerrs, noprs, nrads,
        nats, sats, nbnds, ncons, &cellflag, &intflag, &coordflag,
        &centflag, &errflag);

    nrfac = atoi (nrfacs);
    ncon = atoi (ncons);
    nrem = atoi (nrems);
    ndis = atoi (ndiss);
    nerr = atoi (nerrs);
    nopr = atoi (noprs);
    nrad = atoi (nrads);
    nat = atoi (nats);
    sat = atoi (sats);
    input[0] = ' ';
    input[9] = '\0';                 /* truncate title */
    if (!Quick) fprintf (drvui->flout, "******      %s\n", input);
    if (!cellflag) {
        Error_Box("Error - this CSD entry has neither CELL nor COORDINATES");
        return;
    }

    if (!coordflag) {
        Error_Box("Error - this CSD entry has no COORDINATES");
        return;
    }

    if (errflag) {
        Error_Box("Warning - this CSD entry has the ERROR flag set -\n at least some coordinates may be wrong");
    }

    if (!fgets (input, 100, impin)) {         /* read a line */
        Error_Box("Error reading CSD file.");
        return;
    }
  }

/* lattice constants come in a fixed FORTRAN format followed by integers */
/* encoding precision, esd's, etc and finally by the spacegroup symbol */

  for (i = 0; i <= 5; ++i)
    drvui->lat_con[i] = 0.0f;

  (void)sscanf (input, "%6c%6c%6c%6c%6c%6c%1d%1d%1d%1d%1d%1d%21c%s", lat_s[0],
      lat_s[1], lat_s[2], lat_s[3], lat_s[4], lat_s[5], &ldec[0],
      &ldec[1], &ldec[2], &ldec[3], &ldec[4], &ldec[5], string, string);

  for (i = 0; i < 6; ++i) {
    drvui->lat_con[i] = (float) (atof (lat_s[i]) / pow (10, (double)ldec[i]));
  }

/* separate class symbol from other symmetry elements */
  spstring[0] = 's';
  spstring[1] = 'p';
  spstring[2] = 'g';
  spstring[3] = 'r';
  spstring[4] = ' ';
  spstring[5] = string[0];
  spstring[6] = ' ';
  j = 6;
  k = 0;
  k3 = 0;
  k4 = 0;
  l = (int) strlen (string);
  for (i = 1; i <= l; i++) {
    switch (string[i]) {
    case ' ':
      break;
    case '/':
      spstring[++j] = string[i];
      break;
    default:
      j++;
      /* do we have to collate this symbol (for screw axes or mirror planes) ? */
      if (string[i] > '0' && string[i] <= '9') { /* it is a number */
    if (string[0] == 'R' ||             /* no screw axes in R */
        l == 4 ||                 /* no screw axes in four letter names */
        k != 0 ||                 /* have just appended to a number */
        k3 != 0 ||                 /* cannot have further screw axes after 3_x */
        (k4 != 0 && spstring[j - 1] == '3')) {    /* cannot have 3_x after a 4_x */
      spstring[j] = ' ';             /* so separate it */
      k = 0;                 /* and reset the collation flag */
      j++;
    }
    else if (spstring[j - 1] != '-' && (spstring[j - 1] <= string[i]
                        || spstring[j - 1] >= 'a')) {
      /* neither a minus sign nor a higher number precedes it */
      spstring[j] = ' ';             /* so separate it */
      k = 0;                 /* and reset the collation flag */
      j++;
    }
    else {
      k++;                     /* set flag for collation */
      if (spstring[j - 1] == '3')
        k3 = 1;                 /* if 3_x screw, prohibit further screw axes */
      if (spstring[j - 1] == '4')
        k4 = 1;                 /* if 4_x screw, prohibit 3_x screw axes */
    }
      }
      else {                     /* not a number */
    if (spstring[j - 1] != '/') {         /*but not a mirror plane either */
      spstring[j] = ' ';             /* separate it */
      j++;
    }
      }
      spstring[j] = string[i];             /* now add the symbol */
      break;
    }                         /*switch */
  }                         /*for i */
  spstring[j + 1] = '\0';

/* Space group 90, P 4 21 2, is not correctly identified by this code. 
   It accounts for only 14 of the 175000 entries in the CSD (and not all 
   of them have coordinates), so this quick and dirty fix is all it can
   expect from us. Does this spacegroup really exist ? :-) */
  if (!strcmp (string, "P4212"))
    strcpy (spstring, "P 4 21 2");

  if(!Quick)fprintf (drvui->flout, "******      Space group *%s*\n", spstring);
  symop (spstring);

  if(!Quick)fprintf (drvui->flout,
       "******      Cell     %8.5f %8.5f %8.5f %8.3f %8.3f %8.3f\n",
       drvui->lat_con[0], drvui->lat_con[1], drvui->lat_con[2], drvui->lat_con[3], drvui->lat_con[4],
       drvui->lat_con[5]);

/*skip R value, any textual information, symmetry codes and radii data */

  nskip = (nrfac + nrem + ndis + nerr) / 80 + 1; /* skip comments (80chars per line) */
  nskip += nopr / 5 + 1;             /* skip symmetry operators (five per line) */
  nskip += nrad / 16 + 1;             /* skip atomic radii (sixteen per line) */


  for (i = 0; i < nskip; i++) {
    if (!fgets (input, 100, impin)) {         /* read a line */
        Error_Box("Error reading file while skipping records.");
        if (!Quick) {
          fprintf (drvui->flout,
           "nrfac:%d, nrem:%d, ndis:%d,nerr:%d, nopr:%d, nrad:%d, nskip:%d\n",
           nrfac, nrem, ndis, nerr, nopr, nrad, nskip);
          return;
        }
    }
  }

  natomin = natom;                 /* number of atoms already in list */

/* read coordinates until empty atom symbol encountered */

  for (i = 0; i < (nat + sat) / 3 + 1; i++) {     /*for all expected atom records */

    if (!fgets (input, 100, impin)) {         /* read a line */
        Error_Box("Error reading CSD file while reading atom coordinates.");
        return;
    }

/* atoms come in groups of three */
    (void)sscanf (input,
        "%c%c%c%c %7f %7f %7f %c%c%c%c %7f %7f %7f %c%c%c%c %7f %7f %7f",
        &drvui->atom_l[natom][0], &drvui->atom_l[natom][1], &drvui->atom_l[natom][2],
        &drvui->atom_l[natom][3], &drvui->atom_xyz[natom][0], &drvui->atom_xyz[natom][1],
        &drvui->atom_xyz[natom][2], &drvui->atom_l[natom + 1][0],
        &drvui->atom_l[natom + 1][1], &drvui->atom_l[natom + 1][2],
        &drvui->atom_l[natom + 1][3], &drvui->atom_xyz[natom + 1][0],
        &drvui->atom_xyz[natom + 1][1], &drvui->atom_xyz[natom + 1][2],
        &drvui->atom_l[natom + 2][0], &drvui->atom_l[natom + 2][1],
        &drvui->atom_l[natom + 2][2], &drvui->atom_l[natom + 2][3],
        &drvui->atom_xyz[natom + 2][0], &drvui->atom_xyz[natom + 2][1],
        &drvui->atom_xyz[natom + 2][2]);

/* and the atom list is terminated by numerical data */
    if (drvui->atom_l[natom][0] < 'A') {
      break;
    }

/* scale coordinates */
    for (j = 0; j < 3; j++) {
      drvui->atom_xyz[natom][j] = drvui->atom_xyz[natom][j] / 100000.0f;
      drvui->atom_xyz[natom + 1][j] = drvui->atom_xyz[natom + 1][j] / 100000.0f;
      drvui->atom_xyz[natom + 2][j] = drvui->atom_xyz[natom + 2][j] / 100000.0f;
    }

/* separate numbers from labels and remove trailing identifiers */

    for (l = 0; l < 3; l++) {
      k = 0;
      for (j = 1; j < 4; j++) {
    number[k] = ' ';
    if (drvui->atom_l[natom + l][j] >= '0' && drvui->atom_l[natom + l][j] <= '9') {
      number[k] = drvui->atom_l[natom + l][j];
      k++;
    }
    if (k)
      drvui->atom_l[natom + l][j] = ' ';
      }
      number[k] = '\0';
      drvui->atom_n[natom + l] = atoi (number);
    }

    if (Unique_Atom () == 1) {
/* report converted data of first atom */
      if (!Quick)fprintf (drvui->flout,
           "******      Atom %c%c%c%c%3d %8.5f %8.5f %8.5f\n",
           drvui->atom_l[natom][0], drvui->atom_l[natom][1], drvui->atom_l[natom][2],
           drvui->atom_l[natom][3]
           , drvui->atom_n[natom], drvui->atom_xyz[natom][0], drvui->atom_xyz[natom][1],
           drvui->atom_xyz[natom][2]);
      drvui->atom_fn[natom] = frame_no;
      drvui->sv_atom_n[natom] = drvui->atom_n[natom];
      natom++;
      if (natom >= MAX_ATOM) {             /* first unique */
          Error_Box("Too many atoms in asymmetric unit - increase MAX_ATOM.");
          return;
      }
    }                         /* first unique */
/* return now if no second atom on the line */
    if (natom == nat + natomin) {
      break;                     /* got all expected atoms */
    }
    if (drvui->atom_l[natom][0] == '\0')
      break;
    if (Unique_Atom () == 1) {
/* report converted data of second atom */
      if (!Quick)fprintf (drvui->flout,
           "******      Atom %c%c%c%c%3d %8.5f %8.5f %8.5f\n",
           drvui->atom_l[natom][0], drvui->atom_l[natom][1], drvui->atom_l[natom][2],
           drvui->atom_l[natom][3]
           , drvui->atom_n[natom], drvui->atom_xyz[natom][0], drvui->atom_xyz[natom][1],
           drvui->atom_xyz[natom][2]);
      drvui->atom_fn[natom] = frame_no;
      drvui->sv_atom_n[natom] = drvui->atom_n[natom];
      natom++;
      if (natom >= MAX_ATOM) {             /* second unique */
          Error_Box("Too many atoms in asymmetric unit - increase MAX_ATOM.");
          return;
      }
    }                         /* second unique */
/* return now if no third atom on the line */
    if (natom == nat + natomin) {
      break;                     /* got all expected atoms */
    }
    if (drvui->atom_l[natom][0] == '\0')
      break;
    if (Unique_Atom () == 1) {             /* third unique */
/* report converted data of third atom */
      if (!Quick)fprintf (drvui->flout,
           "******      Atom %c%c%c%c%3d %8.5f %8.5f %8.5f\n",
           drvui->atom_l[natom][0], drvui->atom_l[natom][1], drvui->atom_l[natom][2],
           drvui->atom_l[natom][3]
           , drvui->atom_n[natom], drvui->atom_xyz[natom][0], drvui->atom_xyz[natom][1],
           drvui->atom_xyz[natom][2]);
      drvui->atom_fn[natom] = frame_no;
      drvui->sv_atom_n[natom] = drvui->atom_n[natom];
      natom++;
      if (natom >= MAX_ATOM) {
          Error_Box("Too many atoms in asymmetric unit - increase MAX_ATOM.");
          return;
      }
      if (natom == nat + natomin) {
    break;                     /* got all expected atoms */
      }
    }                         /* third unique */
  }                         /*for all expected atom records */
  for (i++; i < (nat + sat) / 3; i++) {         /* skip remainder of atom records */

    if (!fgets (input, 100, impin)) {         /* read a line */
        Error_Box("Error reading file while skipping extra atom records.");
        return;
    }
  }
  if (!errflag) {                 /* only error-free entries have connectivity records */
    nskip = ncon / 40 + 1;
    if (nat + sat > 100)
      nskip = ncon / 26 + 1;
    for (i = 0; i < nskip; i++) {
      if (!fgets (input, 100, impin)) {         /* read a line */
    if (!in_line)
      (void)fclose (impin);
    return;
      }
    }
  }
  if (!in_line)
    (void)fclose (impin);

}                         /* end of import_fdat */

/* ************************************************************** */
/* ************************************************************** */

void import_gsas (char input[100], int in_line, int Quick, int frame_no)

/* routine to extract structural information from a GSAS exp file */
{
  char string[256], filename[256], temp[5], tstring[20];
  int phase_no, i, j;
  int ii,jj;
  FILE *impin;

  if (in_line) {
      Error_Box("Cannot read inline GSAS data. Please use the import instruction");
      return;
  }

  ii=strlen(input)-1;
  if (isdigit(input[ii])) { // if number at end 
    for (jj=ii;jj>ii-6;jj--) {
      if (isspace(input[jj])) { // separate number
        input[jj]='\0';
        phase_no=atoi(&input[jj+1]);
        break;
      }
      if (isalpha(input[jj]))
        break; // unless it is attached to filename
      }
   }
//  (void)sscanf (input, "%s %s %d", filename, filename, &phase_no);    /* get file name and phase number */
  strcpy(filename,input);
  if ((phase_no < 1) || (phase_no > 9)) {
      Error_Box("Illegal phase number in GSAS import instruction, Run aborted.");
      return;
  }
  if (!(impin = fopen (filename, "r"))) {
      Error_Box("Cannot Open GSAS Import File, Run aborted.");
      return;
  }
  sprintf (temp, "CRS%d", phase_no);         /* make search object */
  for (;;) {
    if (!fgets (string, 100, impin)) {         /* search for phase info */
        Error_Box("Error reading phase info from GSAS Import File, Run aborted.");
        return;
    }
    if (strncmp (string, temp, 4) == 0)
      break;
  }
  for (;;) {                     /* search for lattice constants */
    if (!fgets (string, 100, impin)) {
        Error_Box("Error reading lattice constants GSAS Import File, Run aborted.");
        return;
    }
    (void)sscanf (string, "%s %s", tstring, tstring);
    if (strncmp (tstring, "ABC", 3) == 0) {
      (void)sscanf (string, "%s %s %f %f %f", tstring, tstring, &drvui->lat_con[0],
          &drvui->lat_con[1]
          , &drvui->lat_con[2]);             /* read lattice constants */
      break;
    }
  }                         /* search for lattice constants */
  for (;;) {                     /* search for angles */
    if (!fgets (string, 100, impin)) {
        Error_Box("Error reading lattice angles GSAS Import File, Run aborted.");
        return;
    }
    (void)sscanf (string, "%s %s", tstring, tstring);
    if (strncmp (tstring, "ANGLES", 6) == 0) {
      (void)sscanf (string, "%s %s %f %f %f", tstring, tstring, &drvui->lat_con[3],
          &drvui->lat_con[4]
          , &drvui->lat_con[5]);             /* read lattice constants */
      break;
    }
  }                         /* search for angles */
  if (!Quick)
    fprintf (drvui->flout,
         "******      Cell     %8.5f %8.5f %8.5f %8.3f %8.3f %8.3f\n",
         drvui->lat_con[0], drvui->lat_con[1], drvui->lat_con[2], drvui->lat_con[3], drvui->lat_con[4],
         drvui->lat_con[5]);
  for (;;) {                     /* position to atomic coordinates */
    if (!fgets (string, 100, impin)) {
        Error_Box("Error reading atomic coordinates from GSAS Import File, Run aborted.");
        return;
    }
    (void)sscanf (string, "%s %s", tstring, tstring);
    if (strncmp (tstring, "AT", 2) == 0)
      break;
  }                         /* position to atomic coordinates */

/* at this point, positioned at first atom line */
  for (;;) {                     /* process atom parameters */
    (void)sscanf (string, "%s %s", tstring, tstring);
    if (strncmp (tstring, "AT", 2) != 0)
      break;                     /* out when atom stuff done */
    (void)sscanf (string, "%s %s %s %s %f %f %f", tstring, tstring, tstring,
        tstring, &drvui->atom_xyz[natom][0], &drvui->atom_xyz[natom][1],
        &drvui->atom_xyz[natom][2]);
    drvui->atom_n[natom] = 0;
    for (i = 0; i < 4; i++)
      drvui->atom_l[natom][i] = ' ';             /* initialize atom name */
    j = 0;
    if (string[62] >= '0' && string[62] <= '9')
      drvui->atom_n[natom] = (int) string[62] - 48;
    else
      drvui->atom_l[natom][j++] = string[62];
    if (string[63] >= '0' && string[63] <= '9')
      drvui->atom_n[natom] = 10 * drvui->atom_n[natom] + (int) string[63] - 48;
    else
      drvui->atom_l[natom][j++] = string[63];
    if (string[64] >= '0' && string[64] <= '9')
      drvui->atom_n[natom] = 10 * drvui->atom_n[natom] + (int) string[64] - 48;
    else
      drvui->atom_l[natom][j++] = string[64];
    if (string[65] >= '0' && string[65] <= '9')
      drvui->atom_n[natom] = 10 * drvui->atom_n[natom] + (int) string[65] - 48;
    else
      drvui->atom_l[natom][j++] = string[65];
    if (!Quick)
      fprintf (drvui->flout,
           "******      Atom %c%c%c%c%3d %8.5f %8.5f %8.5f\n",
           drvui->atom_l[natom][0]
           , drvui->atom_l[natom][1], drvui->atom_l[natom][2], drvui->atom_l[natom][3],
           drvui->atom_n[natom]
           , drvui->atom_xyz[natom][0], drvui->atom_xyz[natom][1], drvui->atom_xyz[natom][2]);
    (void)fgets (string, 100, impin);
    if (string[74] == 'A') {
      drvui->ell_type[drvui->n_ellips] = 1;         // Uij
      (void)sscanf (string, "%s %s %s %f %f %f %f %f %f", tstring, tstring, 
            tstring, &drvui->ellips[drvui->n_ellips][0], &drvui->ellips[drvui->n_ellips][1],
            &drvui->ellips[drvui->n_ellips][2], &drvui->ellips[drvui->n_ellips][3],
            &drvui->ellips[drvui->n_ellips][4], &drvui->ellips[drvui->n_ellips][5]);    /* read unique coefficients */
      for (i = 0; i < 4; i++)
        drvui->ellips_l[drvui->n_ellips][i] = drvui->atom_l[natom][i];
      drvui->ellips_n[drvui->n_ellips] = drvui->atom_n[natom];
      drvui->save_el_number[drvui->n_ellips] = drvui->ellips_n[drvui->n_ellips];
      if (!Quick)
        fprintf (drvui->flout,
             "******      Uij  %c%c%c%c%3d %8.5f %8.5f %8.5f %8.5f %8.5f %8.5f\n",
             drvui->atom_l[natom][0], drvui->atom_l[natom][1], drvui->atom_l[natom][2],
             drvui->atom_l[natom][3]
             , drvui->atom_n[natom], drvui->ellips[drvui->n_ellips][0],
             drvui->ellips[drvui->n_ellips][1], drvui->ellips[drvui->n_ellips][2]
             , drvui->ellips[drvui->n_ellips][3], drvui->ellips[drvui->n_ellips][4],
             drvui->ellips[drvui->n_ellips][5]);
      if (++drvui->n_ellips >= MAX_EL) {
          Error_Box("Too many ellipsoid types specified - increase MAX_EL.");
          return;
      }
    }                         /* string[74] == 'A' */
    drvui->atom_fn[natom] = frame_no;
    drvui->sv_atom_n[natom] = drvui->atom_n[natom];
    if (++natom >= MAX_ATOM) {
        Error_Box("Too many atoms in asymmetric unit - increase MAX_ATOM.");
        return;
    }
    if (!fgets (string, 100, impin)) {
        Error_Box("Error reading GSAS Import File, Run aborted.");
        return;
    }
  }                         /* process atom parameters */
  for (;;) {                     /* start loop for Space Group information */
    if (!fgets (string, 100, impin)) {
        Error_Box("Error reading GSAS Import File, Run aborted.");
        return;
    }
    (void)sscanf (string, "%s %s", tstring, tstring);
    if (strncmp (tstring, "SG", 2) == 0)
      break;                     /* out when space group found */
  }                         /* start loop for Space Group information */
  string[0] = 's';
  string[1] = 'p';
  string[2] = 'g';
  string[3] = 'r';
  for (i = 3; i < 60; i++)
    string[i + 1] = string[i + 10];
  string[60] = '\0';
  if (!Quick)fprintf (drvui->flout, "******      %s\n", string);
  symop (string);
  (void)fclose (impin);
}

/* ************************************************************** */
/* ************************************************************** */

void import_schakal (char input[100], int in_line, int Quick, int frame_no)

/* routine to extract structural information from a SCHAKAL data file */
{
  FILE *impin;
  char string[256];               /* temporary string */
  char filename[256];             /* string for filename */
  int intype;                     /* type of datum read */
  int i, j, k;                     /* temporary */

  if (in_line) {
    impin = drvui->fpin;
  }
  else {
//    (void)sscanf (input, "%s %s", filename, filename); /* get file name */
   strcpy(filename,input); 
   if (!(impin = fopen (filename, "r"))) {
        Error_Box("Cannot Open SCHAKAL Import File, Run aborted.");
        return;
    }
  }
  if (!Quick)
    fprintf (drvui->flout, "****** Now interpreting SCHAKAL-style data\n");
  while (!feof (impin)) {
    if (!fgets (input, 100, impin)) {         /* read a line */
        Error_Box("Error reading SCHAKAL file.");
        return;
    }
    intype = 0;
    for (i = 0; i < 4; i++) {
      if (input[i] == '\0')
    intype = 3;
    }
    if (strncmp (input, "TITL", 4) == 0)
      intype = 1;                 /* echo and ignore */
    if (strncmp (input, "titl ", 4) == 0)
      intype = 1;                 /* echo and ignore */
    if (strncmp (input, "CELL", 4) == 0)
      intype = 2;
    if (strncmp (input, "cell", 4) == 0)
      intype = 2;
    if (strncmp (input, "ATOM", 4) == 0)
      intype = 3;                 /* atom coordinate card */
    if (strncmp (input, "atom", 4) == 0)
      intype = 3;
    if (strncmp (input, "END", 3) == 0)
      intype = 4;                 /* trigger exit */
    if (strncmp (input, "end", 3) == 0)
      intype = 4;                 /* trigger exit */


    switch (intype) {
    case 1:
      for (i = 0; i < 51; i++)
    if (input[i] == '\n')
      input[i] = ' ';
      input[50] = '\0';                 /* truncate title */
      if (!Quick)
    fprintf (drvui->flout, "******      %s\n", input);
      break;                     /* ignore title except for listing */
    case 2:
      for (i = 3; i <= 5; ++i)
          drvui->lat_con[i] = 0.0f;
      (void)sscanf (input, "%s %f %f %f %f %f %f", string, &drvui->lat_con[0], &drvui->lat_con[1]
          , &drvui->lat_con[2], &drvui->lat_con[3], &drvui->lat_con[4], &drvui->lat_con[5]);
      if (!Quick)
          fprintf (drvui->flout,
             "******      Cell     %8.5f %8.5f %8.5f %8.3f %8.3f %8.3f\n",
             drvui->lat_con[0], drvui->lat_con[1], drvui->lat_con[2], drvui->lat_con[3],
             drvui->lat_con[4], drvui->lat_con[5]);
      break;
    case 3:                     /* read an atom line */
      (void)sscanf (input, "%s %c%c%c%c %f %f %f", string,
          &drvui->atom_l[natom][0], &drvui->atom_l[natom][1]
          , &drvui->atom_l[natom][2], &drvui->atom_l[natom][3],
          &drvui->atom_xyz[natom][0], &drvui->atom_xyz[natom][1], &drvui->atom_xyz[natom][2]);
      drvui->atom_n[natom] = 0;
      k = 0;
      for (j = 0; j < 4; j++) {            /* extract atom number from SCHAKAL name */
          if (drvui->atom_l[natom][j] >= '0' && drvui->atom_l[natom][j] <= '9') {
              drvui->atom_n[natom] = drvui->atom_n[natom] * 10 + (int) drvui->atom_l[natom][j] - 48;
          } else {
              drvui->atom_l[natom][k++] = drvui->atom_l[natom][j];
          }
      }
      for (j = k; j < 4; j++)
          drvui->atom_l[natom][j] = ' ';
      if (drvui->atom_n[natom] == 0)
          drvui->atom_n[natom] = 1;
      if (!Quick)
          fprintf (drvui->flout,
              "******      Atom %c%c%c%c%3d %8.5f %8.5f %8.5f\n",
              drvui->atom_l[natom][0], drvui->atom_l[natom][1], drvui->atom_l[natom][2],
              drvui->atom_l[natom][3], drvui->atom_n[natom], drvui->atom_xyz[natom][0],
              drvui->atom_xyz[natom][1], drvui->atom_xyz[natom][2]);
      drvui->atom_fn[natom] = frame_no;
      drvui->sv_atom_n[natom] = drvui->atom_n[natom];
      natom++;
      if (natom >= MAX_ATOM) {
          Error_Box("Too many atoms in asymmetric unit - increase MAX_ATOM.");
          return;
      }
      break;
    case 4:
      if (!Quick)
    fprintf (drvui->flout, "****** END - end of SCHAKAL dataset\n");
      if (!in_line)
    (void)fclose (impin);
      return;
    default:
      break;                     /* end of default */
    }                         /* end of switch */
  }
}                         /* end of import_schakal */

/* ************************************************************** */
/* ************************************************************** */

void import_shelx (char string[100], int in_line, int Quick, int frame_no)

/* routine to extract structural information from a SHELX data file */
{
  FILE *impin;
  char input[200];                /* temporary for continuation cards */
  char filename[100];             /* string for filename */
  int intype;                     /* type of datum read */
  int i, j, k;                    /* temporary */
  float xray;                     /* xray wavelength (read, but not used) */
  int read_start;                 /* flag for start of atom list */
  int read_uij;                   /* flag for uij card */
  float u11, u22;                 /* temporary storage for uij */
  float site_occup;               /* occupancy (read, but not used) */
  static char shcmds[][68] =
    { "ZERR", "LATT", "SYMM", "SFAC", "DISP", "UNIT", "LAUE",
    "REM ", "MORE", "TIME", "OMIT", "SHEL", "BASF", "TWIN",
    "EXTI", "SWAT", "HOPE", "MERG", "SPEC", "RESI", "MOVE",
    "ANIS", "AFIX", "HFIX", "FRAG", "FEND", "EXYZ", "EADP",
    "EQIV", "CONN", "PART", "BIND", "FREE", "DFIX", "DANG",
    "BUMP", "SAME", "SADI", "CHIV", "FLAT", "DELU", "SIMU",
    "DEFS", "ISOR", "NCSY", "SUMP", "L.S.", "CGLS", "BLOC",
    "DAMP", "STIR", "WGHT", "FVAR", "BOND", "CONF", "MPLA",
    "RTAB", "HTAB", "LIST", "ACTA", "SIZE", "TEMP", "WPDB",
    "FMAP", "GRID", "PLAN", "MOLE", "REM\0"
  };
  static char cmd[] = "     ";             /*temporary storage for start of line */
  float fvarpar[30];
  int nfvar=0;
  int fv;
  int latt;
  char *p,*pp;
    
  memset (input, ' ', (size_t) 199);
  latt = 0;

  if (in_line) {
    impin = drvui->fpin;
  } else {
//    (void) sscanf (string, "%*s %s", filename);     /* get file name */
    strcpy(filename,string);
    if (!(impin = fopen (filename, "r"))) {
      Error_Box("Cannot Open SHELX Import File, Run aborted.");
      return;
    }
  }
  read_start = 0;
  read_uij = 0;

  if (drvui->sys == 0) { /* only do this if no spgr command given */
    /* initialize symmetry arrays */
    for (i = 0; i <= 23; ++i) {
      for (j = 0; j <= 2; ++j) {
        drvui->ts[i][j] = 0.0;
        for (k = 0; k <= 2; ++k)
          drvui->ss[i][j][k] = 0;
      }
    }
    for (i = 0; i <= 2; ++i) {
      drvui->lat_pos[0][i] = 0.0;
      drvui->ss[0][i][i] = 1;
      for (j = 0; j <= 3; ++j)
        drvui->spg[i][j] = ' ';
    }

  /* add trivial x,y,z symmetry */
    drvui->ts[0][0]=0;
    drvui->ts[0][1]=0;
    drvui->ts[0][2]=0;
    drvui->ss[0][0][0]=1;
    drvui->ss[0][1][1]=1;
    drvui->ss[0][2][2]=1;
    drvui->ng=1;
  }

  if (!Quick)
    fprintf (drvui->flout, "****** Now interpreting SHELX-style data\n");
  while (!feof (impin)) {
    if (!fgets (string, 100, impin)) {         /* read a line */
      if (!Quick)
         fprintf (drvui->flout, "Error reading input file.");
      return;
    }
    strcpy (input, string);
    if (strchr (input, '=') && (strncmp(input,"TITL",4) && strncmp(input,"REM",3))) {
      (*strchr (input, '=')) = '\0';
      if (!fgets (string, 100, impin)) {
          Error_Box("Error reading str file.");
          return;
      }
      strcat (input, string);
    }
    intype = 0;
    for (i = 0; i < 4; i++) {
      cmd[i] = (char) toupper (input[i]);
      if (input[i] == '\0')
        intype = 3;
    }
    cmd[4] = '\0';

    if (strcmp (cmd, "TITL") == 0)
      intype = 1;                 /* echo and ignore */
    if (strcmp (cmd, "CELL") == 0)
      intype = 2;

    if (strcmp (cmd, "HKLF") == 0) 
    intype = 4;                 /* trigger exit */
    if (strncmp (cmd, "END", 3) == 0)
      intype = 4;                 /* trigger exit */

    if (strncmp (cmd, "FVAR", 4) == 0)
      intype = 5;                 /* store fixed parameters */

    if (drvui->sys == 0) { /* ignore LATT/SYMM if spgr already seen */
      if (strncmp (cmd, "LATT", 4) == 0)
        intype = 6;

      if (strncmp (cmd, "SYMM", 4) == 0)
        intype = 7;
    }
  
    if (intype == 0) {                 /* check against list of 'other' commands */
      for (i = 0; i < 68; i++) {
        if (strlen (cmd) < 4 || strcmp (cmd, shcmds[i]) == 0) {
          intype = 3;
          break;
        }
      }
    }
    if ((strncmp (cmd, "REM", 3) == 0) || (strncmp(cmd,"    ",4) == 0))
      intype = 3;
    switch (intype) {
    case 1:
      trim_string(input,51);
      if (!Quick)
        fprintf (drvui->flout, "******      %s\n", input);
      break;                     /* ignore title except for listing */
    case 2:
      for (i = 3; i <= 5; ++i)
        drvui->lat_con[i] = 0.0f;
      (void) sscanf (input, "%*s %f %f %f %f %f %f %f", &xray, &drvui->lat_con[0]
             , &drvui->lat_con[1], &drvui->lat_con[2], &drvui->lat_con[3], &drvui->lat_con[4]
             , &drvui->lat_con[5]);
      if (!Quick)
        fprintf (drvui->flout,
             "******      Cell     %8.5f %8.5f %8.5f %8.3f %8.3f %8.3f\n",
             drvui->lat_con[0], drvui->lat_con[1], drvui->lat_con[2], drvui->lat_con[3],
             drvui->lat_con[4], drvui->lat_con[5]);
      break;
    case 3:                     /* just ignore most SHELX commands */
      break;
    case 4:
      read_start = 0;
      if (!Quick)
        fprintf (drvui->flout, "****** HKLF or END - end of SHELX dataset\n");
      if (!in_line)
        (void) fclose (impin);
      if (latt == 0 && drvui->sys == 0) {
          drvui->acentric = 0;    /* No LATT line given, spgp = P -1 */
          latt = 1;
          drvui->nlat = 1;
          drvui->nbr = 1;
      }
      if (latt == 3) drvui->sys = 5;
      if (latt == 4) drvui->sys = 6;
      if (drvui->sys == 0) findsys();
      for (i=1; i<drvui->n_ellips; i++) {
        if (drvui->ellips[i][0] <= 0.0f)
          drvui->ell_type[i] = -100;
      }
      if (drvui->autolabel==2) {
        for (i=0;i<natom;i++) {
          drvui->label_fn[drvui->nlabel]=drvui->frame_no;
          drvui->label_x[drvui->nlabel][0]=drvui->atom_xyz[i][0];
          drvui->label_x[drvui->nlabel][1]=drvui->atom_xyz[i][1];
          drvui->label_x[drvui->nlabel][2]=drvui->atom_xyz[i][2];
          int nn=0;
          for (j=0;j<4;j++) {
            if (drvui->atom_l[i][j] != ' ')
              drvui->label_label[drvui->nlabel][nn++]=drvui->atom_l[i][j];
          }
          drvui->label_label[drvui->nlabel][nn]='\0';
          sprintf(cmd,"%d",drvui->atom_n[i]);
          strcat(drvui->label_label[drvui->nlabel],cmd);

          drvui->nlabel++;
        }
        drvui->autolabel=1;
      }

      drvui->auto_ellipse = 0;
      return;
      break;
    case 5: 
      i = sscanf(input,"%*s %f %f %f %f %f %f %f",
            &fvarpar[nfvar],&fvarpar[nfvar+1],&fvarpar[nfvar+2],
            &fvarpar[nfvar+3],&fvarpar[nfvar+4],&fvarpar[nfvar+5],
            &fvarpar[nfvar+6]);
            nfvar +=7;
      if (i<7) nfvar -= (7-i);
      break;
    case 6:
      (void) sscanf(input,"%*s %d",&latt);
      if (latt <0) {
        drvui->acentric = 1;
        latt *= -1;
      } else
        drvui->acentric = 0;
      switch (latt) {
      case 1:
        drvui->nlat = 1;
        drvui->nbr = 1;
        break;
      case 2:
        drvui->nlat = 2;
        drvui->nbr = 6;
        drvui->lat_pos[1][0] = 0.5f;
        drvui->lat_pos[1][1] = 0.5f;
        drvui->lat_pos[1][2] = 0.5f;
        break;
      case 3:
        drvui->nlat = 3;
        drvui->nbr = 7;
        drvui->lat_pos[1][0] = (1.f / 3.f);
        drvui->lat_pos[1][1] = (2.f / 3.f);
        drvui->lat_pos[1][2] = (2.f / 3.f);
        drvui->lat_pos[2][0] = (2.f / 3.f);
        drvui->lat_pos[2][1] = (1.f / 3.f);
        drvui->lat_pos[2][2] = (1.f / 3.f);
        break;
      case 4:
        drvui->nlat = 4;
        drvui->nbr = 5;
        drvui->lat_pos[1][0] = 0.0;
        drvui->lat_pos[1][1] = 0.5f;
        drvui->lat_pos[1][2] = 0.5f;
        drvui->lat_pos[2][0] = 0.5f;
        drvui->lat_pos[2][1] = 0.0;
        drvui->lat_pos[2][2] = 0.5f;
        drvui->lat_pos[3][0] = 0.5f;
        drvui->lat_pos[3][1] = 0.5f;
        drvui->lat_pos[3][2] = 0.0;
        break;
      case 5:
        drvui->nlat = 2;
        drvui->nbr = 2;
        drvui->lat_pos[1][0] = 0.0;
        drvui->lat_pos[1][1] = 0.5f;
        drvui->lat_pos[1][2] = 0.5f;
        break;
      case 6:
        drvui->nlat = 2;
        drvui->nbr = 3;
        drvui->lat_pos[1][0] = 0.5f;
        drvui->lat_pos[1][1] = 0.0;
        drvui->lat_pos[1][2] = 0.5f;
        break;
      case 7:
        drvui->nlat = 2;
        drvui->nbr = 4;
        drvui->lat_pos[1][0] = 0.5f;
        drvui->lat_pos[1][1] = 0.5f;
        drvui->lat_pos[1][2] = 0.0;
      }
      break;

    case 7: /* SYMM */
      drvui->ng++;
      input[0]=' ';
      input[1]=' ';
      input[2]=' ';
      input[3]=' ';
      p = strstr(input,",");
      *p='\0';
      getsym(input,drvui->ng-1,0);
      p++;
      pp = strstr(p,",");
      *pp='\0';
      getsym(p,drvui->ng-1,1);
      pp++;
      getsym(pp,drvui->ng-1,2);
      break;

    default:
      site_occup=1.;
      u11=u22=0.05f;
      i = sscanf (input, "%c%c%c%c %d %f %f %f %f %f %f %f %f %f %f",
          &drvui->atom_l[natom][0], &drvui->atom_l[natom][1],
          &drvui->atom_l[natom][2], &drvui->atom_l[natom][3],
          &drvui->atom_n[natom], &drvui->atom_xyz[natom][0],
          &drvui->atom_xyz[natom][1], &drvui->atom_xyz[natom][2],
          &site_occup, &u11, &u22, &drvui->ellips[drvui->n_ellips][2],
          &drvui->ellips[drvui->n_ellips][5], &drvui->ellips[drvui->n_ellips][4],
          &drvui->ellips[drvui->n_ellips][3]);
      for (j = 0; j < 3; j++)
        if (fabs(drvui->atom_xyz[natom][j]) >15.) {
          fv=((int)fabs(drvui->atom_xyz[natom][j]))/10;
      if (drvui->atom_xyz[natom][j] <0.)
        drvui->atom_xyz[natom][j] = (float)(1.- fvarpar[fv -1]* (drvui->atom_xyz[natom][j]-fv*10));
      else
        drvui->atom_xyz[natom][j] = (float)fvarpar[fv -1]* (drvui->atom_xyz[natom][j]-fv*10);
    } else  
      if (drvui->atom_xyz[natom][j] > 9.)
        drvui->atom_xyz[natom][j] -= 10.0f;
      
    if (fabs(site_occup) >11.) { 
      fv= ((int)fabs(site_occup))/10;
      if (site_occup <0)
        site_occup = (float)(1.0-fvarpar[fv-1]);
      else
        site_occup = fvarpar[fv-1];
    }
    if (fabs(u11) >10.) { 
      fv = ((int)fabs(u11))/10;
      if (u11 <0.)
        u11 = (float)(1.0- fvarpar[fv-1] *(fabs(u11)-fv*10));
      else
        u11 = fvarpar[fv-1] * (u11-fv*10);
    }    

      drvui->atom_n[natom] = 0;
      int k = 0;
      for (j = 0; j < 4; j++)             /* extract atom number if part of SHELX name */
        if (drvui->atom_l[natom][j] >= '0' && drvui->atom_l[natom][j] <= '9') {
          drvui->atom_n[natom] = 10 * drvui->atom_n[natom] + (int) drvui->atom_l[natom][j] - 48;
        } else {
            drvui->atom_l[natom][k++] = drvui->atom_l[natom][j];
        }
        for (j=k;j<4;j++)
            drvui->atom_l[natom][j] = ' ';
        drvui->atom_l[natom][4] = '\0';
        if (drvui->atom_n[natom] == 0)
          drvui->atom_n[natom] = 1;

        if (i > 11) {
          strcpy(drvui->ellips_l[drvui->n_ellips],drvui->atom_l[natom]);
          drvui->ellips_n[drvui->n_ellips] = drvui->atom_n[natom];
          drvui->save_el_number[drvui->n_ellips] = drvui->ellips_n[drvui->n_ellips];
          if (fabs(u22) >10.) { 
            fv = ((int)fabs(u22))/10;
            if (u22 <0.)
              u22 = (float)(1. - fvarpar[fv-1] * (fabs(u22)-fv*10));
            else
              u22 = fvarpar[fv-1] * (u22-fv*10);
          }
          if (fabs(drvui->ellips[drvui->n_ellips][2]) >10.) { 
            fv= ((int)fabs(drvui->ellips[drvui->n_ellips][2]))/10;
            if (drvui->ellips[drvui->n_ellips][2] <0. )
              drvui->ellips[drvui->n_ellips][2] = (float)(1.0 -fvarpar[fv-1] * (fabs(drvui->ellips[drvui->n_ellips][2])-fv*10));
            else
              drvui->ellips[drvui->n_ellips][2] =  fvarpar[fv-1] * (drvui->ellips[drvui->n_ellips][2]-fv*10);
          }
          if (fabs(drvui->ellips[drvui->n_ellips][5]) >10.) { 
            fv= ((int)fabs(drvui->ellips[drvui->n_ellips][5]))/10;
            if (drvui->ellips[drvui->n_ellips][5] <0. )
              drvui->ellips[drvui->n_ellips][5] = (float)(1. -fvarpar[fv-1] * (fabs(drvui->ellips[drvui->n_ellips][5])-fv*10));
            else
              drvui->ellips[drvui->n_ellips][5] =  fvarpar[fv-1] * (drvui->ellips[drvui->n_ellips][5]-fv*10);
          }
          if (drvui->ellips[drvui->n_ellips][4] >10.) { 
            fv= ((int)fabs(drvui->ellips[drvui->n_ellips][4]))/10;
            if (drvui->ellips[drvui->n_ellips][4] <0. )
              drvui->ellips[drvui->n_ellips][4] = (float)(1.0 -fvarpar[fv-1] * (fabs(drvui->ellips[drvui->n_ellips][4])-fv*10));
            else
              drvui->ellips[drvui->n_ellips][4] =  fvarpar[fv-1] * (drvui->ellips[drvui->n_ellips][4]-fv*10);
          }
          if (drvui->ellips[drvui->n_ellips][3] >10.) { 
            fv= ((int)fabs(drvui->ellips[drvui->n_ellips][3]))/10;
            if (drvui->ellips[drvui->n_ellips][3] <0. )
              drvui->ellips[drvui->n_ellips][3] = (float)(1.0 -fvarpar[fv-1] * (fabs(drvui->ellips[drvui->n_ellips][3])-fv*10));
            else
              drvui->ellips[drvui->n_ellips][3] =  fvarpar[fv-1] * (drvui->ellips[drvui->n_ellips][3]-fv*10);
          }

          drvui->ellips[drvui->n_ellips][0] = u11;
          drvui->ellips[drvui->n_ellips][1] = u22;
          drvui->ellips_ismod[drvui->n_ellips] = 0;
          if (!Quick)
            fprintf (drvui->flout,
                 "******      Atom %c%c%c%c%3d %8.5f %8.5f %8.5f (%8.5f)\n",
                 drvui->atom_l[natom][0], drvui->atom_l[natom][1],
                 drvui->atom_l[natom][2], drvui->atom_l[natom][3],
                 drvui->atom_n[natom], drvui->atom_xyz[natom][0],
                 drvui->atom_xyz[natom][1], drvui->atom_xyz[natom][2],
                 site_occup);
          if (!Quick)
            fprintf (drvui->flout,
               "******      Uij  %c%c%c%c%3d %8.5f %8.5f %8.5f %8.5f %8.5f %8.5f\n",
               drvui->atom_l[natom][0], drvui->atom_l[natom][1],
               drvui->atom_l[natom][2], drvui->atom_l[natom][3]
             , drvui->atom_n[natom], drvui->ellips[drvui->n_ellips][0],
               drvui->ellips[drvui->n_ellips][1], drvui->ellips[drvui->n_ellips][2]
             , drvui->ellips[drvui->n_ellips][3], drvui->ellips[drvui->n_ellips][4],
               drvui->ellips[drvui->n_ellips][5]);
          drvui->save_el_number[drvui->n_ellips] = drvui->ellips_n[drvui->n_ellips];
          drvui->do_ellipsoids = 1;
          if (drvui->auto_ellipse==1) {
              drvui->ell_type[drvui->n_ellips] = 1001;       // Uij with ellipcolor
              memset(drvui->ellips_col[drvui->n_ellips],0,40);
              strcpy(drvui->ellips_col[drvui->n_ellips],"Gray30");
          } else {
              drvui->ell_type[drvui->n_ellips] = 1;          // Uij
          }
          drvui->n_ellips++;
          if (drvui->n_ellips >= MAX_EL) {
            Error_Box("Too many ellipsoid types specified - increase MAX_EL.");
            return;
          }
        } else {
          if (i < 10) {
            if (!Quick)
              fprintf (drvui->flout,
                     "******WARNING - too few parameters given for following atom:\n");
            if (site_occup < 0)
              site_occup = 0.;
            if (u11 < 0)
              u11 = 0.;
          }
          if (!Quick)
            fprintf (drvui->flout,
                "******      Atom %c%c%c%c%3d %8.5f %8.5f %8.5f (%8.5f) Uiso %8.5f\n",
                drvui->atom_l[natom][0], drvui->atom_l[natom][1],
                drvui->atom_l[natom][2], drvui->atom_l[natom][3],
                drvui->atom_n[natom], drvui->atom_xyz[natom][0],
                drvui->atom_xyz[natom][1], drvui->atom_xyz[natom][2], site_occup, u11);
        }
        drvui->atom_fn[natom] = frame_no;
        drvui->sv_atom_n[natom] = drvui->atom_n[natom];
        natom++;
        if (natom >= MAX_ATOM) {
           Error_Box("Too many atoms in asymmetric unit - increase MAX_ATOM.");
           return;
        }
        break;                     /* end of default */
      }                         /* end of switch */
  }
  if (!in_line)
    (void) fclose (impin);
  
  if (latt == 3) drvui->sys = 5;
  if (latt == 4) drvui->sys = 6;
  
  if (drvui->sys == 0) findsys();
  for (i=1; i<drvui->n_ellips; i++) {
      if (drvui->ellips[i][0] <= 0.0f)
          drvui->ell_type[i] = -100;
  }
  if (drvui->autolabel==2) {
    for (i=0;i<natom;i++) {
      if(drvui->atom_fn[i] != drvui->frame_no) continue;
      drvui->label_fn[drvui->nlabel]=drvui->frame_no;
      drvui->label_x[drvui->nlabel][0]=drvui->atom_xyz[i][0];
      drvui->label_x[drvui->nlabel][1]=drvui->atom_xyz[i][1];
      drvui->label_x[drvui->nlabel][2]=drvui->atom_xyz[i][2];
      int nn=0;
      for (j=0;j<4;j++) {
        if (drvui->atom_l[i][j] != ' ')
          drvui->label_label[drvui->nlabel][nn++]=drvui->atom_l[i][j];
      }
      drvui->label_label[drvui->nlabel][nn]='\0';
      sprintf(cmd,"%d",drvui->atom_n[i]);
      strcat(drvui->label_label[drvui->nlabel],cmd);

      drvui->nlabel++;
    }
    drvui->autolabel=1;
    drvui->auto_ellipse = 0;
  }
}                         /* end of import_shelx */

/* ************************************************************** */
/* ************************************************************** */

void import_wien (char string[100], int in_line, int Quick, int frame_no)

/* routine to extract structural information from a WIEN2k struct file */
{
  FILE *impin;
  char filename[100];             /* string for filename */
  char line[85];
  int i, j, k, l, k3, k4;                    /* temporary */
  char spgp[50], spstring[40],*spgr;
  char xs[15],ys[15],zs[15];
  int nat=0;
  char lattice[4];

  if (in_line) {
    impin = drvui->fpin;
  } else {
//    (void) sscanf (string, "%*s %s", filename);     /* get file name */
    strcpy(filename,string);
    if (!(impin = fopen (filename, "r"))) {
      Error_Box("Cannot Open WIEN2k Import File, Run aborted.");
      return;
    }
  }

  if (!Quick)
    fprintf (drvui->flout, "****** Now interpreting WIEN2k-style data\n");

   memset(string,0,85);


    if (!fgets (string, 85, impin)) {         /* skip title */
      if (!Quick)
         fprintf (drvui->flout, "Error reading title.");
      return;
    }
    trim_string(string,85);          // kill any ^M or ^J at end of line
    string[85]='\0';

    if (!Quick) fprintf(drvui->flout,"%s\n",string);

    if (!fgets (line, 85, impin)) {         
      if (!Quick)
         fprintf (drvui->flout, "Error reading header.");
      return;
    }


    memset(spgp,0,40);
    sscanf(line,"%s %*s %1d%s",lattice,&natom,spgp);
    if (natom==0) 
      sscanf(line,"%s %*s %*s %d %*d %s",lattice,&natom,spgp);

    if (strchr(spgp,'_'))
      spgr=strchr(spgp,'_')+1;
    else
      spgr=spgp;

    if (!Quick) 
      fprintf(drvui->flout,"%s Lattice, %d atoms, space group %s\n",lattice,natom,spgr);

    if (drvui->sys == 0) {

      strcpy(line,spgr);

/* separate class symbol from other symmetry elements */
      spstring[0] = 's';
      spstring[1] = 'p';
      spstring[2] = 'g';
      spstring[3] = 'r';
      spstring[4] = ' ';
      spstring[5] = line[0];
      spstring[6] = ' ';
      j = 6;
      k = 0;
      k3 = 0;
      k4 = 0;
      l = (int) strlen (line);
      for (i = 1; i <= l; i++) {
        switch (line[i]) {
          case ' ':
            break;
          case '/':
            spstring[++j] = line[i];
            break;
          default:
            j++;
       /* do we have to collate this symbol (for screw axes or mirror planes) ? */
            if (line[i] > '0' && line[i] <= '9') { /* it is a number */
              if (line[0] == 'R' ||             /* no screw axes in R */
                  l == 4 ||                 /* no screw axes in four letter names */
                  k != 0 ||                 /* have just appended to a number */
                  k3 != 0 ||                 /* cannot have further screw axes after 3_x */
                  (k4 != 0 && spstring[j - 1] == '3')) {    /* cannot have 3_x after a 4_x */
                spstring[j] = ' ';             /* so separate it */
                k = 0;                 /* and reset the collation flag */
                j++;
              } else if (spstring[j - 1] != '-' && (spstring[j - 1] <= line[i]
                        || spstring[j - 1] >= 'a')) {
       /* neither a minus sign nor a higher number precedes it */
                spstring[j] = ' ';             /* so separate it */
                k = 0;                 /* and reset the collation flag */
                j++;
              } else {
                k++;                     /* set flag for collation */
                if (spstring[j - 1] == '3')
                  k3 = 1;                 /* if 3_x screw, prohibit further screw axes */
                if (spstring[j - 1] == '4')
                  k4 = 1;                 /* if 4_x screw, prohibit 3_x screw axes */
              }
            } else {                     /* not a number */
              if (spstring[j - 1] != '/') {         /*but not a mirror plane either */
                spstring[j] = ' ';             /* separate it */
                j++;
              }
            }
            spstring[j] = line[i];             /* now add the symbol */
            break;
        }                         /*switch */
      }                         /*for i */
      spstring[j + 1] = '\0';
 
/* Space group 90, P 4 21 2, is not correctly identified by this code. */
      if (!strcmp (line, "P4212"))
        strcpy (spstring, "spgp P 4 21 2");

      symop(spstring);
    } // if spacegroup not already known

   if (!fgets (line, 85, impin)) {         /* read a line */
     if (!Quick)
       fprintf (drvui->flout, "Error reading units flag.");
     return;
   }

  if (!fgets (line, 85, impin)) {         /* read a line */
    if (!Quick)
         fprintf (drvui->flout, "Error reading lattice constants.");
    return;
  }

  i=sscanf(line,"%f %f %f %f %f %f",
		&drvui->lat_con[0],&drvui->lat_con[1],&drvui->lat_con[2],
		&drvui->lat_con[3],&drvui->lat_con[4],&drvui->lat_con[5]); 

  if (i<6 || drvui->lat_con[4]==0. || drvui->lat_con[5]==0.) { //fortran fixed format lacking space characters
    line[39]=' '; 
    line[49]=' ';
    i=sscanf(line,"%f %f %f %f %f %f",
		&drvui->lat_con[0],&drvui->lat_con[1],&drvui->lat_con[2],
		&drvui->lat_con[3],&drvui->lat_con[4],&drvui->lat_con[5]); 
  }

  drvui->lat_con[0] *= (float)0.529; /* conversion from a.u. to angstroms */
  drvui->lat_con[1] *= (float)0.529;
  drvui->lat_con[2] *= (float)0.529;

  if (!Quick) fprintf(drvui->flout,"Lattice constants %f %f %f %5.2f %5.2f %5.2f\n",
		drvui->lat_con[0],drvui->lat_con[1],drvui->lat_con[2],
		drvui->lat_con[3],drvui->lat_con[4],drvui->lat_con[5]); 

  nat=0;

  while (!feof (impin) &&nat <natom) {
    if (!fgets (line, 85, impin)) {         /* read a line */
      if (!Quick)
        fprintf (drvui->flout, "Error reading atom record.");
      return;
    }
    if (!strncmp(line,"ATOM",4)) {
      sscanf(line,"%*s %*s %s %s %s",xs,ys,zs);
      drvui->atom_xyz[nat][0]=(float) atof(xs+2); 
      drvui->atom_xyz[nat][1]=(float) atof(ys+2); 
      drvui->atom_xyz[nat][2]=(float) atof(zs+2); 
      drvui->atom_fn[nat]=frame_no;
    } else if (strstr(line,"NPT=")) {
      sscanf (line,"%c%c%c%c",
         &drvui->atom_l[nat][0], &drvui->atom_l[nat][1],
         &drvui->atom_l[nat][2], &drvui->atom_l[nat][3]);
      drvui->atom_l[nat][4]='\0';

      drvui->atom_n[nat] = 0;
      k = 0;
      for (j = 0; j < 4; j++)             /* extract atom number if part of name */
        if (drvui->atom_l[nat][j] >= '0' && drvui->atom_l[nat][j] <= '9') {
          drvui->atom_n[nat] = 10 * drvui->atom_n[nat] + (int) drvui->atom_l[nat][j] - 48;
        } else {
            drvui->atom_l[nat][k++] = drvui->atom_l[nat][j];
        }
        for (j=k;j<4;j++)
            drvui->atom_l[nat][j] = ' ';
        drvui->atom_l[nat][4] = '\0';
        if (drvui->atom_n[nat] == 0)
          drvui->atom_n[nat] = 1;
        drvui->sv_atom_n[nat] = drvui->atom_n[nat];

      nat++;
    }
  }

  if (!Quick) {
    for (i=0;i<natom;i++) fprintf(drvui->flout,"Atom %c%c%c%c %d  %f %f %f\n",
          drvui->atom_l[i][0], drvui->atom_l[i][1],
          drvui->atom_l[i][2], drvui->atom_l[i][3],
          drvui->atom_n[i],
          drvui->atom_xyz[i][0], drvui->atom_xyz[i][1], 
          drvui->atom_xyz[i][2]); 
  }

  if (!in_line)
    (void) fclose (impin);
}                         /* end of import_wien */

/* ************************************************************** */
/* ************************************************************** */
void import_discus (char input[100], int in_line, int Quick, int frame_no)

/* routine to extract structural information from a DISCUS file */
{
  FILE *impin;
  char string[200];                 /* temporary string */
  char filename[256];                 /* string for filename */
  int i, j;                     /* temporary */
  int k, l, k3, k4;                 /*flags for space group symbol parsing */
  char spstring[100];                 /* space group symbol */
  int nx,ny,nz,nc;              /* number of cells if structure */
  int structflag=0;		/* cell or structure */
  int ng=-1;                     /* counter for symmetry generators */
  int gss[10][3][3];            /* temporary storage for */
  float gts[10][3];             /* additional generator symmetry */

  memset (spstring, 0, 100);
  memset (string, 0, 100);

  if (in_line) {
    impin = drvui->fpin;
  } else {
//    (void)sscanf (input, "%s %s", filename, filename); /* get file name */
    strcpy(filename,input);
    if (!(impin = fopen (filename, "r"))) {
        Error_Box("Cannot Open DISCUS Import File, Run aborted.");
        return;
    }
  }

  if(!Quick)fprintf (drvui->flout, "****** Now interpreting DISCUS-style data\n");

  if (!fgets (input, 100, impin)) {         /* read a line */
      Error_Box("Error reading DISCUS file - end of file on first line.");
  if (!in_line)
    (void)fclose (impin);
      return;
  }

  if (strncmp(input,"title",5)) {
      Error_Box("Error reading DISCUS file - file does not start with a title.");
  if (!in_line)
    (void)fclose (impin);
      return;
  }

  while (!feof(impin)) {
  if (!fgets (input, 100, impin)) {         /* read a line */
      Error_Box("Error - unexpected end of file while reading DISCUS file.");
  if (!in_line)
    (void)fclose (impin);
      return;
  }
  
  if (!strncmp(input,"cell",4)) {
    sscanf(input,"%*s %f%*[ ,]%f%*[ ,]%f%*[ ,]%f%*[ ,]%f%*[ ,]%f",&drvui->lat_con[0],&drvui->lat_con[1],
           &drvui->lat_con[2],&drvui->lat_con[3],&drvui->lat_con[4],&drvui->lat_con[5]);
    if(!Quick)fprintf (drvui->flout, "******      Cell %5.3f %5.3f %5.3f %5.2f %5.2f %5.2f\n",
      drvui->lat_con[0],drvui->lat_con[1],drvui->lat_con[2],drvui->lat_con[3],drvui->lat_con[4],drvui->lat_con[5]);
  } else if (!strncmp(input,"ncell",5)) {
    structflag=1;
    sscanf(input,"%*s %d,%d,%d,%d",&nx,&ny,&nz,&nc);
    strcpy(spstring,"spgr P 1");
    drvui->lat_con[0] *= (float)nx;
    drvui->lat_con[1] *= (float)ny;
    drvui->lat_con[2] *= (float)nz;
    if (!Quick)
      fprintf(drvui->flout, "******      %d x %d x %d supercell, transforming to P1\n",
              nx,ny,nz);
  } else if (!strncmp(input,"spcgr",5)) {
    sscanf(input,"%*s %s",string);
    /* separate class symbol from other symmetry elements */
    spstring[0] = 's';
    spstring[1] = 'p';
    spstring[2] = 'g';
    spstring[3] = 'r';
    spstring[4] = ' ';
    spstring[5] = string[0];
    spstring[6] = ' ';
    j = 6;
    k = 0;
    k3 = 0;
    k4 = 0;
    l = (int) strlen (string);
    for (i = 1; i <= l; i++) {
      switch (string[i]) {
        case ' ':
        break;
        case '/':
          spstring[++j] = string[i];
        break;
        default:
          j++;
      /* do we have to collate this symbol (for screw axes or mirror planes) ? */
          if (string[i] > '0' && string[i] <= '9') { /* it is a number */
            if (string[0] == 'R' ||             /* no screw axes in R */
            l == 4 ||                 /* no screw axes in four letter names */
            k != 0 ||                 /* have just appended to a number */
            k3 != 0 ||                 /* cannot have further screw axes after 3_x */
            (k4 != 0 && spstring[j - 1] == '3')) {    /* cannot have 3_x after a 4_x */
              spstring[j] = ' ';             /* so separate it */
              k = 0;                 /* and reset the collation flag */
              j++;
            } 
            else if (spstring[j - 1] != '-' && (spstring[j - 1] <= string[i]
                        || spstring[j - 1] >= 'a')) {
         /* neither a minus sign nor a higher number precedes it */
              spstring[j] = ' ';             /* so separate it */
              k = 0;                 /* and reset the collation flag */
              j++;
            }
            else {
              k++;                     /* set flag for collation */
              if (spstring[j - 1] == '3')
                k3 = 1;                 /* if 3_x screw, prohibit further screw axes */
                if (spstring[j - 1] == '4')
                 k4 = 1;                 /* if 4_x screw, prohibit 3_x screw axes */
            }
          }
          else {                     /* not a number */
            if (spstring[j - 1] != '/') {         /*but not a mirror plane either */
              spstring[j] = ' ';             /* separate it */
              j++;
            }
          }
          spstring[j] = string[i];             /* now add the symbol */
        break;
      }                         /*switch */
    }                         /*for i */
    spstring[j + 1] = '\0';

/* Space group 90, P 4 21 2, is not correctly identified by this code. 
   It accounts for only 14 of the 175000 entries in the CSD (and not all 
   of them have coordinates), so this quick and dirty fix is all it can
   expect from us. Does this spacegroup really exist ? :-) */
    if (!strcmp (string, "P4212"))
      strcpy (spstring, "P 4 21 2");

    if(!Quick)fprintf (drvui->flout, "******      Space group %s\n", string);
/* spacegroup might be overridden by ncell, so do not call symop immediately */
  } else if (!strncmp(input,"generator",9)) {
    ng++;
    sscanf(input,"*s %d%*[ ,]%d%*[ ,]%d%*[ ,]%f%*[ ,]%d%*[ ,]%d%*[ ,]%d%*[, ]%f%*[ ,]%d%*[ ,]%d%*[ ,]%d%*[ ,]%f",
      &gss[ng][0][0],&gss[ng][0][1],&gss[ng][0][2],&gts[ng][0],
      &gss[ng][1][0],&gss[ng][1][1],&gss[ng][1][2],&gts[ng][1],
      &gss[ng][2][0],&gss[ng][2][1],&gss[ng][2][2],&gts[ng][2]);
    if (!Quick) {
      fprintf(drvui->flout,"****** Generator (%d %d %d) (%5.1f)\n", 
        gss[ng][0][0],gss[ng][0][1],gss[ng][0][2],gts[ng][0]);
      fprintf(drvui->flout,"******           (%d %d %d) (%5.1f)\n", 
        gss[ng][1][0],gss[ng][1][1],gss[ng][1][2],gts[ng][1]);
      fprintf(drvui->flout,"******           (%d %d %d) (%5.1f)\n", 
        gss[ng][2][0],gss[ng][2][1],gss[ng][2][2],gts[ng][2]);
    }
  } else if (!strncmp(input,"atoms",5)) {
    natom=0;
    while (fgets(input,100,impin)!=NULL) {
      if (input[0]=='#') continue;
    if (strstr(input,"molecule")) {
      if (!Quick)
        fprintf(drvui->flout,"*** molecule keyword is currently not supported !\n");
      continue;
    } 
    memset(drvui->atom_l[natom],0,4);
    sscanf(input,"%4c %f %f %f %*f", drvui->atom_l[natom],
           &drvui->atom_xyz[natom][0],&drvui->atom_xyz[natom][1],&drvui->atom_xyz[natom][2]);
    if (structflag == 1) {
      drvui->atom_xyz[natom][0] /= (float)nx;
      drvui->atom_xyz[natom][1] /= (float)ny;
      drvui->atom_xyz[natom][2] /= (float)nz;
    } 
    drvui->atom_fn[natom]=frame_no;
    int k = 0;
    for (j = 0; j < 4; j++) {             /* extract atom number if part of name */
      if (drvui->atom_l[natom][j] >= '0' && drvui->atom_l[natom][j] <= '9') {
        drvui->atom_n[natom] = 10 * drvui->atom_n[natom] + (int) drvui->atom_l[natom][j] - 48;
      } else {
        drvui->atom_l[natom][k++] = drvui->atom_l[natom][j];
      }
    }
    for (j=k;j<4;j++)
      drvui->atom_l[natom][j] = ' ';

      drvui->atom_l[natom][4] = '\0';
      if (drvui->atom_n[natom] == 0)
        drvui->atom_n[natom] = 1;
      drvui->sv_atom_n[natom] = drvui->atom_n[natom];
      if (++natom >= MAX_ATOM) {
        Error_Box("Too many atoms in asymmetric unit - increase MAX_ATOM.");
        if (!in_line)
          (void)fclose (impin);
        return;
      }
    } /* here at end of input, i.e. when the last atom has been read */
    if (!Quick) for (i=0;i<natom;i++) fprintf(drvui->flout,"******      Atom %s %d %f %f %f\n",
        drvui->atom_l[i],drvui->atom_n[i],drvui->atom_xyz[i][0],drvui->atom_xyz[i][1],drvui->atom_xyz[i][2]);

    symop (spstring);
    if (ng > -1 && structflag==0) {
      for (i=0;i<=ng;i++) {
        int ig=++drvui->ng;
        drvui->ss[ig][0][0]=gss[i][0][0];
        drvui->ss[ig][0][1]=gss[i][0][1];
        drvui->ss[ig][0][2]=gss[i][0][2];
        drvui->ss[ig][1][0]=gss[i][1][0];
        drvui->ss[ig][1][1]=gss[i][1][1];
        drvui->ss[ig][1][2]=gss[i][1][2];
        drvui->ss[ig][2][0]=gss[i][2][0];
        drvui->ss[ig][2][1]=gss[i][2][1];
        drvui->ss[ig][2][2]=gss[i][2][2];
        drvui->ts[ig][0]=gts[i][0];
        drvui->ts[ig][1]=gts[i][1];
        drvui->ts[ig][2]=gts[i][2];
      }
    }  
    if (!in_line)
      (void)fclose (impin);

    return;
    } /* end of atom records */
  } /* main keyword parsing loop */

/* normal exit is through the EOF condition in the atoms loop */
  Error_Box("Unexpected end of DISCUS file - no atom records found.");
  if (!in_line)
    (void)fclose (impin);
  return;
}

/* ************************************************************** */
/* ************************************************************** */

void import_pcr (char input[100], int in_line, int Quick, int frame_no)

/* routine to extract structural information from a FULLPROF pcr file */
{
  char string[256], filename[256], temp[5], tstring[21];
  int phase_no, i, j;
  int ii,jj;
  int nat,anis;
  FILE *impin;

  if (in_line) {
      Error_Box("Cannot read inline FULLPROF data. Please use the import instruction");
      return;
  }

  phase_no = 0;

  ii=strlen(input)-1;
  if (isdigit(input[ii])) { // if number at end 
    for (jj=ii;jj>ii-6;jj--) {
      if (isspace(input[jj])) { // separate number
        input[jj]='\0';
        phase_no=atoi(&input[jj+1]);
        break;
      }
      if (isalpha(input[jj]))
        break; // unless it is attached to filename
      }
   }
  strcpy(filename,input);
  if (phase_no < 1) {
      Error_Box("Illegal phase number in FULLPROF import instruction, Run aborted.");
      return;
  }
  if (!(impin = fopen (filename, "r"))) {
      Error_Box("Cannot Open FULLPROF Import File, Run aborted.");
      return;
  }
  sprintf (temp, "for PHASE number: %3d", phase_no);         /* make search object */
  for (;;) {
    if (!fgets (string, 200, impin)) {         /* search for phase info */
        Error_Box("Error reading phase info from FULLPROF Import File, Run aborted.");
        (void)fclose (impin);
        return;
    }
    if (strstr (string, temp) != 0)
      break;
  }

    if (!fgets (string, 200, impin)) {  
        Error_Box("Error reading from FULLPROF Import File, Run aborted.");
        (void)fclose (impin);
        return;
    }

    if (!fgets (string, 200, impin)) {  
        Error_Box("Error reading from FULLPROF Import File, Run aborted.");
        (void)fclose (impin);
        return;
    }
    (void)sscanf(string,"%20c",tstring);
    tstring[20]='\0';
    if (!Quick)
      fprintf(drvui->flout,"******      Phase %d: %s\n",phase_no,tstring); //title

    do {
        if (!fgets (string, 200, impin)) {  
        Error_Box("Error reading from FULLPROF Import File, Run aborted.");
        (void)fclose (impin);
        return;
        }    
    trim_string(string,200);
    } while (string[0]=='!' || !strlen(string) ); //skip any comment lines and empty lines

    nat=0;
    sscanf(string,"%d",&nat); // next record must start with number of atoms

  for (;;) {                     /* search for spacegroup */
    if (!fgets (string, 200, impin)) {
        Error_Box("Error reading spacegroup from FULLPROF File, Run aborted.");
        (void)fclose (impin);
        return;
    }
    if (strstr (string, "Space group")) {
      (void)sscanf (string, "%20c", tstring);
      tstring[20]='\0';
      strcpy(string,"spgr ");
      strcat(string,tstring);
      if (!Quick)fprintf (drvui->flout, "******      %s\n", string);
      symop (string);
      break;
    }
  }

  for (;;) {                     /* search for start of atom list */
    if (!fgets (string, 200, impin)) {
        Error_Box("Error reading atom records from FULLPROF Import File, Run aborted.");
        (void)fclose (impin);
        return;
    }
    if (!strncmp(string,"!Atom Typ",9)) 
      break;
  }
  
    do {
        if (!fgets (string, 200, impin)) {  
        Error_Box("Error reading atom data from FULLPROF Import File, Run aborted.");
        (void)fclose(impin);
        return;
        }    
        if (string[0]=='!') continue; //skip any comment lines

        anis=0;
        if (isalpha(string[0])) {
          sscanf(string,"%s %*s %f %f %f %*f %*f %*d %*d %d %*d",
          tstring, &drvui->atom_xyz[natom][0], &drvui->atom_xyz[natom][1],
          &drvui->atom_xyz[natom][2],&anis);
          
          drvui->atom_n[natom] = 0;
          for (i = 0; i < 4; i++)
            drvui->atom_l[natom][i] = ' ';             /* initialize atom name */
          j = 0;
          if (string[0] >= '0' && string[0] <= '9')
            drvui->atom_n[natom] = (int) string[0] - 48;
          else
            drvui->atom_l[natom][j++] = string[0];
          if (string[1] >= '0' && string[1] <= '9')
            drvui->atom_n[natom] = 10 * drvui->atom_n[natom] + (int) string[1] - 48;
          else
            drvui->atom_l[natom][j++] = string[1];
          if (string[2] >= '0' && string[2] <= '9')
            drvui->atom_n[natom] = 10 * drvui->atom_n[natom] + (int) string[2] - 48;
          else
            drvui->atom_l[natom][j++] = string[2];
          if (string[3] >= '0' && string[3] <= '9')
            drvui->atom_n[natom] = 10 * drvui->atom_n[natom] + (int) string[3] - 48;
          else
            drvui->atom_l[natom][j++] = string[3];
          if (!Quick)
            fprintf (drvui->flout,
                     "******      Atom %c%c%c%c%3d %8.5f %8.5f %8.5f\n",
                      drvui->atom_l[natom][0], drvui->atom_l[natom][1], 
                      drvui->atom_l[natom][2], drvui->atom_l[natom][3],
                      drvui->atom_n[natom], drvui->atom_xyz[natom][0], 
                      drvui->atom_xyz[natom][1], drvui->atom_xyz[natom][2]);
                      drvui->atom_fn[natom] = frame_no;
                      drvui->sv_atom_n[natom] = drvui->atom_n[natom];
          drvui->atom_fn[natom] = frame_no;
          drvui->sv_atom_n[natom] = drvui->atom_n[natom];
          fgets(string,200,impin); // skip codewords
          if (anis == 2) {
            strcpy(drvui->ellips_l[drvui->n_ellips],drvui->atom_l[natom]);
            drvui->ellips_n[drvui->n_ellips] = drvui->atom_n[natom];
            drvui->save_el_number[drvui->n_ellips] = drvui->ellips_n[drvui->n_ellips];
            drvui->ellips_ismod[drvui->n_ellips] = 0;
            drvui->ell_type[drvui->n_ellips] = 0;    
            fgets(string,200,impin); // b factors
            if (string[0]=='!') fgets(string,100,impin);
            sscanf (string,"%f %f %f %f %f %f",
              &drvui->ellips[drvui->n_ellips][0], &drvui->ellips[drvui->n_ellips][1],
              &drvui->ellips[drvui->n_ellips][2], &drvui->ellips[drvui->n_ellips][3],
              &drvui->ellips[drvui->n_ellips][4], &drvui->ellips[drvui->n_ellips][5]);
          if (!Quick)
            fprintf (drvui->flout,
                     "******      Beta_ij %8.5f %8.5f %8.5f %8.5f %8.5f %8.5f\n",
                      drvui->ellips[drvui->n_ellips][0], drvui->ellips[drvui->n_ellips][1],
                      drvui->ellips[drvui->n_ellips][2], drvui->ellips[drvui->n_ellips][3],
                      drvui->ellips[drvui->n_ellips][4], drvui->ellips[drvui->n_ellips][5]);
            if (++drvui->n_ellips >= MAX_EL) {
              Error_Box("Too many ellipsoid types specified - increase MAX_EL.");
              (void)fclose (impin);
              return;
            }
          }          
          if (++natom >= MAX_ATOM) {
              Error_Box("Too many atoms in asymmetric unit - increase MAX_ATOM.");
              (void)fclose (impin);
              return;
          }
        } // if line starts with (atom) label
      } while (natom<nat); 

  for (;;) {                     /* search for lattice constants */
    if (!fgets (string, 200, impin)) {
        Error_Box("Error reading lattice constants from FULLPROF Import File, Run aborted.");
        (void)fclose (impin);
        return;
    }
    if (strstr(string,"alpha") && strstr(string,"beta") && strstr(string,"gamma")) {
      if (!fgets (string, 200, impin)) {
        Error_Box("Error reading lattice constants from FULLPROF Import File, Run aborted.");
        (void)fclose (impin);
        return;
      }
      sscanf(string,"%f %f %f %f %f %f", &drvui->lat_con[0],&drvui->lat_con[1], 
                   &drvui->lat_con[2],&drvui->lat_con[3],&drvui->lat_con[4],
                   &drvui->lat_con[5]);             /* read lattice constants */
      if (!Quick) fprintf (drvui->flout,
         "******      Cell     %8.5f %8.5f %8.5f %8.3f %8.3f %8.3f\n",
         drvui->lat_con[0], drvui->lat_con[1], drvui->lat_con[2], drvui->lat_con[3], drvui->lat_con[4],
         drvui->lat_con[5]);
      (void)fclose (impin);
      return;
    }
  }
}

/* ************************************************************** */
/* ************************************************************** */

/* ************************************************************** */
/* ************************************************************** */

void Init_DRAWxtl(void)
{
  int j, l, m;
/* Initialize */
  drvui->X_Origin = drvui->Y_Origin = drvui->Z_Origin = 0.5f;
  drvui->X_Boxlim = drvui->Y_Boxlim = drvui->Z_Boxlim = 20.0f;
  drvui->Trans[0] = drvui->Trans[1] = drvui->Trans[2] = 0.0f;
  xrot = yrot = zrot = 0.0f;
  boxflag = packflag = clipflag = 0;
  drvui->do_ellipsoids = edges = 0;
  drvui->Ellipsoid_Prob = 0.5f;
  cur_cen[0] = 0.25f;
  cur_cen[1] = 0.25f;
  cur_cen[2] = 0.25f;                // location of cursor (fractional coordinates)
  drvui->cur_step = 0.5f;            // amount to jump cursor in A
  Unit_Cell = M_cameras = no_comment = Labels = Vrml2 = 1;
  Display_axes = drvui->El_Cutout = 0;
  strcpy (drvui->Cutout_color, "Gray20");
  strcpy (drvui->Ellipaxis_color, "Gray20");
  drvui->Ellipaxis_width = 0.;
  drvui->SpMult = drvui->BndMult = Magnification = 1.0f;
  drvui->Ellipsoid_Scale = P_to_C (0.5f);
  Options = 0;
  FourierMapType = 0;
  Map_Info.info_valid = 0;
  Map_Info.map_type = -1;
  drvui->sys = 0;
  drvui->polylimit = 0.1f;
  drvui->Phong_Value = 0.2f;
  drvui->Phong_Size = 1.0f;
  DepthCue = 0.0f;
  drvui->noshadow = 0;
/*
  drvui->ambient = 0.0f;
  drvui->diffuse = 0.0f;
  drvui->specular = 0.0f;
  drvui->roughness = 0.0f;
*/
  printdist = 3.5f;
  origin[0] = drvui->X_Origin;
  origin[1] = drvui->Y_Origin;
  origin[2] = drvui->Z_Origin;
  boxlim[0] = drvui->X_Boxlim;
  boxlim[1] = drvui->Y_Boxlim;
  boxlim[2] = drvui->Z_Boxlim;
  memset(drvui->poly_rad_edge,0,sizeof(drvui->poly_rad_edge));
  docell = Unit_Cell;
  Color_Warning = 0;
  drvui->El_Cutout = 0;
  strcpy(drvui->Cutout_color,"");
  rad_cell = 0.02f;
  drvui->Sphere_Mult = drvui->SpMult;
  drvui->Bond_Mult = drvui->BndMult;
  natom = 0;
  drvui->nmag = 0;
  drvui->nsphere = drvui->npoly = drvui->nbond = drvui->nplane = 1;
  drvui->n_ellips = drvui->ncone = drvui->nlabel = drvui->nbplane = 1;
  drvui->nlabel = 1;
  drvui->poly_size[0] = 2.5f;
  drvui->poly_min[0] = 0.005f;
  memset(drvui->poly_l[0],' ',4);
  drvui->plane_size[0] = 2.5f;
  memset(drvui->plane_l[0],' ',4);
  drvui->sphere_size[0] = 0.0f;
  memset(drvui->sphere_l[0],' ',4);
//  POV_Max[0] = POV_Max[1] = POV_Max[2] = -99999.0f;
//  POV_Min[0] = POV_Min[1] = POV_Min[2] = 99999.0f;
  drvui->glback[0] = drvui->glback[1] = drvui->glback[2] = 1.0f;
  slabmode = 0;
  domolcomp = 0;
  drvui->mol_d = 0.0f;
  drvui->modulated = 0;
  for (j=0; j<3; j++)
      drvui->phaseshift[j] = 0.0;
  Omit->nomits=0;
  drvui->no_subsys = 1;   /* initialize the subsystem variables */
  for (l=0; l<3; l++) {
      for (m=0; m<3; m++)
          drvui->subsys_fact[0][l][m] = 0.0f;
      drvui->subsys_fact[0][l][l] = 1.0f; /* set to identity */
  }
  drvui->subsys_ref_volume = 1.0f;
  drvui->subsys_vol[0] = 1.0;
  memset(Omit->omit1,0,1000);
  memset(Omit->omit2,0,1000);
  memset(drvui->slab_con,0,sizeof(drvui->slab_con));
  memset(drvui->slab_rot,0,sizeof(drvui->slab_rot));
  memset(drvui->slab_off,0,sizeof(drvui->slab_off));
// clear all the color variables
//  memset(drvui->Ellipaxis_color,0,sizeof(drvui->Ellipaxis_color));
  memset(drvui->col_cell,0,sizeof(drvui->col_cell));
  memset(drvui->col_edge,0,sizeof(drvui->col_edge));
  memset(drvui->col_cone,0,sizeof(drvui->col_cone));
  memset(drvui->col_arrow,0,sizeof(drvui->col_arrow));
  memset(drvui->col_bg,0,sizeof(drvui->col_bg));
  memset(drvui->col_bond,0,sizeof(drvui->col_bond));
  memset(drvui->Cutout_color,0,sizeof(drvui->Cutout_color));
  memset(drvui->ellips_col,0,sizeof(drvui->ellips_col));
  memset(drvui->col_edge,0,sizeof(drvui->col_edge));
  memset(drvui->poly_col,0,sizeof(drvui->poly_col));
  memset(drvui->plane_col,0,sizeof(drvui->plane_col));
  memset(drvui->sphere_col,0,sizeof(drvui->sphere_col));
// clear all label variables
  memset(drvui->atom_l,0,sizeof(drvui->atom_l));
  memset(drvui->bond_l1,0,sizeof(drvui->bond_l1));
  memset(drvui->bond_l2,0,sizeof(drvui->bond_l2));
  memset(drvui->cone_l1,0,sizeof(drvui->cone_l1));
  memset(drvui->ellips_l,0,sizeof(drvui->ellips_l));
  memset(drvui->plane_l,0,sizeof(drvui->plane_l));
  memset(drvui->poly_l,0,sizeof(drvui->poly_l));
  memset(drvui->sphere_l,0,sizeof(drvui->sphere_l));

  strcpy (drvui->col_cell, "Black");             /* preset unit-cell colors to Black */
  strcpy (drvui->col_bg, "White");
  strcpy(Map_Info.title," Title not given in map file");

}

/* ************************************************************** */
/* ************************************************************** */

float P_to_C (float prob)

/* Routine to convert probability to ellipsoid scaling, C.  Given probability
   'prob', calculate value of C required for ellipsoid,

   C^2 = (delx/sigx)^2 + (dely/sigy)^2 + (delz/sigz)^2,

   where 'prob' is the probability that a random point in the distribution
   will fall inside the ellipsoid.  Table values taken from D.B. Owen,
   "Handbook of Statistical Tables", Addison-Wesley, Reading, Mass., 1962.

   N.B. The present version of the tables only handles probabilities from
   0.01 to 0.99.
*/
#define N_C 99
{
  static double C_table[N_C + 1] =
    { 0.3389, 0.4299, 0.4951, 0.5479, 0.5932, 0.6334, 0.6699, 0.7035,
    0.7349, 0.7644,
    0.7924, 0.8192, 0.8447, 0.8694, 0.8932, 0.9162, 0.9386, 0.9605,
    0.9818, 1.0026,
    1.0230, 1.0430, 1.0627, 1.0821, 1.1012, 1.1200, 1.1386, 1.1570,
    1.1751, 1.1932,
    1.2110, 1.2288, 1.2464, 1.2638, 1.2812, 1.2985, 1.3158, 1.3330,
    1.3501, 1.3672,
    1.3842, 1.4013, 1.4183, 1.4354, 1.4524, 1.4695, 1.4866, 1.5037,
    1.5209, 1.5382,
    1.5555, 1.5729, 1.5904, 1.6080, 1.6257, 1.6436, 1.6616, 1.6797,
    1.6980, 1.7164,
    1.7351, 1.7540, 1.7730, 1.7924, 1.8119, 1.8318, 1.8519, 1.8724,
    1.8932, 1.9144,
    1.9360, 1.9580, 1.9804, 2.0034, 2.0269, 2.0510, 2.0757, 2.1012,
    2.1274, 2.1544,
    2.1824, 2.2114, 2.2416, 2.2730, 2.3059, 2.3404, 2.3767, 2.4153,
    2.4563, 2.5003,
    2.5478, 2.5997, 2.6571, 2.7216, 2.7955, 2.8829, 2.9912, 3.1365,
    3.3682, 5.9503
  };

  int i;
  double C;

  i = (int) (100.0 * prob);             /* find correct table entry */
  if (i > N_C)
    i = N_C;
/* interpolate (linear) */
  C = C_table[i - 1] + (prob - 0.01 * i) * (C_table[i] - C_table[i - 1]);
  return ((float) C);
}


/* ************************************************************** */
/* ************************************************************** */

void read_dn6(char * infile, int Quick)
{
/* routine to read a dn6 or 'O' format Fourier map */
      FILE * mapin;
      float rhomin=1.0e15f, rhomax=-1.0e15f;
      char string[200];
      uchar cube[512];
      float rhoscal;
      int rhoadd;
      int iscal1, iscal2;
      int i, j, k;
      int ori[3];
      int nga, ngb, ngc;
      int kk, jj, ii;
      int ncube;
      int mappos;
      int nz, ny, nx;

      if ((mapin = fopen (infile, "rb")) == NULL) {
        sprintf(string,"Cannot open Fourier map (.dn6) file %s", infile);
        Error_Box(string);
        return;
      }
      if (!fillcube(cube, mapin)) {
          Error_Box("Error reading DN6 Fourier File.");
          fclose(mapin);
          return;
      }
/* TODO confirm the description of ori */
      ori[0] = (int)cube[0] * 256 + (int)cube[1]; /* get starting point of map */
      ori[1] = (int)cube[2] * 256 + (int)cube[3];
      ori[2] = (int)cube[4] * 256 + (int)cube[5];
      nga = (int)cube[6] * 256 + (int)cube[7];
      ngb = (int)cube[8] * 256 + (int)cube[9];
      ngc = (int)cube[10] * 256 + (int)cube[11];
      mapstep_a = (int)cube[12] * 256 + (int)cube[13];
      mapstep_b = (int)cube[14] * 256 + (int)cube[15];
      mapstep_c = (int)cube[16] * 256 + (int)cube[17];
      iscal1 = (int)cube[34] * 256 + (int)cube[35];
      map_a = (float)((int)cube[18] * 256 + (int)cube[19])/(float)iscal1;
      map_b = (float)((int)cube[20] * 256 + (int)cube[21])/(float)iscal1;
      map_c = (float)((int)cube[22] * 256 + (int)cube[23])/(float)iscal1;
      map_alpha = (float)((int)cube[24] * 256 + (int)cube[25])/(float)iscal1;
      map_beta = (float)((int)cube[26] * 256 + (int)cube[27])/(float)iscal1;
      map_gamma = (float)((int)cube[28] * 256 + (int)cube[29])/(float)iscal1;
      iscal2 = (int)cube[36] * 256 + (int)cube[37];
      rhoscal = (float)((int)cube[30] * 256 + (int)cube[31])/(float)iscal2;
      rhoadd = (int)cube[32] * 256 + (int)cube[33];
      Map_Info.lat_con[0] = map_a;
      Map_Info.lat_con[1] = map_b;
      Map_Info.lat_con[2] = map_c;
      Map_Info.lat_con[3] = map_alpha;
      Map_Info.lat_con[4] = map_beta;
      Map_Info.lat_con[5] = map_gamma;
      Map_Info.map_int[0] = mapstep_a;
      Map_Info.map_int[1] = mapstep_b;
      Map_Info.map_int[2] = mapstep_c;
      Map_Info.xlim[0] = (float)ori[0]/(float)mapstep_a;
      Map_Info.ylim[0] = (float)ori[1]/(float)mapstep_b;
      Map_Info.zlim[0] = (float)ori[2]/(float)mapstep_c;
      Map_Info.xlim[1] = (float)(nga + ori[0] - 1)/(float)mapstep_a++;
      Map_Info.ylim[1] = (float)(ngb + ori[1] - 1)/(float)mapstep_b++;
      Map_Info.zlim[1] = (float)(ngc + ori[2]- 1)/(float)mapstep_c++;
       
      Map_Info.info_valid = 1;

      if (FourierPt) free(FourierPt);

      FourierPt = (float*)zalloc( mapstep_a * mapstep_b * mapstep_c * sizeof(float));
      if (FourierPt == NULL) {
          Error_Box("ERROR -- Unable to allocate space for map.\n");
          fclose(mapin);
          return;
      }

/* read the Rho values - map file has x fastest, and z slowest,
 * In FourierPt, we store with z fastest */

      for(k=0; k<ngc; k=k+8) {
          for(j=0; j<ngb; j=j+8) {
            for(i=0; i<nga; i=i+8) {
                if (!fillcube(cube, mapin)) {   /* read the next 8 x 8 x 8 block */
                    Error_Box("Error reading DN6 Fourier File.");
                    fclose(mapin);
                    return;
                } 
                for (kk=0; kk<min(8,ngc-k); kk++) { /* unpack the cube */
                    nz = k + kk + ori[2];
                    for (jj=0; jj<min(8,ngb-j); jj++) {
                        ny = (j + jj + ori[1]) * mapstep_c;
                        for (ii=0; ii<min(8,nga-i); ii = ii+2) {
                            nx = (i + ii + ori[0]) * mapstep_c * mapstep_b;
                            mappos = nx + ny + nz;
                            ncube = 64*kk + 8*jj + ii;
                            FourierPt[mappos] = (float)(cube[ncube + 1] - rhoadd)/rhoscal;
                            if (FourierPt[mappos] > rhomax) rhomax = FourierPt[mappos];
                            if (FourierPt[mappos] < rhomin) rhomin = FourierPt[mappos];
                            if (ii+1 < min(8,nga-i)) {
                                FourierPt[mappos + 1] = 
                                    (float)(cube[ncube] - rhoadd)/rhoscal;
                                if (FourierPt[mappos + 1] > rhomax)
                                    rhomax = FourierPt[mappos + 1];
                                if (FourierPt[mappos + 1] < rhomin)
                                    rhomin = FourierPt[mappos + 1];
                            }
                        }
                    }
                }
            }
          }
      }
      if (!Quick) {
          fprintf(drvui->flout,"Reading Fourier map file %s\n"
                "cell = %f %f %f %f %f %f\n  map steps: a=%d b=%d c=%d\n  "
                "Range of rho values: %f to %f\n", infile, 
                map_a, map_b, map_c, map_alpha, map_beta, map_gamma, 
                mapstep_a, mapstep_b, mapstep_c, rhomin, rhomax);
      }
      Map_Info.rhomn = rhomin;
      Map_Info.rhomx = rhomax;
      ReadFourMap = 1;
      fclose(mapin);
}

/* ************************************************************** */
/* ************************************************************** */

void read_fcf(char * infile, int Quick)
{
// read CIF-standard Fo/Fc file and compute density

    FILE * mapin;
    float *fo,*fc,*f2,*ftmp1, *ftmp2, *ftmp3;
    short *ih,*ik,*il,*itmp1, *itmp2, *itmp3;
    float * Cklx;
    float * Dklx;
    float * Elxy;
    float * Flxy;
    double x,y,z;
    char string[200];
    char maptitle[61];
    int squared,phases,ab;
    int nval=200;
    int nr;
    int i,j,k,l;
    int ijk = 0;
    float rhomin=1.0e15f, rhomax=-1.0e15f;
    float factor;
    short ih0,ik0,il0;
    short ih1,ik1,il1;
    short im,in,io;
    float fo0,fc0,fc1,f20,f21;
    int kmin, kmax, nk;
    int lmin, lmax, nl;
    float phase;
    int skip;
    int progress;
    int dimension=3;

    if (FourierPt != NULL) {
        return;
    }

    if ((mapin = fopen (infile, "r")) == NULL) {
        sprintf(string,"Cannot open Fo/Fc (CIF fcf) file %s", infile);
        Error_Box(string);
        return;
    }

    fo=(float*)zalloc(200*sizeof(float));
    if (fo==NULL) {
        sprintf(string,"Unable to obtain storage for Fo data");
        Error_Box(string);
        return;
    }
    fc=(float*)zalloc(200*sizeof(float));
    if (fc==NULL) {
        sprintf(string,"Unable to obtain storage for Fc data");
        Error_Box(string);
        return;
    }
    f2=(float*)zalloc(200*sizeof(float));
    if (f2==NULL) {
        sprintf(string,"Unable to obtain storage for B or phase data");
        Error_Box(string);
        return;
    }
    ih=(short*)zalloc(200*sizeof(short));
    if (ih==NULL) {
        sprintf(string,"Unable to obtain storage for h data");
        Error_Box(string);
        return;
    }
    ik=(short*)zalloc(200*sizeof(short));
    if (ik==NULL) {
        sprintf(string,"Unable to obtain storage for k data");
        Error_Box(string);
        return;
    }
    il=(short*)zalloc(200*sizeof(short));
    if (il==NULL) {
        sprintf(string,"Unable to obtain storage for l data");
        Error_Box(string);
        return;
    }
 
    do {
      fgets(string,80,mapin);
      if (strstr(string,"title")) {
        memset(maptitle,0,61); 
        sscanf(string,"%*s %60c",maptitle); 
        trim_string(maptitle,60);
        strcpy(Map_Info.title,maptitle);
        Map_Info.info_valid = 1;
      }
    } while (strncmp(string," _refln",6) ); // search for start of _refln group

    if (strncmp(string," _refln_index_h",14)) {
      sprintf(string,"Unsupported field sequence in fcf file - need h k l first");
      Error_Box(string);
      return;
    }

    squared = 0; 
    phases = 0;
    ab = 0;
    while (!strncmp(string," _refln",6) ) {
      fgets(string,80,mapin);
      if (strstr(string,"squared")) squared=1;
      if (strstr(string,"_phase_")) phases=1;
      if (strstr(string,"A_calc")) ab = 1;
      if (strstr(string,"index_m")) dimension=4;
      if (strstr(string,"index_n")) dimension=5;
      if (strstr(string,"index_o")) dimension=6;
    }

    if (dimension >3) {
      sprintf(string,"%d-dimensional data encountered - only main reflections will be used",dimension);
      Error_Box(string);
    }

    if (phases == 0 && ab == 0) {
      sprintf(string,"fcf file does not contain phase angle or A,B data");
      Error_Box(string);
      return;
    }

    nr = 0;
    do {
      skip=0;
      if (dimension >3) {
          switch(dimension) {
           case 4:
             im=0;
             i=sscanf(string,"%hd %hd %hd %hd %f %f %f",&ih0,&ik0,&il0,&im,&fo0,&fc0,&f20);
             if (im != 0) skip=1;
             break;
           case 5:
             im=in=0;
             i=sscanf(string,"%hd %hd %hd %hd %hd %f %f %f",&ih0,&ik0,&il0,&im,&in,&fo0,&fc0,&f20);
             if (im != 0 || in != 0) skip=1;
             break;
           case 6:
             im=in=io=0;
             i=sscanf(string,"%hd %hd %hd %hd %hd %hd %f %f %f",
                              &ih0,&ik0,&il0,&im,&in,&io,&fo0,&fc0,&f20);
             if (im != 0 || in != 0 || io != 0) skip=1;
             break;
          }
      } else {
        i=sscanf(string,"%hd %hd %hd %f %*f %f %f",&ih0,&ik0,&il0,&fo0,&fc0,&f20);
      }
      if (i<dimension+3) break;
      if (skip == 1) {
          fgets(string,80,mapin);
          continue;
      }

      if (squared==1) fo0=(float)sqrt(fo0);
      if (ab) {       // change all data to 'phase' type
          fc1 = (float) sqrt(fc0 * fc0 + f20 * f20);
          f20 = (float)atan2(f20/fc1, fc0/fc1);
          fc0 = fc1;
      } else {
          f20 *= (float)(PI/180.0);     // phase in radians
      }

// apply spacegroup symmetry (including xyz - catch duplicates in input list)
      for (k = 0; k < drvui->ng; ++k) {
          skip=0;

          ih1=drvui->ss[k][0][0]*ih0 + drvui->ss[k][1][0]*ik0 + drvui->ss[k][2][0]*il0;
          ik1=drvui->ss[k][0][1]*ih0 + drvui->ss[k][1][1]*ik0 + drvui->ss[k][2][1]*il0;
          il1=drvui->ss[k][0][2]*ih0 + drvui->ss[k][1][2]*ik0 + drvui->ss[k][2][2]*il0;
          phase = 2.0f * (float)PI * (drvui->ts[k][0]*ih0 + drvui->ts[k][1]*ik0 
              + drvui->ts[k][2]*il0);
          f21 = f20 + phase;
          f21 = (float)atan2(sin(f21),cos(f21));    // get f21 in range -PI to PI
       
          for (j=0;j<nr;j++) {
              if (ih[j] == ih1 && ik[j] == ik1 && il[j] == il1) { 
                skip=1;
                break;
              }
          }
          if (skip==0) {
            ih[nr] = ih1;
            ik[nr] = ik1;
            il[nr] = il1;
            fo[nr] = fo0;
            fc[nr] = fc0;
            f2[nr] = f21;
            nr++;

          }
          ih1 *= -1;               // add Friedel opposite
          ik1 *= -1;
          il1 *= -1;
          f21 *= -1.0;
          skip = 0;

          for (j=0;j<nr;j++) {
              if (ih[j] == ih1 && ik[j] == ik1 && il[j] == il1) {
                skip=1;
                break;
              }
          }
          if (skip==0) {
            ih[nr]=ih1;
            ik[nr]=ik1;
            il[nr]=il1;
            fo[nr]=fo0;
            fc[nr] = fc0;
            f2[nr] = f21;
            nr++;
          }

          if (nr > nval-10) {
            nval +=100;
            ftmp1=(float*)realloc(fo,nval*sizeof(float));
            ftmp2=(float*)realloc(fc,nval*sizeof(float));
            ftmp3=(float*)realloc(f2,nval*sizeof(float));
            itmp1=(short*)realloc(ih,nval*sizeof(short));
            itmp2=(short*)realloc(ik,nval*sizeof(short));
            itmp3=(short*)realloc(il,nval*sizeof(short));
            if (!ftmp1 || !ftmp2 || !ftmp3 || !itmp1 || !itmp2 || !itmp3) {
              sprintf(string,"Unable to expand storage for h, k, l, fo, fc or phase data");
              free(fo);
              free(fc);
              free(f2);
              free(ih);
              free(ik);
              free(il);
              Error_Box(string);
              return;
            } else {
              fo = ftmp1;
              fc = ftmp2;
              f2 = ftmp3;
              ih = itmp1;
              ik = itmp2;
              il = itmp3;
            }
          } // if we need to realloc
      } // loop over all symmetry operators
      fgets(string,80,mapin);
    } while (i>0 && !feof(mapin));

    fclose(mapin);


    factor = 1.0f/(drvui->lat_con[0]*drvui->lat_con[1]*drvui->lat_con[2]
        * (float)sqrt(1.0 - cos(drvui->lat_con[3] * PI / 180.0) *
                     cos(drvui->lat_con[3] * PI / 180.0) *
                     cos(drvui->lat_con[4] * PI / 180.0) *
                     cos(drvui->lat_con[4] * PI / 180.0) *
                     cos(drvui->lat_con[5] * PI / 180.0) *
                     cos(drvui->lat_con[5] * PI / 180.0)));
    kmin = kmax = ik[0];
    lmin = lmax = il[0];

    for (l=0; l < nr; l++) {
        if (ik[l] < kmin) kmin = ik[l];     // find range of k and l
        if (ik[l] > kmax) kmax = ik[l];
        if (il[l] < lmin) lmin = il[l];
        if (il[l] > lmax) lmax = il[l];
        switch (Map_Info.map_type) {        // get map coefficients/V
        default:
        case 0:                             // Fo map
            break;
        case 1:                             // Fc map
            fo[l] = fc[l];
            break;
        case 2:                             // Fo - Fc map
            fo[l] = fo[l] - fc[l];
            break;
        case 3:                             // 2Fo - Fc map
            fo[l] = 2.0f * fo[l] - fc[l];
            break;
        case 4:                             // Fo2 (Patterson)
            fo[l] = fo[l] * fo[l];
            f2[l] = 0.0f;
            break;
        }
        fo[l] *= factor;                          // update coefficients times 1/V
    }

    nk = kmax - kmin +1;
    nl = lmax - lmin +1;
    Cklx = (float *)zalloc(nk * nl * sizeof(float));
    Dklx = (float *)zalloc(nk * nl * sizeof(float));
    Elxy = (float *)zalloc(nl * sizeof(float));
    Flxy = (float *)zalloc(nl * sizeof(float));

    if (!Cklx || !Dklx || !Elxy || !Flxy) {
        Error_Box("Unable to allocate Beevers-Lipson arrays.");
        return;
    }
    mapstep_a = (int) (4.0f * drvui->lat_con[0] + 0.5f);
    mapstep_b = (int) (4.0f * drvui->lat_con[1] + 0.5f);
    mapstep_c = (int) (4.0f * drvui->lat_con[2] + 0.5f);

    FourierPt = (float *) zalloc( mapstep_a*mapstep_b*mapstep_c * sizeof(float));
    if (FourierPt == NULL) {
      Error_Box("ERROR -- Unable to allocate space for map.");
      return;
    }

// start the Beevers-Lipson expansion

    Progress_Window(-1,"Computing Density",(float)mapstep_a*mapstep_b);
    
    progress=0;
    for(i=0; i < mapstep_a; i++) {
        x = (float)i/(float)mapstep_a;
        memset(Cklx, 0, nk * nl * sizeof(float));
        memset(Dklx, 0, nk * nl * sizeof(float));
        for (l=0; l<nr; l++) {
            ijk = (il[l] - lmin) * nk + ik[l] - kmin;
            Cklx[ijk] += fo[l] * (float)cos(2.0 * PI * ih[l] * x - f2[l]);
            Dklx[ijk] -= fo[l] * (float)sin(2.0 * PI * ih[l] * x - f2[l]);
        }
        for(j=0; j < mapstep_b; j++) {
            Progress_Window(0,NULL,(float)++progress);
            y = (float)j/(float)mapstep_b;
            memset(Elxy, 0, nl * sizeof(float));
            memset(Flxy, 0, nl * sizeof(float));
            for (l=0; l<nl; l++) {
                for (k=0; k<nk; k++) {
                    ijk = l * nk + k;
                    Elxy[l] += Cklx[ijk] * (float)cos(2.0 * PI * (k+kmin) * y)
                             + Dklx[ijk] * (float)sin(2.0 * PI * (k+kmin) * y);
                    Flxy[l] += Dklx[ijk] * (float)cos(2.0 * PI * (k+kmin) * y)
                             - Cklx[ijk] * (float)sin(2.0 * PI * (k+kmin) * y);
                }
            }
            for(k=0; k < mapstep_c; k++) {
              z = (float)k/(float)mapstep_c;
              ijk =  i*mapstep_b*mapstep_c + j*mapstep_c + k;

              for (l=0; l<nl; l++) {
                FourierPt[ijk] += Elxy[l] * (float)cos(2.0 * PI * (l + lmin) * z)
                                + Flxy[l] * (float)sin(2.0 * PI * (l + lmin) * z);
              }
              if (FourierPt[ijk] < rhomin) 
                  rhomin = FourierPt[ijk];
              if (FourierPt[ijk] > rhomax) 
                  rhomax = FourierPt[ijk];
            }
        }
    }
    if(!Quick)
        fprintf(drvui->fcns,"Reading Fourier map file %s\n  "
            "map steps: a=%d b=%d c=%d\n  "
            "Range of rho values: %f to %f, gridsize %d\n", infile, 
 	        mapstep_a, mapstep_b, mapstep_c, rhomin, rhomax, ijk);


    Progress_Window(-2, NULL, 0.0f);
    ReadFourMap = 1;
    Map_Info.rhomn = rhomin;
    Map_Info.rhomx = rhomax;
    Map_Info.map_int[0] = mapstep_a;
    Map_Info.map_int[1] = mapstep_b;
    Map_Info.map_int[2] = mapstep_c;
    Map_Info.lat_con[0] = drvui->lat_con[0];
    Map_Info.lat_con[1] = drvui->lat_con[1];
    Map_Info.lat_con[2] = drvui->lat_con[2];
    Map_Info.lat_con[3] = drvui->lat_con[3];
    Map_Info.lat_con[4] = drvui->lat_con[4];
    Map_Info.lat_con[5] = drvui->lat_con[5];
    Map_Info.xlim[0] = 0.0f;
    Map_Info.xlim[1] = 1.0f;
    Map_Info.ylim[0] = 0.0f;
    Map_Info.ylim[1] = 1.0f;
    Map_Info.zlim[0] = 0.0f;
    Map_Info.zlim[1] = 1.0f;
	
    free(Cklx);
    free(Dklx);
    free(Elxy);
    free(Flxy);
    free(fo);
    free(fc);
    free(f2);
    free(ih);
    free(ik);
    free(il);
}

/* ************************************************************** */
/* ************************************************************** */

void read_m80(char * infile, int Quick)
{
// read JANA .m80 Fo/Fc file and compute density

    FILE * mapin;
    float *fo,*fc,*f2,*ftmp1, *ftmp2, *ftmp3;
    short *ih,*ik,*il,*itmp1, *itmp2, *itmp3;
    float * Cklx;
    float * Dklx;
    float * Elxy;
    float * Flxy;
    double x,y,z;
    char string[200];
    int nval=200;
    int nr;
    int i,j,k,l;
    int ijk = 0;
    float rhomin,rhomax;
    float factor;
    short ih0,ik0,il0;
    short ih1,ik1,il1;
    short im,in,io,is;
    float fo0,fc0,fc1,f20,f21;
    int kmin, kmax, nk;
    int lmin, lmax, nl;
    float phase;
    int skip;
    int progress;
    float foj;
    int warnings;
    int dimension;

    if (FourierPt != NULL) {
        return;
    }

    if ((mapin = fopen (infile, "r")) == NULL) {
        sprintf(string,"Cannot open Fo/Fc (JANA m80) file %s", infile);
        Error_Box(string);
        return;
    }

    fo=(float*)zalloc(200*sizeof(float));
    if (fo==NULL) {
        sprintf(string,"Unable to obtain storage for Fo data");
        Error_Box(string);
        return;
    }
    fc=(float*)zalloc(200*sizeof(float));
    if (fc==NULL) {
        sprintf(string,"Unable to obtain storage for Fc data");
        Error_Box(string);
        return;
    }
    f2=(float*)zalloc(200*sizeof(float));
    if (f2==NULL) {
        sprintf(string,"Unable to obtain storage for B or phase data");
        Error_Box(string);
        return;
    }
    ih=(short*)zalloc(200*sizeof(short));
    if (ih==NULL) {
        sprintf(string,"Unable to obtain storage for h data");
        Error_Box(string);
        return;
    }
    ik=(short*)zalloc(200*sizeof(short));
    if (ik==NULL) {
        sprintf(string,"Unable to obtain storage for k data");
        Error_Box(string);
        return;
    }
    il=(short*)zalloc(200*sizeof(short));
    if (il==NULL) {
        sprintf(string,"Unable to obtain storage for l data");
        Error_Box(string);
        return;
    }

    if (fgets(string,199,mapin)) {
      i=sscanf(string,"%hd %hd %hd %hd %hd %hd %hd %f",&ih0,&ik0,&il0,&im,&in,&io,&is,&fo0);
      if (i > 4 ) Map_Info.info_valid = 1;
      dimension=i-2;
    } else {
      sprintf(string,"Error reading M80 file");
      Error_Box(string);
      return;
    }

    if (dimension <3) {
      sprintf(string,"Error reading M80 file");
      Error_Box(string);
      return;
    }

    nr = 0;
    warnings=0;    

    do {
      skip=0;
      if (dimension >3) {
          switch(dimension) {
           case 4:
             im=0;
             i=sscanf(string,"%hd %hd %hd %hd %*d %f %*f %*f %f %f",&ih0,&ik0,&il0,&im,&fo0,&fc0,&f20);
             if (im != 0) skip=1;
             break;
           case 5:
             im=in=0;
             i=sscanf(string,"%hd %hd %hd %hd %hd %*d %f %*f %*f %f %f",&ih0,&ik0,&il0,&im,&in,&fo0,&fc0,&f20);
             if (im != 0 || in != 0) skip=1;
             break;
           case 6:
             im=in=io=0;
             i=sscanf(string,"%hd %hd %hd %hd %hd %hd %*d %f %*f %*f %f %f",
                              &ih0,&ik0,&il0,&im,&in,&io,&fo0,&fc0,&f20);
             if (im != 0 || in != 0 || io != 0) skip=1;
             break;
          }
      } else {
        i=sscanf(string,"%hd %hd %hd %*d %f %*f %*f %f %f",&ih0,&ik0,&il0,&fo0,&fc0,&f20);
      }
        if (i<dimension+3) break;

        if (skip == 1) { // satellite reflection of modulated structure
          fgets(string,199,mapin);
          continue;
        }

      // change all data to 'phase' type
          fc1 = (float) sqrt(fc0 * fc0 + f20 * f20);
          if (fc1==0.) // happens with data generated by superflip
            f20=0.;
          else
            f20 = (float)atan2(f20/fc1, fc0/fc1);
          fc0 = fc1;

// apply spacegroup symmetry (including xyz - catch duplicates in input list)
      for (k = 0; k < drvui->ng; ++k) {
          skip=0;

          ih1=drvui->ss[k][0][0]*ih0 + drvui->ss[k][1][0]*ik0 + drvui->ss[k][2][0]*il0;
          ik1=drvui->ss[k][0][1]*ih0 + drvui->ss[k][1][1]*ik0 + drvui->ss[k][2][1]*il0;
          il1=drvui->ss[k][0][2]*ih0 + drvui->ss[k][1][2]*ik0 + drvui->ss[k][2][2]*il0;
          phase = 2.0f * (float)PI * (drvui->ts[k][0]*ih0 + drvui->ts[k][1]*ik0 
              + drvui->ts[k][2]*il0);
          f21 = f20 + phase;
          f21 = (float)atan2(sin(f21),cos(f21));    // get f21 in range -PI to PI
       
          for (j=0;j<nr;j++) {
              if (ih[j] == ih1 && ik[j] == ik1 && il[j] == il1) { 
                skip=1;
                foj=fo[j];
                break;
              }
          }
          if (skip==0) {
            ih[nr] = ih1;
            ik[nr] = ik1;
            il[nr] = il1;
            fo[nr] = fo0;
            fc[nr] = fc0;
            f2[nr] = f21;
            nr++;
          } else { 
            if (fabs (foj-fo0)> 1.e-3) { 
              if (!Quick && warnings<100)
                fprintf(drvui->flout,"Warning - different Fo for %d %d %d and %d %d %d\n",
                                      ih0,ik0,il0,ih1,ik1,il1);
            warnings++;
            }
          }

          ih1 *= -1;               // add Friedel opposite
          ik1 *= -1;
          il1 *= -1;
          f21 *= -1.0;
          skip = 0;

          for (j=0;j<nr;j++) {
              if (ih[j] == ih1 && ik[j] == ik1 && il[j] == il1) {
                skip=1;
                foj=fo[j];
                break;
              }
          }
          if (skip==0) {
            ih[nr]=ih1;
            ik[nr]=ik1;
            il[nr]=il1;
            fo[nr]=fo0;
            fc[nr] = fc0;
            f2[nr] = f21;
            nr++;
          } else { 
            if (fabs (foj-fo0)> 1.e-3) { 
              if (!Quick && warnings<100) 
              fprintf(drvui->flout,"Warning - different Fo for %d %d %d and %d %d %d\n",
                                    ih0,ik0,il0,ih1,ik1,il1);
              warnings++;
            }
	  }

          if (nr > nval-10) {
            nval +=100;
            ftmp1=(float*)realloc(fo,nval*sizeof(float));
            ftmp2=(float*)realloc(fc,nval*sizeof(float));
            ftmp3=(float*)realloc(f2,nval*sizeof(float));
            itmp1=(short*)realloc(ih,nval*sizeof(short));
            itmp2=(short*)realloc(ik,nval*sizeof(short));
            itmp3=(short*)realloc(il,nval*sizeof(short));
            if (!ftmp1 || !ftmp2 || !ftmp3 || !itmp1 || !itmp2 || !itmp3) {
              sprintf(string,"Unable to expand storage for h, k, l, fo, fc or phase data");
              free(fo);
              free(fc);
              free(f2);
              free(ih);
              free(ik);
              free(il);
              Error_Box(string);
              return;
            } else {
              fo = ftmp1;
              fc = ftmp2;
              f2 = ftmp3;
              ih = itmp1;
              ik = itmp2;
              il = itmp3;
            }
          } // if we need to realloc
      } // loop over all symmetry operators
      fgets(string,199,mapin);
    } while (i>0 && !feof(mapin));

    fclose(mapin);

    if (dimension >3) {
              sprintf(string,"%d-dimensional data encountered - only main reflections will be used",dimension);
              Error_Box(string); 
    }
    if (warnings >0) {
              sprintf(string,"Symmetry problem - %d mismatches in symmetry-equivalent Fo values\n",warnings);
              Error_Box(string);
    }
    factor = 1.0f/(drvui->lat_con[0]*drvui->lat_con[1]*drvui->lat_con[2]
        * (float)sqrt(1.0 - cos(drvui->lat_con[3] * PI / 180.0) *
                     cos(drvui->lat_con[3] * PI / 180.0) *
                     cos(drvui->lat_con[4] * PI / 180.0) *
                     cos(drvui->lat_con[4] * PI / 180.0) *
                     cos(drvui->lat_con[5] * PI / 180.0) *
                     cos(drvui->lat_con[5] * PI / 180.0)));
    kmin = kmax = ik[0];
    lmin = lmax = il[0];

    for (l=0; l < nr; l++) {
        if (ik[l] < kmin) kmin = ik[l];     // find range of k and l
        if (ik[l] > kmax) kmax = ik[l];
        if (il[l] < lmin) lmin = il[l];
        if (il[l] > lmax) lmax = il[l];
        switch (Map_Info.map_type) {        // get map coefficients/V
        default:
        case 0:                             // Fo map
            break;
        case 1:                             // Fc map
            fo[l] = fc[l];
            break;
        case 2:                             // Fo - Fc map
            fo[l] = fo[l] - fc[l];
            break;
        case 3:                             // 2Fo - Fc map
            fo[l] = 2.0f * fo[l] - fc[l];
            break;
        case 4:                             // Fo2 (Patterson)
            fo[l] = fo[l] * fo[l];
            f2[l] = 0.0f;
            break;
        }
        fo[l] *= factor;                          // update coefficients times 1/V
    }

    nk = kmax - kmin +1;
    nl = lmax - lmin +1;
    Cklx = (float *)zalloc(nk * nl * sizeof(float));
    Dklx = (float *)zalloc(nk * nl * sizeof(float));
    Elxy = (float *)zalloc(nl * sizeof(float));
    Flxy = (float *)zalloc(nl * sizeof(float));

    if (!Cklx || !Dklx || !Elxy || !Flxy) {
        Error_Box("Unable to allocate Beevers-Lipson arrays.");
        return;
    }
    mapstep_a = (int) (4.0f * drvui->lat_con[0] + 0.5f);
    mapstep_b = (int) (4.0f * drvui->lat_con[1] + 0.5f);
    mapstep_c = (int) (4.0f * drvui->lat_con[2] + 0.5f);

    FourierPt = (float *) zalloc( mapstep_a*mapstep_b*mapstep_c * sizeof(float));
    if (FourierPt == NULL) {
      Error_Box("ERROR -- Unable to allocate space for map.");
      return;
    }

    rhomin=1.0e15f, rhomax=-1.0e15f;

// start the Beevers-Lipson expansion

    Progress_Window(-1,"Computing Density",(float)mapstep_a*mapstep_b);
    
    progress=0;
    for(i=0; i < mapstep_a; i++) {
        x = (float)i/(float)mapstep_a;
        memset(Cklx, 0, nk * nl * sizeof(float));
        memset(Dklx, 0, nk * nl * sizeof(float));
        for (l=0; l<nr; l++) {
            ijk = (il[l] - lmin) * nk + ik[l] - kmin;
            Cklx[ijk] += fo[l] * (float)cos(2.0 * PI * ih[l] * x - f2[l]);
            Dklx[ijk] -= fo[l] * (float)sin(2.0 * PI * ih[l] * x - f2[l]);
        }
        for(j=0; j < mapstep_b; j++) {
            Progress_Window(0,NULL,(float)++progress);
            y = (float)j/(float)mapstep_b;
            memset(Elxy, 0, nl * sizeof(float));
            memset(Flxy, 0, nl * sizeof(float));
            for (l=0; l<nl; l++) {
                for (k=0; k<nk; k++) {
                    ijk = l * nk + k;
                    Elxy[l] += Cklx[ijk] * (float)cos(2.0 * PI * (k+kmin) * y)
                             + Dklx[ijk] * (float)sin(2.0 * PI * (k+kmin) * y);
                    Flxy[l] += Dklx[ijk] * (float)cos(2.0 * PI * (k+kmin) * y)
                             - Cklx[ijk] * (float)sin(2.0 * PI * (k+kmin) * y);
                }
            }
            for(k=0; k < mapstep_c; k++) {
              z = (float)k/(float)mapstep_c;
              ijk =  i*mapstep_b*mapstep_c + j*mapstep_c + k;

              for (l=0; l<nl; l++) {
                FourierPt[ijk] += Elxy[l] * (float)cos(2.0 * PI * (l + lmin) * z)
                                + Flxy[l] * (float)sin(2.0 * PI * (l + lmin) * z);
              }
              if (FourierPt[ijk] < rhomin) 
                  rhomin = FourierPt[ijk];
              if (FourierPt[ijk] > rhomax) 
                  rhomax = FourierPt[ijk];
            }
        }
    }
    if(!Quick)
        fprintf(drvui->fcns,"Reading Fourier map file %s\n  "
            "map steps: a=%d b=%d c=%d\n  "
            "Range of rho values: %f to %f, gridsize %d\n", infile, 
 	        mapstep_a, mapstep_b, mapstep_c, rhomin, rhomax, ijk);


    Progress_Window(-2, NULL, 0.0f);
    ReadFourMap = 1;
    Map_Info.rhomn = rhomin;
    Map_Info.rhomx = rhomax;
    Map_Info.map_int[0] = mapstep_a;
    Map_Info.map_int[1] = mapstep_b;
    Map_Info.map_int[2] = mapstep_c;
    Map_Info.lat_con[0] = drvui->lat_con[0];
    Map_Info.lat_con[1] = drvui->lat_con[1];
    Map_Info.lat_con[2] = drvui->lat_con[2];
    Map_Info.lat_con[3] = drvui->lat_con[3];
    Map_Info.lat_con[4] = drvui->lat_con[4];
    Map_Info.lat_con[5] = drvui->lat_con[5];
    Map_Info.xlim[0] = 0.0f;
    Map_Info.xlim[1] = 1.0f;
    Map_Info.ylim[0] = 0.0f;
    Map_Info.ylim[1] = 1.0f;
    Map_Info.zlim[0] = 0.0f;
    Map_Info.zlim[1] = 1.0f;
	
    free(Cklx);
    free(Dklx);
    free(Elxy);
    free(Flxy);
    free(fo);
    free(fc);
    free(f2);
    free(ih);
    free(ik);
    free(il);
}

/* ************************************************************** */
/* ************************************************************** */

void read_flp(char * infile, int Quick)
{
// read FullProf binary map file

#include "read_flp.h"                // define all the struct's

    char titulo[81];
    char version[6];
    struct crystal_cell_type celda;
    FILE * mapin;
    float rhomin=1.0e+15f, rhomax=-1.0e+15f;
    int ireslt = 0;
    char string[200];
    int  icount;
    struct space_group spg;
    char temp[16384];
    int i, j, k, ijk=0;
    int natoms;
    struct FFT_param map_param;
    struct Atom_Type atm[200];
    float dens[200];
    int new_mapstep_a, astart, aend;
    int new_mapstep_b, bstart, bend;
    int new_mapstep_c, cstart, cend;
    int wrong_end = 0;

    memset(version, 0, 6);
    memset(titulo, 0, 81);
    if ((mapin = fopen (infile, "r+b")) == NULL) {
        sprintf(string,"Cannot open Fourier map (FulProf) file %s", infile);
        Error_Box(string);
        return;
    }
    ireslt = fread(&icount, sizeof(int), 1, mapin);         // read record length
	if (!ireslt) {
          char string[200];
          sprintf(string, "Error reading Fourier map file %s",infile);
          Error_Box(string);
          fclose(mapin);
    }
    if (icount != 80) {
        icount = end_flip(icount);
        wrong_end = 1;
    }
    if (icount != 80) {
        Error_Box("Map does not have the correct format\nJob aborted.");
        return;
    }
    fread(titulo, sizeof(char), icount, mapin);    // read title
    fread(&icount, sizeof(int), 1, mapin);         // skip trailer
    trim_string(titulo,80);
    strcpy(Map_Info.title,titulo);

    fread(&icount, sizeof(int), 1, mapin);         // read header for next record
    if (wrong_end) icount = end_flip(icount);
    fread(version, sizeof(char), icount, mapin);
    fread(&icount, sizeof(int), 1, mapin);         // skip trailer

    fread(&icount, sizeof(int), 1, mapin);         // read header for next record
    if (wrong_end) icount = end_flip(icount);
    fread(&celda, sizeof(char), icount, mapin);
    if (wrong_end) {
        for (i=0; i<3;i++) {
            celda.ang[i] = end_flip_real(celda.ang[i]);
            celda.cell[i] = end_flip_real(celda.cell[i]);
        }
    }
    fread(&icount, sizeof(int), 1, mapin);         // skip trailer
    for (i=0;i<3;i++) {
        Map_Info.lat_con[i] = celda.cell[i];
        Map_Info.lat_con[i+3] = celda.ang[i];
    }

    fread(&icount, sizeof(int), 1, mapin);         // read header for next record
    if (wrong_end) icount = end_flip(icount);
    fread(&spg, sizeof(char), icount, mapin);
    fread(&icount, sizeof(int), 1, mapin);         // skip trailer

    fread(&icount, sizeof(int), 1, mapin);         // read header for next record
    if (wrong_end) icount = end_flip(icount);
    fread(&natoms, sizeof(char), icount, mapin);
    fread(&icount, sizeof(int), 1, mapin);         // skip trailer

    fread(&icount, sizeof(int), 1, mapin);         // read header for next record
    if (wrong_end) icount = end_flip(icount);
    fread(&atm, sizeof(char), icount, mapin);
    fread(&icount, sizeof(int), 1, mapin);         // skip trailer

	for (i=0; i<2; i++) {                          // skip next 2 records
        fread(&icount, sizeof(int), 1, mapin);     // read header for next record
        if (wrong_end) icount = end_flip(icount);
        fread(&temp, sizeof(char), icount, mapin);
        fread(&icount, sizeof(int), 1, mapin);     // skip trailer
	}

    fread(&icount, sizeof(int), 1, mapin);         // read header for next record
    if (wrong_end) icount = end_flip(icount);
    fread(&map_param, sizeof(char), icount, mapin);
    fread(&icount, sizeof(int), 1, mapin);         // skip trailer
    if (wrong_end) {
        for (i=0; i<3; i++) {
            map_param.ngrid[i] = end_flip(map_param.ngrid[i]);
        }
    }
    Map_Info.map_int[0] = mapstep_a = map_param.ngrid[0];
    Map_Info.map_int[1] = mapstep_b = map_param.ngrid[1];
    Map_Info.map_int[2] = mapstep_c = map_param.ngrid[2];
    for (i=0; i<2; i++) {
        if (wrong_end) {
            map_param.xlim[i] = end_flip_real(map_param.xlim[i]);
            map_param.ylim[i] = end_flip_real(map_param.ylim[i]);
            map_param.zlim[i] = end_flip_real(map_param.zlim[i]);
        }
        Map_Info.xlim[i] = map_param.xlim[i];
        Map_Info.ylim[i] = map_param.ylim[i];
        Map_Info.zlim[i] = map_param.zlim[i];
    }
    Map_Info.info_valid = 1;

    fread(&icount, sizeof(int), 1, mapin);         // read header for next record
    if (wrong_end) icount = end_flip(icount);
    fread(&temp, sizeof(char), icount, mapin);
    fread(&icount, sizeof(int), 1, mapin);         // skip trailer

	if ((fabs(Map_Info.xlim[0]) > 0.001) || (fabs(Map_Info.xlim[1] - 1.0f) > 0.001)) {
// not full cell along x
		new_mapstep_a = (int)(1.0f/(Map_Info.xlim[1] - Map_Info.xlim[0])+0.5f) * mapstep_a;
		astart = (int)(Map_Info.xlim[0] * new_mapstep_a);
		aend = (int)(Map_Info.xlim[1] * new_mapstep_a);
    } else {
		new_mapstep_a = mapstep_a;
		astart = 0;
		aend = mapstep_a;
	}
	if ((fabs(Map_Info.ylim[0]) > 0.001) || (fabs(Map_Info.ylim[1] - 1.0f) > 0.001)) {
// not full cell along y
		new_mapstep_b = (int)(1.0f/(Map_Info.ylim[1] - Map_Info.ylim[0])+0.5f) * mapstep_b;
		bstart = (int)(Map_Info.ylim[0] * new_mapstep_b);
		bend = (int)(Map_Info.ylim[1] * new_mapstep_b);
    } else {
		new_mapstep_b = mapstep_b;
		bstart = 0;
		bend = mapstep_b;
	}
	if ((fabs(Map_Info.zlim[0]) > 0.001) || (fabs(Map_Info.zlim[1] - 1.0f) > 0.001)) {
// not full cell along z
		new_mapstep_c = (int)(1.0f/(Map_Info.zlim[1] - Map_Info.zlim[0])+0.5f) * mapstep_c;
		cstart = (int)(Map_Info.zlim[0] * new_mapstep_c);
		cend = (int)(Map_Info.zlim[1] * new_mapstep_c);
    } else {
		new_mapstep_c = mapstep_c;
		cstart = 0;
		cend = mapstep_c;
	}
    if (FourierPt) free(FourierPt);
    FourierPt = (float *) zalloc( new_mapstep_a*new_mapstep_b*new_mapstep_c * sizeof(float));
    if (FourierPt == NULL) {
        ireslt = 0;
        Error_Box("ERROR -- Unable to allocate space for map.");
        return;
    }

// read the Rho values

     for (i=astart; i< aend; i++) {
        for (j=bstart; j<bend; j++) {
            fread(&icount, sizeof(int), 1, mapin); // read header for next record
            if (wrong_end) icount = end_flip(icount);
            fread(&dens, sizeof(char), icount, mapin);
            fread(&icount, sizeof(int), 1, mapin); // skip trailer
            for (k=cstart; k<cend; k++) {
				ijk = i * new_mapstep_c * new_mapstep_b + j * new_mapstep_c + k;
                if (wrong_end) dens[k] = end_flip_real(dens[k]);
                FourierPt[ijk] = dens[k];
                if (FourierPt[ijk] < rhomin) rhomin = FourierPt[ijk];
                if (FourierPt[ijk] > rhomax) rhomax = FourierPt[ijk];
            }
        }
    }
    fclose(mapin);
    if(!Quick)
        fprintf(drvui->fcns,"Reading Fourier map file %s\n  "
            "cell = %f %f %f %f %f %f\n  map steps: a=%d b=%d c=%d\n  "
            "Range of rho values: %f to %f, gridsize %d\n", infile, 
 	        celda.cell[0], celda.cell[1], celda.cell[2], 
            celda.ang[0], celda.ang[1], celda.ang[2], 
 	        mapstep_a, mapstep_b, mapstep_c, rhomin, rhomax, ijk);
    ReadFourMap = 1;
	Map_Info.rhomn = rhomin;
	Map_Info.rhomx = rhomax;
    mapstep_a = new_mapstep_a;
    mapstep_b = new_mapstep_b;
    mapstep_c = new_mapstep_c;
	Map_Info.map_int[0] = mapstep_a;
	Map_Info.map_int[1] = mapstep_b;
	Map_Info.map_int[2] = mapstep_c;
}

/* ************************************************************** */
/* ************************************************************** */

void read_grd(char * infile, int Quick)
{
      FILE * mapin;
      char line[81];
      char * reslt;
      float rhomin=1.0e15f, rhomax=-1.0e15f;
      int ireslt = 0;
      char string[200];
      int new_mapstep_a = 0, astart, aend;
	  int new_mapstep_b = 0, bstart, bend;
	  int new_mapstep_c = 0, cstart, cend;

      if ((mapin = fopen (infile, "r")) == NULL) {
          sprintf(string,"Cannot open Fourier map (.grd) file %s", infile);
        Error_Box(string);
        return;
      }
      reslt = fgets(line, sizeof(line), mapin);
      trim_string(line,81);
      strcpy(Map_Info.title,line);
      Map_Info.info_valid = 1;
      if (reslt != NULL) {
        ireslt = fscanf(mapin, "%f %f %f %f %f %f", 
 			&map_a, &map_b, &map_c, &map_alpha, &map_beta, &map_gamma);
        Map_Info.lat_con[0] = map_a;
        Map_Info.lat_con[1] = map_b;
        Map_Info.lat_con[2] = map_c;
        Map_Info.lat_con[3] = map_alpha;
        Map_Info.lat_con[4] = map_beta;
        Map_Info.lat_con[5] = map_gamma;
      }
      if (ireslt > 0)
        ireslt = fscanf(mapin, "%d %d %d", 
 			&mapstep_a, &mapstep_b, &mapstep_c);
      if (ireslt > 0) {
        int i, j , k, ijk;
        if ((fabs(Map_Info.xlim[0]) > 0.001) || (fabs(Map_Info.xlim[1] - 1.0f) > 0.001)) {
// not full cell along x
            new_mapstep_a = (int)(1.0f/(Map_Info.xlim[1] - Map_Info.xlim[0])+0.5f) * mapstep_a;
            astart = (int)(Map_Info.xlim[0] * new_mapstep_a);
            aend = (int)(Map_Info.xlim[1] * new_mapstep_a);
        } else {
            new_mapstep_a = mapstep_a;
            astart = 0;
            aend = mapstep_a;
        }
        if ((fabs(Map_Info.ylim[0]) > 0.001) || (fabs(Map_Info.ylim[1] - 1.0f) > 0.001)) {
// not full cell along y
            new_mapstep_b = (int)(1.0f/(Map_Info.ylim[1] - Map_Info.ylim[0])+0.5f) * mapstep_b;
            bstart = (int)(Map_Info.ylim[0] * new_mapstep_b);
            bend = (int)(Map_Info.ylim[1] * new_mapstep_b);
        } else {
            new_mapstep_b = mapstep_b;
            bstart = 0;
            bend = mapstep_b;
        }
        if ((fabs(Map_Info.zlim[0]) > 0.001) || (fabs(Map_Info.zlim[1] - 1.0f) > 0.001)) {
// not full cell along z
            new_mapstep_c = (int)(1.0f/(Map_Info.zlim[1] - Map_Info.zlim[0])+0.5f) * mapstep_c;
            cstart = (int)(Map_Info.zlim[0] * new_mapstep_c);
            cend = (int)(Map_Info.zlim[1] * new_mapstep_c);
        } else {
            new_mapstep_c = mapstep_c;
            cstart = 0;
            cend = mapstep_c;
        }

        if (FourierPt) free(FourierPt);

        FourierPt = (float*)zalloc( new_mapstep_a*new_mapstep_b*new_mapstep_c * sizeof(float));
        if (FourierPt == NULL) {
          ireslt = 0;
          Error_Box("ERROR -- Unable to allocate space for map.\n");
          return;
        }

/* read the Rho values */
        for(i=astart; i < aend; i++) {
          for(j=bstart; j < bend; j++) {
            for(k=cstart; k < cend; k++) {
              ijk =  i*new_mapstep_b*new_mapstep_c + j*new_mapstep_c + k;
              ireslt = fscanf(mapin,"%f", &FourierPt[ijk]);
              if (FourierPt[ijk] < rhomin) rhomin = FourierPt[ijk];
              if (FourierPt[ijk] > rhomax) rhomax = FourierPt[ijk];
            }
          }
        }
      }
      Map_Info.rhomn = rhomin;
      Map_Info.rhomx = rhomax;
      Map_Info.map_int[0] = mapstep_a = new_mapstep_a;
      Map_Info.map_int[1] = mapstep_b = new_mapstep_b;
      Map_Info.map_int[2] = mapstep_c = new_mapstep_c;
       
      if (ireslt > 0) {
          if (!Quick) {
              fprintf(drvui->flout,"Reading Fourier map file %s\n\tTitle= %s\n  "
                    "cell = %f %f %f %f %f %f\n  map steps: a=%d b=%d c=%d\n  "
                    "Range of rho values: %f to %f\n", infile, line, 
 	                 map_a, map_b, map_c, map_alpha, map_beta, map_gamma, 
 	                 mapstep_a, mapstep_b, mapstep_c, rhomin, rhomax);
          }
          ReadFourMap = 1;
      } else {
          char string[200];
          sprintf(string, "Error reading Fourier map file %s",infile);
          Error_Box(string);
      }
      fclose(mapin);
}

/* ************************************************************** */
/* ************************************************************** */

void read_stf(char * infile, int Quick)
{
      FILE * mapin;
      char line[81];
      char * reslt;
      float rhomin=1.0e15f, rhomax=-1.0e15f;
      int ireslt = 0;
      int ijk = 0;
      int i,j,k;
      float bx1,bx2,by1,by2,bz1,bz2;
      char string[200];
	  int new_mapstep_a, astart, aend;
	  int new_mapstep_b, bstart, bend;
	  int new_mapstep_c, cstart, cend;
            
      if ((mapin = fopen (infile, "r")) == NULL) {
        sprintf (string,
           "Cannot open Fourier map (.stf) file %s\n", infile);
        Error_Box(string);
        return;
      }
      reslt = fgets(line, sizeof(line), mapin);
      reslt = fgets(line, sizeof(line), mapin);
      reslt = fgets(line, sizeof(line), mapin);
      if (reslt){
        ireslt = sscanf(line, "%*s %d %d %d", 
 			&mapstep_a, &mapstep_b, &mapstep_c);
      }		
      if (ireslt > 0) {
        reslt = fgets(line, sizeof(line), mapin);
        sscanf (line,"%*s %f %f %f %f %f %f",&bx1,&bx2,&by1,&by2,&bz1,&bz2);
        Map_Info.xlim[0] = bx1;
		Map_Info.xlim[1] = bx2;
        Map_Info.ylim[0] = by1;
		Map_Info.ylim[1] = by2;
        Map_Info.zlim[0] = bz1;
		Map_Info.zlim[1] = bz2;
        Map_Info.lat_con[0] = map_a = drvui->lat_con[0];
        Map_Info.lat_con[1] = map_b = drvui->lat_con[1];
        Map_Info.lat_con[2] = map_c = drvui->lat_con[2];
        Map_Info.lat_con[3] = map_alpha = drvui->lat_con[3];
        Map_Info.lat_con[4] = map_beta = drvui->lat_con[4];
        Map_Info.lat_con[5] = map_gamma = drvui->lat_con[5];

        if ((fabs(Map_Info.xlim[0]) > 0.001) || (fabs(Map_Info.xlim[1] - 1.0f) > 0.001)) {
// not full cell along x
            new_mapstep_a = (int)(1.0f/(Map_Info.xlim[1] - Map_Info.xlim[0])+0.5f) * mapstep_a;
            astart = (int)(Map_Info.xlim[0] * new_mapstep_a);
            aend = (int)(Map_Info.xlim[1] * new_mapstep_a);
        } else {
            new_mapstep_a = mapstep_a;
            astart = 0;
            aend = mapstep_a;
        }
        if ((fabs(Map_Info.ylim[0]) > 0.001) || (fabs(Map_Info.ylim[1] - 1.0f) > 0.001)) {
// not full cell along y
            new_mapstep_b = (int)(1.0f/(Map_Info.ylim[1] - Map_Info.ylim[0])+0.5f) * mapstep_b;
            bstart = (int)(Map_Info.ylim[0] * new_mapstep_b);
            bend = (int)(Map_Info.ylim[1] * new_mapstep_b);
        } else {
            new_mapstep_b = mapstep_b;
            bstart = 0;
            bend = mapstep_b;
        }
        if ((fabs(Map_Info.zlim[0]) > 0.001) || (fabs(Map_Info.zlim[1] - 1.0f) > 0.001)) {
// not full cell along z
            new_mapstep_c = (int)(1.0f/(Map_Info.zlim[1] - Map_Info.zlim[0])+0.5f) * mapstep_c;
            cstart = (int)(Map_Info.zlim[0] * new_mapstep_c);
            cend = (int)(Map_Info.zlim[1] * new_mapstep_c);
        } else {
            new_mapstep_c = mapstep_c;
            cstart = 0;
            cend = mapstep_c;
        }

        if (FourierPt) free(FourierPt);

        FourierPt = (float*)zalloc( new_mapstep_a*new_mapstep_b*new_mapstep_c * sizeof(float));
        if (FourierPt == NULL) {
          ireslt = 0;
          Error_Box("ERROR -- Unable to allocate space for map.\n");
          return;
        }
        reslt = fgets(line, sizeof(line), mapin);
        reslt = fgets(line, sizeof(line), mapin);
        reslt = fgets(line, sizeof(line), mapin);

          for(i=astart; i < aend; i++) {
            for(j=bstart; j < bend-1; j++) {
              for(k=cstart; k < cend-1; k++) {
                ijk =  k*(new_mapstep_a-1)*(new_mapstep_b-1) + j*(new_mapstep_a-1) + i;
                ireslt = fscanf(mapin,"%f", &FourierPt[ijk]);
                if (FourierPt[ijk] < rhomin) rhomin = FourierPt[ijk];
                if (FourierPt[ijk] > rhomax) rhomax = FourierPt[ijk];
              }
              fscanf(mapin,"%*f");
            }
            for (k=0;k<mapstep_c;k++)fscanf(mapin,"%*f");
          }

        mapstep_a = --new_mapstep_a;
        mapstep_b = --new_mapstep_b;
        mapstep_c = --new_mapstep_c;
		Map_Info.map_int[0] = mapstep_a;
		Map_Info.map_int[1] = mapstep_b;
		Map_Info.map_int[2] = mapstep_c;
        Map_Info.rhomn = rhomin;
        Map_Info.rhomx = rhomax;
        Map_Info.info_valid = 1;

      } /* if mapstep line present */  
       
      if (ijk > 0) {
        if (!Quick)
          fprintf(drvui->flout,"Reading Fourier map file %s\n  "
            "cell = %f %f %f %f %f %f\n  map steps: a=%d b=%d c=%d\n  "
            "Range of rho values: %f to %f, gridsize %d\n", infile, 
 	        map_a, map_b, map_c, map_alpha, map_beta, map_gamma, 
 	        mapstep_a, mapstep_b, mapstep_c, rhomin, rhomax,ijk);
        ReadFourMap = 1;
      } else {
        sprintf(string, "Error reading Fourier map file %s", 
 	       infile);
        Error_Box(string);
      }
      fclose(mapin);
}

/* ************************************************************** */
/* ************************************************************** */

void read_vasp(char * infile, int Quick)
{
      FILE * mapin;
      char line[200];
      char * reslt;
      float rhomin=1.0e15f, rhomax=-1.0e15f;
      int ireslt = 0;
      int ijk = 0;
      int i,j,k,l,m;
	  float tmp_rho[10];
      char string[200];
      float scale;
      float cmat[3][3];
      int new_mapstep_a = 0, astart, aend;
      int new_mapstep_b = 0, bstart, bend;
      int new_mapstep_c = 0, cstart, cend;
            
      map_a = drvui->lat_con[0];
      map_b = drvui->lat_con[1];
      map_c = drvui->lat_con[2];
      map_alpha = drvui->lat_con[3];
      map_beta = drvui->lat_con[4];
      map_gamma = drvui->lat_con[5];
	  if ((mapin = fopen (infile, "r")) == NULL) {
        sprintf (string,
           "Cannot open VASP file %s\n", infile);
        Error_Box(string);
        return;
      }
	  fgets(line, sizeof(line), mapin);
      trim_string(line,80);
      strcpy(Map_Info.title,line);
	  fgets(line, sizeof(line), mapin);
      sscanf(line, "%f", &scale);
	  for(i=0; i<3; i++) {   // read the cell matrix
		  if (!(reslt = fgets(line, sizeof(line), mapin))) {
			  sprintf(string,"Error reading VASP file.");
			  Error_Box(string);
			  return;
		  }
          sscanf(line,"%f %f %f",&cmat[i][0],&cmat[i][1],&cmat[i][2]);
	  }
      for (i=0; i<3; i++)
          for (j=0; j<3; j++)
              cmat[i][j] *= scale;
	  fgets(line, sizeof(line), mapin);
      Map_Info.lat_con[0] = (float) sqrt(cmat[0][0]);   // this section needs fixing
      Map_Info.lat_con[1] = (float) sqrt(cmat[1][1]);
      Map_Info.lat_con[2] = (float) sqrt(cmat[2][2]);
      Map_Info.lat_con[3] = 90.0f;            
      Map_Info.lat_con[4] = 90.0f;
      Map_Info.lat_con[5] = 90.0f;
	  j = 0;
	  i = strlen(line)/4;
	  for (k=0; k<i; k++) {
		  sscanf(line + 4 * k,"%4d",&l);
		  j += l;
	  }
	  for(i=0; i<j+2; i++) {   // set to skip k-point stuff
		  if (!(reslt = fgets(line, sizeof(line), mapin))) {
			  sprintf(string,"Error reading VASP file.");
			  Error_Box(string);
			  return;
		  }
	  }
	  fgets(line, sizeof(line), mapin);
      ireslt = sscanf(line, "%d %d %d", 
 			&mapstep_a, &mapstep_b, &mapstep_c);
      if (ireslt > 0) {
        if ((fabs(Map_Info.xlim[0]) > 0.001) || (fabs(Map_Info.xlim[1] - 1.0f) > 0.001)) {
// not full cell along x
            new_mapstep_a = (int)(1.0f/(Map_Info.xlim[1] - Map_Info.xlim[0])+0.5f) * mapstep_a;
            astart = (int)(Map_Info.xlim[0] * new_mapstep_a);
            aend = (int)(Map_Info.xlim[1] * new_mapstep_a);
        } else {
            new_mapstep_a = mapstep_a;
            astart = 0;
            aend = mapstep_a;
        }
        if ((fabs(Map_Info.ylim[0]) > 0.001) || (fabs(Map_Info.ylim[1] - 1.0f) > 0.001)) {
// not full cell along y
            new_mapstep_b = (int)(1.0f/(Map_Info.ylim[1] - Map_Info.ylim[0])+0.5f) * mapstep_b;
            bstart = (int)(Map_Info.ylim[0] * new_mapstep_b);
            bend = (int)(Map_Info.ylim[1] * new_mapstep_b);
        } else {
            new_mapstep_b = mapstep_b;
            bstart = 0;
            bend = mapstep_b;
        }
        if ((fabs(Map_Info.zlim[0]) > 0.001) || (fabs(Map_Info.zlim[1] - 1.0f) > 0.001)) {
// not full cell along z
            new_mapstep_c = (int)(1.0f/(Map_Info.zlim[1] - Map_Info.zlim[0])+0.5f) * mapstep_c;
            cstart = (int)(Map_Info.zlim[0] * new_mapstep_c);
            cend = (int)(Map_Info.zlim[1] * new_mapstep_c);
        } else {
            new_mapstep_c = mapstep_c;
            cstart = 0;
            cend = mapstep_c;
        }

        if (FourierPt) free(FourierPt);

        FourierPt = (float*)zalloc( new_mapstep_a*new_mapstep_b*new_mapstep_c * sizeof(float));
        if (FourierPt == NULL) {
          ireslt = 0;
          Error_Box("ERROR -- Unable to allocate space for map.\n");
          return;
        }

// read the Rho values

		m = 11;
        for(i=cstart; i < cend; i++) {
          for(j=bstart; j < bend; j++) {
            for(k=astart; k < aend; k++) {
			    if (m > 9) {
					reslt = fgets(line, sizeof(line), mapin);
					sscanf(line,"%f %f %f %f %f %f %f %f %f %f",&tmp_rho[0],
						&tmp_rho[1],&tmp_rho[2],&tmp_rho[3],&tmp_rho[4],
						&tmp_rho[5],&tmp_rho[6],&tmp_rho[7],&tmp_rho[8],
						&tmp_rho[9]);
					m = 0;
				}
                ijk =  k * new_mapstep_b * new_mapstep_c + j * new_mapstep_c + i;
				FourierPt[ijk] = tmp_rho[m++];
                if (FourierPt[ijk] < rhomin) rhomin = FourierPt[ijk];
                if (FourierPt[ijk] > rhomax) rhomax = FourierPt[ijk];
            }
          }
        }
      }
       
      if (reslt) {
          ReadFourMap = 1;
      } else {
          char string[200];
          sprintf(string, "Error reading Fourier map file %s",infile);
          Error_Box(string);
      }
      Map_Info.rhomn = rhomin;
      Map_Info.rhomx = rhomax;
      Map_Info.map_int[0] = mapstep_a = new_mapstep_a;
      Map_Info.map_int[1] = mapstep_b = new_mapstep_b;
      Map_Info.map_int[2] = mapstep_c = new_mapstep_c;
	  Map_Info.info_valid = 1;
      fclose(mapin);

}

/* ************************************************************** */
/* ************************************************************** */

void read_w2k(char * infile, int Quick)
{
      FILE * mapin;
      char line[81];
      char * reslt;
      float rhomin=1.0e15f, rhomax=-1.0e15f;
      int ireslt = 0;
      int dimen=0;
      int ijk = 0;
      int i,j,k;
      char string[200];
      int new_mapstep_a = 0, astart, aend;
      int new_mapstep_b = 0, bstart, bend;
      int new_mapstep_c = 0, cstart, cend;

      Map_Info.lat_con[0] = map_a = drvui->lat_con[0];
      Map_Info.lat_con[1] = map_b=drvui->lat_con[1];
      Map_Info.lat_con[2] = map_c=drvui->lat_con[2];
      Map_Info.lat_con[3] = map_alpha=drvui->lat_con[3];
      Map_Info.lat_con[4] = map_beta=drvui->lat_con[4];
      Map_Info.lat_con[5] = map_gamma=drvui->lat_con[5];
            
      if ((mapin = fopen (infile, "r")) == NULL) {
        sprintf (string,
           "Cannot open Fourier map (.w2k) file %s\n", infile);
        Error_Box(string);
        return;
      }

      do {
        reslt = fgets(line, sizeof(line), mapin);
        if (!strncmp(line,"(@3 [3]",7)) dimen=1;
      } while (!feof(mapin) && dimen==0);
 
      if (dimen==0) {
        sprintf (string,
           "Cannot read Fourier map (.w2k) file %s\n", infile);
        Error_Box(string);
        return;
      }

      ireslt=sscanf(line, "%*s %*s %d %d %d",&mapstep_a,&mapstep_b,&mapstep_c);

      if (ireslt > 0) {
        if ((fabs(Map_Info.xlim[0]) > 0.001) || (fabs(Map_Info.xlim[1] - 1.0f) > 0.001)) {
// not full cell along x
            new_mapstep_a = (int)(1.0f/(Map_Info.xlim[1] - Map_Info.xlim[0])+0.5f) * mapstep_a;
            astart = (int)(Map_Info.xlim[0] * new_mapstep_a);
            aend = (int)(Map_Info.xlim[1] * new_mapstep_a);
        } else {
            new_mapstep_a = mapstep_a;
            astart = 0;
            aend = mapstep_a;
        }
        if ((fabs(Map_Info.ylim[0]) > 0.001) || (fabs(Map_Info.ylim[1] - 1.0f) > 0.001)) {
// not full cell along y
            new_mapstep_b = (int)(1.0f/(Map_Info.ylim[1] - Map_Info.ylim[0])+0.5f) * mapstep_b;
            bstart = (int)(Map_Info.ylim[0] * new_mapstep_b);
            bend = (int)(Map_Info.ylim[1] * new_mapstep_b);
        } else {
            new_mapstep_b = mapstep_b;
            bstart = 0;
            bend = mapstep_b;
        }
        if ((fabs(Map_Info.zlim[0]) > 0.001) || (fabs(Map_Info.zlim[1] - 1.0f) > 0.001)) {
// not full cell along z
            new_mapstep_c = (int)(1.0f/(Map_Info.zlim[1] - Map_Info.zlim[0])+0.5f) * mapstep_c;
            cstart = (int)(Map_Info.zlim[0] * new_mapstep_c);
            cend = (int)(Map_Info.zlim[1] * new_mapstep_c);
        } else {
            new_mapstep_c = mapstep_c;
            cstart = 0;
            cend = mapstep_c;
        }

        if (FourierPt) free(FourierPt);

        FourierPt = (float*)zalloc( new_mapstep_a*new_mapstep_b*new_mapstep_c * sizeof(float));
        if (FourierPt == NULL) {
          ireslt = 0;
          Error_Box("ERROR -- Unable to allocate space for map.\n");
          return;
        }

        reslt = fgets(line, sizeof(line), mapin);

        for(k=cstart; k < cend; k++) {
          for(j=bstart; j < bend-1; j++) {
            for(i=astart; i < aend-1; i++) {
              ijk =  j*(new_mapstep_a-1)*(new_mapstep_b-1) + k*(new_mapstep_a-1) + i;
              ireslt = fscanf(mapin,"%f", &FourierPt[ijk]);
              if (FourierPt[ijk] < rhomin) rhomin = FourierPt[ijk];
              if (FourierPt[ijk] > rhomax) rhomax = FourierPt[ijk];
            }
            fscanf(mapin,"%*f");
          }
          for (i=0;i<mapstep_a;i++) fscanf(mapin,"%*f");
        }

     }

     if (ijk > 0) {
       if (!Quick)
         fprintf(drvui->flout,"Reading Fourier map file %s\n  "
            "cell = %f %f %f %f %f %f\n  map steps: a=%d b=%d c=%d\n  "
            "Range of rho values: %f to %f, gridsize %d\n", infile, 
            map_a, map_b, map_c, map_alpha, map_beta, map_gamma, 
            mapstep_a, mapstep_b, mapstep_c, rhomin, rhomax,ijk);
        ReadFourMap = 1;
      } else {
        sprintf(string, "Error reading Fourier map file %s", 
 	       infile);
        Error_Box(string);
      }
      fclose(mapin);
      Map_Info.rhomn = rhomin;
      Map_Info.rhomx = rhomax;
	  Map_Info.map_int[0] = mapstep_a = --new_mapstep_a;
	  Map_Info.map_int[1] = mapstep_b = --new_mapstep_b;
	  Map_Info.map_int[2] = mapstep_c = --new_mapstep_c;
	  Map_Info.info_valid = 1;
}
/* ************************************************************** */
/* ************************************************************** */

void read_exc(char * infile, int Quick)
{
      FILE * mapin;
      char line[81];
      char * reslt;
      float rhomin,rhomax;
      int ireslt = 0;
      int ijk = 0;
      int i,j,k;
      char string[200];
      int new_mapstep_a = 0, astart, aend;
      int new_mapstep_b = 0, bstart, bend;
      int new_mapstep_c = 0, cstart, cend;

      Map_Info.lat_con[0] = map_a = drvui->lat_con[0];
      Map_Info.lat_con[1] = map_b=drvui->lat_con[1];
      Map_Info.lat_con[2] = map_c=drvui->lat_con[2];
      Map_Info.lat_con[3] = map_alpha=drvui->lat_con[3];
      Map_Info.lat_con[4] = map_beta=drvui->lat_con[4];
      Map_Info.lat_con[5] = map_gamma=drvui->lat_con[5];
     
      rhomin=1.0e15f;
      rhomax=-1.0e15f;
            
      if ((mapin = fopen (infile, "r")) == NULL) {
        sprintf (string,
           "Cannot open Fourier map (.OUT) file %s\n", infile);
        Error_Box(string);
        return;
      }

      ireslt=fscanf(mapin, "%d %d %d",&mapstep_a,&mapstep_b,&mapstep_c);

      if (ireslt > 0) {
        if ((fabs(Map_Info.xlim[0]) > 0.001) || (fabs(Map_Info.xlim[1] - 1.0f) > 0.001)) {
// not full cell along x
            new_mapstep_a = (int)(1.0f/(Map_Info.xlim[1] - Map_Info.xlim[0])+0.5f) * mapstep_a;
            astart = (int)(Map_Info.xlim[0] * new_mapstep_a);
            aend = (int)(Map_Info.xlim[1] * new_mapstep_a);
        } else {
            new_mapstep_a = mapstep_a;
            astart = 0;
            aend = mapstep_a;
        }
        if ((fabs(Map_Info.ylim[0]) > 0.001) || (fabs(Map_Info.ylim[1] - 1.0f) > 0.001)) {
// not full cell along y
            new_mapstep_b = (int)(1.0f/(Map_Info.ylim[1] - Map_Info.ylim[0])+0.5f) * mapstep_b;
            bstart = (int)(Map_Info.ylim[0] * new_mapstep_b);
            bend = (int)(Map_Info.ylim[1] * new_mapstep_b);
        } else {
            new_mapstep_b = mapstep_b;
            bstart = 0;
            bend = mapstep_b;
        }
        if ((fabs(Map_Info.zlim[0]) > 0.001) || (fabs(Map_Info.zlim[1] - 1.0f) > 0.001)) {
// not full cell along z
            new_mapstep_c = (int)(1.0f/(Map_Info.zlim[1] - Map_Info.zlim[0])+0.5f) * mapstep_c;
            cstart = (int)(Map_Info.zlim[0] * new_mapstep_c);
            cend = (int)(Map_Info.zlim[1] * new_mapstep_c);
        } else {
            new_mapstep_c = mapstep_c;
            cstart = 0;
            cend = mapstep_c;
        }

        if (FourierPt) free(FourierPt);

        FourierPt = (float*)zalloc( new_mapstep_a*new_mapstep_b*new_mapstep_c * sizeof(float));
        if (FourierPt == NULL) {
          ireslt = 0;
          Error_Box("ERROR -- Unable to allocate space for map.\n");
          return;
        }

        reslt = fgets(line, sizeof(line), mapin); // trailing comments from mapstep line
        fgetc(mapin);

        for(i=cstart; i < cend; i++) {
          for(j=bstart; j < bend; j++) {
            for(k=astart; k <aend; k++) {
                ijk =  k * new_mapstep_b * new_mapstep_c + j * new_mapstep_b + i;
                reslt = fgets(line, sizeof(line), mapin); 
                ireslt = sscanf(line,"%*f %*f %*f %f", &FourierPt[ijk]);
                if (FourierPt[ijk] < rhomin) rhomin = FourierPt[ijk];
                if (FourierPt[ijk] > rhomax) rhomax = FourierPt[ijk];
            }
          }
        }
     }

     if (ijk > 0) {
       if (!Quick)
         fprintf(drvui->flout,"Reading Fourier map file %s\n  "
            "cell = %f %f %f %f %f %f\n  map steps: a=%d b=%d c=%d\n  "
            "Range of rho values: %f to %f, gridsize %d\n", infile, 
            map_a, map_b, map_c, map_alpha, map_beta, map_gamma, 
            mapstep_a, mapstep_b, mapstep_c, rhomin, rhomax,ijk);
        ReadFourMap = 1;
      } else {
        sprintf(string, "Error reading Fourier map file %s", 
 	       infile);
        Error_Box(string);
      }
      fclose(mapin);
      Map_Info.rhomn = rhomin;
      Map_Info.rhomx = rhomax;
	  Map_Info.map_int[0] = mapstep_a = new_mapstep_a;
	  Map_Info.map_int[1] = mapstep_b = new_mapstep_b;
	  Map_Info.map_int[2] = mapstep_c = new_mapstep_c;
	  Map_Info.info_valid = 1;
}

/* ************************************************************** */
/* ************************************************************** */

void read_inp (int Quick)

/* routine to read and process input file */
{
  char input[100];                 /* input buffer for data read from file */
  char input2[100];                /* input buffer for data read from file */
  char string[100];                /* temporary string */
  char filename[100];              /* used for map file name */
  int intype;                      /* type of datum read */
  int i, j, k;                     /* temporary */
  char t_name[100];                /* temporary atom name storage */
  char t_color[40];                /* temporary ellipsoid color storage */
  int in_line = 0;                 /* flag for inline import of foreign files */
  float t_rad;
  char el_color_tmp[MAX_EL][100];
  int n_el_tmp = 0;
  int done;
  int tmp_frame_no = 1;            // number of frame being processed
  FILE * tout = 0;
  int curframe;drvui->lat_con[0] = 0.0f;
  int MapRegionRead = 0;
  char atom_pos[3][30];
  int average = 0;
  char modl[5];
  int modnum;
  float avg_occ,min_occ;

/* ************************************************************** */
/* ************************************************************** */
/*
    Commands Implemented

    arrow
    atom
    average
    axislines
    background
    bestplane
    betaij
    bij,Bij
    bond
    bounds
    box
    cell
    clip
    cutout
    dash
    depthcue
    edges
    ellipcolor
    ellipsoids
    finish
    frame
    import
    inline
    labelscale
    labeltext
    list
    lonepair
    lookat
    magtrans
    magnification
    mapcalclimits
    mapcontour
    mapread
    mapregion
    molcomp
    nolabels
    noshadow
    occupancy
    origin
    orthographic
    pack
    phong
    plane
    polyedge
    polysz
    polytolerance
    polylimit
    polyfudge
    polyvert
    rem, REM
    shell
    slab
    special
    spgp, sgrp, spgr
    sphere
    title, titl
    uij,Uij
    vectors    
    view
    vrml1
    vrml2
    vrml97
    xyzoff
    end, END
                                                                  */
/* ************************************************************** */
/* ************************************************************** */

  drvui->xyzoff_read = 0;
  Map_Info.xlim[0] = 0.0f;
  Map_Info.xlim[1] = 1.0f;
  Map_Info.ylim[0] = 0.0f;
  Map_Info.ylim[1] = 1.0f;
  Map_Info.zlim[0] = 0.0f;
  Map_Info.zlim[1] = 1.0f;
  numOfFourierContours = 0;
  curframe = drvui->frame_no;
  if (curframe > 1 && packflag) {
      for (i=0; i<6; i++) {
          drvui->cryst_lim[curframe][i] = drvui->cryst_lim[curframe-1][i];
      }
  } 
  for (i = 0; i < 3; i++)
    drvui->xyzoff[i] = 0.0f;
  memset(el_color_tmp, 0, sizeof(el_color_tmp));
  while (!feof (drvui->fpin)) {
    memset(t_color,0,40);
    memset(string,0,100);
    memset(input,0,sizeof(input));
    if (fgets (input, 100, drvui->fpin)) {         /* read a line */
      trim_string(input,100);          // kill any ^M or ^J at end of line
      Blank_Strip (input);             // suppress any leading blanks
      strcpy(input2,input);            // make a copy for listing file
      for (i=strlen(input); i< 99; i++) { //pad line with blanks
          input[i] = ' ';
      }
      input[99] = 0;                   // force a terminator
    } else {
      Error_Box("Error reading file.");
      return;
    }
    intype = 1000;
    in_line = 0;
    if (!strncmp (input, "titl", 4) || !strncmp (input, "TITL", 4))
        if (drvui->frame_no == 1)
            intype = 1;                // only echo title in first frame
        else
            intype = 0;
    if (strncmp (input, "rem", 3) == 0 || strncmp (input, "REM", 3) == 0)
        if (tmp_frame_no == drvui->frame_no)
            intype = 1;                // only echo rem in its frame
        else
            intype = 0;
    if (strncmp (input, "cell", 4) == 0)
      intype = 2;
    if (strncmp (input, "sgrp", 4) == 0)
      if(tmp_frame_no == drvui->frame_no)
          intype = 3;
        else
          intype = 0;                       // ignore sgrp command if not current frame
    if (strncmp (input, "spgp", 4) == 0)
      if(tmp_frame_no == drvui->frame_no)
          intype = 3;
      else
          intype = 0;                       // ignore spgp command if not current frame
    if (strncmp (input, "spgr", 4) == 0)
      if(tmp_frame_no == drvui->frame_no)
          intype = 3;
      else
          intype = 0;                       // ignore spgr command if not current frame
    if (strncmp (input, "ellipsoids", 10) == 0)
      intype = 4;
    if (strncmp (input, "lookat", 6) == 0)
      intype = 5;
    if (strncmp (input, "atom", 4) == 0)
      if(tmp_frame_no == drvui->frame_no)
          intype = 6;
        else
          intype = 0;                     // ignore atom command if not current frame
    if (strncmp (input, "polysz", 6) == 0)
      if(tmp_frame_no == drvui->frame_no)
          intype = 7;
        else
          intype = 0;                     // ignore polysz command if not current frame
    if (strncmp (input, "sphere", 6) == 0)
      if(tmp_frame_no == drvui->frame_no)
          intype = 8;
        else
          intype = 0;                     // ignore sphere command if not current frame
    if (strncmp (input, "end", 3) == 0)
      intype = 9;
    if (strncmp (input, "END", 3) == 0)
      intype = 9;
    if (strncmp (input, "xyzoff", 6) == 0)
      intype = 10;
    if (strncmp (input, "bond", 4) == 0)
      if(tmp_frame_no == drvui->frame_no)
          intype = 11;
        else
          intype = 0;                     // ignore bond command if not current frame
    if (strncmp (input, "box", 3) == 0)
      intype = 12;
    if (strncmp (input, "edges", 5) == 0)
      intype = 13;
    if (strncmp (input, "view", 4) == 0)
      intype = 14;
    if (strncmp (input, "plane", 5) == 0)
      if(tmp_frame_no == drvui->frame_no)
          intype = 15;
        else
          intype = 0;                     // ignore plane command if not current frame
    if (strncmp (input, "bij", 3) == 0)
      intype = 16;
    if (strncmp (input, "Bij", 3) == 0)
      intype = 16;
    if (strncmp (input, "betaij", 6) == 0)
      intype = 17;
    if (strncmp (input, "uij", 3) == 0)
      intype = 18;
    if (strncmp (input, "Uij", 3) == 0)
      intype = 18;
    if (strncmp (input, "bounds", 6) == 0)
      intype = 1; //19;
    if (strncmp (input, "pack", 4) == 0)
      if(tmp_frame_no == drvui->frame_no)
          intype = 20;
        else
          intype = 0;                     // ignore pack command if not current frame
    if (strncmp (input, "magnification", 13) == 0)
      intype = 21;
    if (strncmp (input, "origin", 6) == 0)
      intype = 22;
    if (strncmp (input, "ellipcolor", 10) == 0)
      intype = 23;
    if (strncmp (input, "inline", 6) == 0) {
      if(tmp_frame_no == drvui->frame_no) {
        intype = 24;
      } else {    // skip ahead to next frame or end
        int end_seen=0; // some formats have their own END card
	while(!feof(drvui->fpin) && intype!=0 ) {
          if (fgets (input, 100, drvui->fpin)) {         /* read a line */
             trim_string(input,100);          // kill any ^M or ^J at end of line
             Blank_Strip (input);             // suppress any leading blanks
             input[99] = 0;                   // force a terminator
             if (strncmp (input, "frame", 5) == 0) {
               intype=37;
               break;
             }
             if (strncmp (input, "end", 3) == 0) {
	       if (strstr(input2,"schakal") && end_seen==0 ) {
                 end_seen=1; 
               } else {
                 intype=9;
                 break;
               }
             }
          } else {
            Error_Box("Error reading file.");
            return;
          }
        } // advance to next frame or end command
      } // if not in current frame 
    }
    if (strncmp (input, "import", 6) == 0)
      if(tmp_frame_no == drvui->frame_no)
          intype = 25;
        else 
          intype = 0;                     // ignore import command if not current frame
    if (strncmp (input, "orthographic", 12) == 0)
      intype = 26;
    if (strncmp (input, "vectors", 7) == 0) {
        if (tmp_frame_no == 1) {
            intype = 27;
        } else {
            intype = 0;
        }
    }
    if (strncmp (input, "phong", 5) == 0)
      intype = 28;
    if (strncmp (input, "molcomp", 7) == 0)
      intype = 29;
    if (strncmp (input, "cutout", 6) == 0)
      intype = 30;
    if (strncmp (input, "depthcue", 8) == 0)
      intype = 31;
    if (strncmp (input, "vrml1", 5) == 0)
      intype = 32;
    if (strncmp (input, "vrml2", 5) == 0)
      intype = 1;
    if (strncmp (input, "vrml97", 6) == 0)
      intype = 1;
    if (strncmp (input, "polytol", 7) == 0)
      intype = 33;
    if (strncmp (input, "polylimit", 9) == 0)
      intype = 33;
    if (strncmp (input, "polyfudge", 9) == 0)
      intype = 33;
    if (strncmp (input, "clip", 4) == 0)
      if(tmp_frame_no == drvui->frame_no)
          intype = 34;
        else
          intype = 0;                     // ignore clip command if not current frame
    if (strncmp (input, "list", 4) == 0)
      intype = 35;
    if (strncmp (input, "polyedge", 8) == 0)
      intype = 36;
    if (strncmp (input, "frame", 5) == 0)
      intype = 37;
    if (strncmp (input, "axislines", 9) == 0)
      intype = 38;
    if (strncmp (input, "shell", 5) == 0)
      if(tmp_frame_no == drvui->frame_no)
          intype = 39;
        else
          intype = 0;                     // ignore shell command if not current frame
    if (strncmp (input, "nolabels", 8) == 0)
      intype = 40;
    if (strncmp (input, "lonepair", 8) == 0)
      if(tmp_frame_no == drvui->frame_no)
          intype = 41;
        else
          intype = 0;                     // ignore lonepair command if not current frame
    if (strncmp (input, "labeltext", 9) == 0)
      intype = 42;
    if (strncmp (input, "polyvert", 8) == 0)
      if(tmp_frame_no == drvui->frame_no)
          intype = 43;
        else
          intype = 0;                     // ignore polyvert command if not current frame
    if (strncmp (input, "background", 10) == 0)
      intype = 44;
    if (strncmp (input, "slab", 4) == 0)
      intype = 45;
    if (strncmp (input, "mag_trans", 9) == 0)
      intype = 46;
    if (strncmp (input, "special", 7) == 0)
      intype = 47;
    if (strncmp (input, "noshadow", 8) == 0)
      intype = 48;
    if (strncmp (input, "finish", 6) == 0)
      intype = 49;
    if (strncmp (input, "arrow", 5) == 0)
      if(tmp_frame_no == drvui->frame_no)
          intype = 50;
        else
          intype = 0;                     // ignore arrow command if not current frame
    if (strncmp (input, "dash", 4) == 0)
      if(tmp_frame_no == drvui->frame_no)
          intype = 51;
        else
          intype = 0;                     // ignore bond command if not current frame
    if (strncmp (input, "mapcontour", 10) == 0)
      intype = 52;
    if (strncmp (input, "mapread", 7) == 0)
      intype = 53;
    if (strncmp (input, "mapregion", 9) == 0)
      intype = 54;
    if (strncmp (input, "mapcalclimits", 13) == 0)
      intype = 55;
    if (strncmp (input, "labelscale", 10) == 0)
      intype = 56;
    if (strncmp (input, "bestplane", 9) == 0)
      intype = 57;
    if (strncmp (input, "average", 7) == 0)
          intype = 58;
    if (strncmp (input, "occupancy", 9) == 0)
          intype = 59;
    if (strncmp (input, "phaseshift", 10) == 0)
          intype = 60;

    switch (intype) {

    case 0:
      break;
    case 1:
      break;                    // ignore title except for listing
    case 2:                     // cell command
      for (i = 3; i <= 5; ++i)
         drvui->lat_con[i] = 0.0f;
         sscanf (input, "%s %f %f %f %f %f %f", string, &drvui->lat_con[0]
          , &drvui->lat_con[1], &drvui->lat_con[2], &drvui->lat_con[3], &drvui->lat_con[4]
          , &drvui->lat_con[5]);
      break;
    case 3:
      symop (input);                 /* process space group information */
      break;
    case 4:                     /* ellipsoids command */
      drvui->do_ellipsoids = 1;
      (void)sscanf (input, "%s %f", string, &drvui->Ellipsoid_Prob);
      if (drvui->Ellipsoid_Prob > 1)
        drvui->Ellipsoid_Prob *= 0.01f;   /* Probability to fraction */
      break;
    case 5:                     // read vectors for lookat command
      sscanf (input, "%s %f %f %f %f %f %f", string, &drvui->lookat_v1[0],
          &drvui->lookat_v1[1], &drvui->lookat_v1[2], &drvui->lookat_v2[0],
          &drvui->lookat_v2[1], &drvui->lookat_v2[2]);
      Options = Options | L_OPT;
      break;
    case 6:                     /* atom command */
      char t_atom[5];
      get_label (input, &t_atom[0], &t_atom[1], &t_atom[2], &t_atom[3], 1);
      t_atom[4] = 0;
      strcpy(drvui->atom_l[natom],t_atom);
      (void)sscanf (input, "%d %s %s %s",&drvui->atom_n[natom],atom_pos[0],
          atom_pos[1],atom_pos[2]);
      drvui->sv_atom_n[natom] = drvui->atom_n[natom];
      for (i=0; i<3; i++)
          drvui->atom_xyz[natom][i] = convert_pos(atom_pos[i]);
      drvui->atom_fn[natom] = tmp_frame_no;
      natom++;
      if (natom >= MAX_ATOM) {
        Error_Box("Too many atoms in asymmetric unit - increase MAX_ATOM.");
        return;
      }
      break;
    case 7:                      /* polysz command */
    case 39:                     /* shell command */
    case 43:                     /* polyvert command */
      get_label (input, &drvui->poly_l[drvui->npoly][0], &drvui->poly_l[drvui->npoly][1]
           , &drvui->poly_l[drvui->npoly][2], &drvui->poly_l[drvui->npoly][3], 1);
      if (intype == 7) {
          (void)sscanf (input, "%f %39c", &drvui->poly_size[drvui->npoly], string);
      } else if (intype == 39) { 
          (void)sscanf (input, "%f %f %39c",  &drvui->poly_min[drvui->npoly],
              &drvui->poly_size[drvui->npoly], string);
      } else if (intype == 43) {
          get_label (input, &drvui->poly_t[drvui->npoly][0], &drvui->poly_t[drvui->npoly][1]
             , &drvui->poly_t[drvui->npoly][2], &drvui->poly_t[drvui->npoly][3], 0);
         (void)sscanf (input, "%f %39c", &drvui->poly_size[drvui->npoly], string);
      }
      trim_string(string,40);
      if (!strlen(string)) strcpy(string,"White");
      strcpy(drvui->poly_col[drvui->npoly],string);
      if (intype == 7 || intype == 43)
          drvui->poly_min[drvui->npoly] = 0.005f;
      drvui->poly_fn[drvui->npoly] = tmp_frame_no;
      if (intype == 7 || intype == 39)
          strcpy(drvui->poly_t[drvui->npoly],"");
      drvui->npoly++;
      if (drvui->npoly >= MAX_POLY) {
          Error_Box("Too many polyhedra specified - increase MAX_POLY");
          return;
      }
      break;
    case 8:                     /* sphere command */
      get_label (input, &drvui->sphere_l[drvui->nsphere][0], &drvui->sphere_l[drvui->nsphere][1]
         , &drvui->sphere_l[drvui->nsphere][2], &drvui->sphere_l[drvui->nsphere][3], 1);
      char str[9][40], *pnt;
      double inpval[9];
      i = sscanf(input,"%s %s %s %s %s %s %s %s %s", str[0], str[1], str[2], str[3], str[4],
          str[5], str[6], str[7], str[8]);
      for (j=0; j<i; j++)
          inpval[j] = strtod(str[j],&pnt);
      drvui->sphere_n[drvui->nsphere] = -1;
      if (i == 2 || i == 4 || i == 6) {
          drvui->sphere_size[drvui->nsphere] = (float)inpval[0];
          j = 1;
      } else if (i == 3 || i == 5 || i == 7) {
          drvui->sphere_n[drvui->nsphere] = (int)(inpval[0] + 0.01);
          drvui->sphere_size[drvui->nsphere] = (float)inpval[1];
          j = 2;
      } else {
          Error_Box("Improperly formed sphere command.");
      }
      strcpy(string,str[j]);
      for (k=j+1; k<i; k++) {
          strcat(string," ");
          strcat(string,str[k]);
      }
      trim_string(string,40);
      if (!strlen(string)) strcpy(string,"White");
      strcpy(drvui->sphere_col[drvui->nsphere],string);
      drvui->sphere_fn[drvui->nsphere] = tmp_frame_no;
      drvui->poly_rad_edge[drvui->npoly]=0.0f;
      drvui->nsphere++;
      if (drvui->nsphere >= MAX_SPHERE) {
          Error_Box("Too many sphere types specified - increase MAX_SPHERE.");
          return;
      }
      break;
    case 9:                     /* end or END */
      if (!drvui->lat_con[0]) {
        if (errorbox) { // make sure that the original error message gets a chance to appear
          int n;
          for (n=0;n<10;n++)Fl::wait(1); 
        }
          Error_Box("There is no 'cell' line in your input file.");
      }
      label_cell();
      if (Quick == 2) {
          drvui->max_frame = tmp_frame_no;
          drvui->Old_Xrot = xrot;
          drvui->Old_Yrot = yrot;
          drvui->Old_Zrot = zrot;
      }
      if (average==1) drvui->modulated=-1;
      int frame;
      for (frame = 1; frame<=drvui->max_frame; frame++) {
          if (!Quick) {
              strcpy(input,drvui->Cur_Root);
              sprintf(input2,".frm%d",frame);
              strcat(input,input2);
              tout = fopen(input,"w");
          }
          for (i = 0; i < natom; i++)  {       /* shift atom coordinates for origin offset */
              if (drvui->atom_fn[i] != frame) continue;
              drvui->TF_status[i] = -1;                 // initialize TF type/status
              for (j = 0; j < 3; j++)
                  drvui->atom_xyz[i][j] -= drvui->xyzoff[j];
              strncpy(t_atom,drvui->atom_l[i],4);
              t_atom[4] = 0;
              if (!Quick)
                  fprintf(tout,"%s %d\n",t_atom,drvui->sv_atom_n[i]);
          }
          if (!Quick) {
              fclose(tout);
          }
      }
      if (n_el_tmp > 0) {
          strcpy(input,"");
          for (i = 0; i < n_el_tmp; i++) {
            get_label (el_color_tmp[i], &t_name[0], &t_name[1],
                 &t_name[2], &t_name[3], 0);
            t_name[4] = 0;
            if (el_color_tmp[i][0] == '*') {
              (void)sscanf (el_color_tmp[i], "%s %39c", string, t_color);
              j = -1;
            } else {
              (void)sscanf (el_color_tmp[i], "%d %39c", &j, t_color);
            }
            trim_string(t_color,40);
            if (!strlen(t_color))strcpy(t_color,"White");
            for (k=1; k<drvui->n_ellips; k++) {     /* find this atom in ellipsoid list */
                if (check_atom_name(t_name,drvui->ellips_l[k])) {
                  if ((j == -1) || (j == drvui->ellips_n[k])) {
                    drvui->ell_type[k] += 1000;                  // >= 1000 means ellipcolor read
                    strcpy (drvui->ellips_col[k], t_color);    /* copy color */
                    drvui->save_el_number[k]=j;
                  }
                }
            } 
          }
      }
      for (i=1; i<drvui->nsphere; i++) {        // remove any auto-generated ellipsoids that
          for (j=1; j<drvui->n_ellips; j++) {   //     interfere with spheres
              if (drvui->ell_type[j] == 1001) {
                  if (check_atom_name(drvui->sphere_l[i],drvui->ellips_l[j])) {
                      if (drvui->sphere_n[i] == -1
                          || drvui->sphere_n[i] == drvui->save_el_number[j]) {
                          drvui->ell_type[j] = 1;
                      }
                  }
              }
          }
      }
      if (FourierMapType)
	drvui->mainWindow->cursor(FL_CURSOR_WAIT);
      switch (FourierMapType) {
      case 1:
          read_grd( FourierFileName, Quick);
          break;
      case 2:
          read_stf( FourierFileName, Quick);
          break;
      case 3:
          read_w2k( FourierFileName, Quick);
          break;
      case 4:
          read_vasp( FourierFileName, Quick);
          break;
      case 5:
          read_flp( FourierFileName, Quick);
          break;
      case 6:
          read_fcf( FourierFileName, Quick);
          break;
      case 7:
          read_dn6( FourierFileName, Quick);
          break;
      case 8:
          read_m80( FourierFileName, Quick);
          break;
      case 9:
          read_exc( FourierFileName, Quick);
          break;
      default:
          break;
      }
      if (FourierMapType)
	drvui->mainWindow->cursor(FL_CURSOR_DEFAULT);
      if (!MapRegionRead) {             // mapregion line NOT read
          xMin = Map_Info.xlim[0];
          xMax = Map_Info.xlim[1];
          yMin = Map_Info.ylim[0];
          yMax = Map_Info.ylim[1];
          zMin = Map_Info.zlim[0];
          zMax = Map_Info.zlim[1];
      }
      if (drvui->subsys_vol[0] == 1.0f)
          drvui->subsys_vol[0] = drvui->subsys_ref_volume;
      return;                     /* thru with end */
    case 10:                     /* xyzoff command */
      (void)sscanf (input, "%s %s %s %s",string,atom_pos[0],atom_pos[1],atom_pos[2]);
      for (i=0; i<3; i++)
          drvui->xyzoff[i] = convert_pos(atom_pos[i]);
      drvui->xyzoff_read = 1;
      break;
    case 51:                     /* dash command */
    case 11:                     /* bond command */
      get_label (input, &drvui->bond_l1[drvui->nbond][0], &drvui->bond_l1[drvui->nbond][1]
         , &drvui->bond_l1[drvui->nbond][2], &drvui->bond_l1[drvui->nbond][3], 1);
      get_label (input, &drvui->bond_l2[drvui->nbond][0], &drvui->bond_l2[drvui->nbond][1]
         , &drvui->bond_l2[drvui->nbond][2], &drvui->bond_l2[drvui->nbond][3], 0);
      (void)sscanf (input, "%f %f %f %39c",
          &drvui->bond_size[drvui->nbond], &drvui->bond_min[drvui->nbond], 
          &drvui->bond_max[drvui->nbond], string);
      trim_string(string,40);
      if (strlen (string) == 0)
          strcpy (string, "Grey");
      strcpy(drvui->col_bond[drvui->nbond],string);
      drvui->bond_fn[drvui->nbond] = tmp_frame_no;
      if (intype == 11)
        drvui->bond_style[drvui->nbond] = 0;
      else
        drvui->bond_style[drvui->nbond] = 1; 
      drvui->nbond++;
      if (drvui->nbond >= MAX_BOND) {
          Error_Box("Too many bond types specified - increase MAX_BOND.");
      }
      break;
    case 12:                     /* box command */
      (void)sscanf (input, "%s %f %39c", string, &rad_cell, t_color);
      trim_string(t_color,40);
      strcpy(drvui->col_cell,t_color);
      if (strlen (drvui->col_cell) == 0)
        strcpy (drvui->col_cell, "Black");
      if (rad_cell == 0.0) {
        docell = 0;
//        Labels = 0;
      }
      break;
    case 13:                     /* edges command */
      (void)sscanf (input, "%s %f %39c", string, &drvui->rad_edge, t_color);
      trim_string(t_color,40);
      strcpy(drvui->col_edge,t_color);
      if (drvui->rad_edge > 0.005)
        edges = 2;
      if (strlen (drvui->col_edge) == 0)
        strcpy (drvui->col_edge, "Black");
      break;
    case 14:                     /* view command */
//      if ((Options & V_OPT) == 0)         // only if no -v switch 
        (void)sscanf (input, "%s %f %f %f", string, &xrot, &yrot, &zrot);
      break;
    case 15:                     /* plane command */
      get_label (input, &drvui->plane_l[drvui->nplane][0], &drvui->plane_l[drvui->nplane][1]
         , &drvui->plane_l[drvui->nplane][2], &drvui->plane_l[drvui->nplane][3], 1);
      (void)sscanf (input, "%f %39c", &drvui->plane_size[drvui->nplane], string);
      trim_string(string,40);
      if (!strlen(string))strcpy(string,"White");
      strcpy(drvui->plane_col[drvui->nplane],string);
      drvui->plane_fn[drvui->nplane] = tmp_frame_no;
      drvui->nplane++;
      if (drvui->nplane >= MAX_PLANE) {
          Error_Box("Too many plane types specified - increase MAX_PLANE.");
          return;
      }
      break;
    case 16:                     /* bij or Bij command */
      drvui->ell_type[drvui->n_ellips] = 2;
    case 17:                     /* betaij command */
      if (intype == 17)
        drvui->ell_type[drvui->n_ellips] = 0;
    case 18:                     /* uij or Uij command */
      if (intype == 18)
        drvui->ell_type[drvui->n_ellips] = 1;
      get_label (input, &drvui->ellips_l[drvui->n_ellips][0], &drvui->ellips_l[drvui->n_ellips][1]
         , &drvui->ellips_l[drvui->n_ellips][2], &drvui->ellips_l[drvui->n_ellips][3], 1);
      drvui->ellips_l[drvui->n_ellips][4] = 0;
      (void)sscanf (input, "%d %f %f %f %f %f %f %39c", &drvui->ellips_n[drvui->n_ellips],
          &drvui->ellips[drvui->n_ellips][0], &drvui->ellips[drvui->n_ellips][1],
          &drvui->ellips[drvui->n_ellips][2], &drvui->ellips[drvui->n_ellips][3], 
          &drvui->ellips[drvui->n_ellips][4], &drvui->ellips[drvui->n_ellips][5],
          string);                                       // read unique coefficients and color
      drvui->save_el_number[drvui->n_ellips] = drvui->ellips_n[drvui->n_ellips];
      trim_string(string,40);
      strcpy(drvui->ellips_col[drvui->n_ellips],string);
      if (!strlen (drvui->ellips_col[drvui->n_ellips]))
        strcpy (drvui->ellips_col[drvui->n_ellips], "Gray80");
      drvui->n_ellips++;
      if (drvui->n_ellips >= MAX_EL) {
          Error_Box("Too many ellipsoid types specified - increase MAX_EL.");
          return;
      }
      break;
    case 19:                     /* bounds command */
//      if ((Options & B_OPT) == 0) {         // only do this if no -b switch 
        (void)sscanf (input, "%s %f %f %f", string, &boxlim[0], &boxlim[1],
            &boxlim[2]);
        boxflag = 1;
//      }
      break;
    case 20:                     /* pack command */
//      if ((Options & P_OPT) == 0) {         // only if no -p switch
        (void)sscanf (input, "%s %f %f %f %f %f %f", string, 
            &drvui->cryst_lim[tmp_frame_no][0], &drvui->cryst_lim[tmp_frame_no][3],
            &drvui->cryst_lim[tmp_frame_no][1], &drvui->cryst_lim[tmp_frame_no][4],
            &drvui->cryst_lim[tmp_frame_no][2], &drvui->cryst_lim[tmp_frame_no][5]);
        packflag = 1;
//      }
      break;
    case 21:                     /* magnification command */
//      if ((Options & M_OPT) == 0)         // only if no -m switch
        (void)sscanf (input, "%s %f", string, &Magnification);
      break;
    case 22:                     /* origin command */
//      if ((Options & O_OPT) == 0) {         // only if no -o switch
        (void)sscanf (input, "%s %f %f %f", string, &origin[0], &origin[1],
            &origin[2]);
//      }
      break;
    case 23:                     /* ellipcolor command */
      Token_Strip (input, 1);
      strcpy (el_color_tmp[n_el_tmp++], input);
      break;
    case 24:                     /* inline command */
      in_line = 1;
    case 25:                     /* import command */
      char filetype[10];
      done = 0;
      if (!Quick)
        fprintf (drvui->flout, "*         %s\n======================= Start of import/inline commands\n",
            input2);     // echo to listing file
      intype = 0;
      Token_Strip (input, 1);             /* strip command*/
        sscanf(input,"%s",filetype);
      Token_Strip (input, 1);             /* strip filetype*/
        trim_string(input,100);          /* and remove trailing blanks */

      if (strncmp (filetype, "gsas", 4) == 0) {
        import_gsas (input, in_line, Quick, tmp_frame_no);
        done = 1;
      }
      if (strncmp (filetype, "pcr", 3) == 0) {
        import_pcr (input, in_line, Quick, tmp_frame_no);
        done = 1;
      }
      if (strncmp (filetype, "cif", 3) == 0) {
        int ii,jj;
        ii=strlen(input)-1;
        if (isdigit(input[ii])) { // if number at end 
          for (jj=ii;jj>ii-6;jj--) {
            if (isspace(input[jj])) { // separate number
              input[jj]='\0';
	      Block_CIF=atoi(&input[jj+1]);
              break;
            }
            if (isalpha(input[jj]))
              break; // unless it is attached to filename
          }
        }
        import_cif (input, in_line, Quick, &Block_CIF, tmp_frame_no);
        done = 1;
      }
      if (strncmp (filetype, "schakal", 7) == 0) {
        import_schakal (input, in_line, Quick, tmp_frame_no);
        done = 1;
      }
      if (strncmp (filetype, "shakal", 6) == 0) {
        import_schakal (input, in_line, Quick, tmp_frame_no);
        done = 1;
      }
      if (strncmp (filetype, "shelx", 5) == 0) {
        import_shelx (input, in_line, Quick, tmp_frame_no);
        done = 1;
      }
      if (strncmp (filetype, "fdat", 4) == 0) {
        import_fdat (input, in_line, Quick, tmp_frame_no);
        done = 1;
      }
      if (strncmp (filetype, "csd", 3) == 0) {
        import_fdat (input, in_line, Quick, tmp_frame_no);
        done = 1;
      }
      if (strncmp (filetype, "ccdf", 4) == 0) {
        import_fdat (input, in_line, Quick, tmp_frame_no);
        done = 1;
      }
      if (strncmp (filetype, "wien2k", 6) == 0) {
        import_wien (input, in_line, Quick, tmp_frame_no);
        done = 1;
      }
      if (strncmp (filetype, "discus", 6) == 0) {
        import_discus (input, in_line, Quick, tmp_frame_no);
        done = 1;
      }
      if (!done) {
          char string[256];
          sprintf (string,"Import/inline source type '%s' not recognized.", input);
          Error_Box(string);
          return;
      }
      if (!Quick)
        fprintf (drvui->flout, "======================= End of import/inline commands\n");
      break;
    case 26:                     /* orthographic */
      M_cameras = 0;
      break;
    case 27: {                   /* unit cell direction vectors */
      float v[3];
      Display_axes = 1;
      offset[0] = offset[1] = offset[2] = 0.0f;
      i = sscanf( input, "%s %f %f %f", string, &v[0], &v[1], &v[2]);
      if (i == 4)
          for (i=0; i<3; i++)
              offset[i] = v[i];
      }
      break;
    case 28:                     /* phong */
      (void)sscanf (input, "%s %f %f", string, &drvui->Phong_Value, &drvui->Phong_Size);
      break;
    case 29:                     /* molcomp */
      (void)sscanf (input, "%s %f", string, &drvui->mol_d);
      domolcomp = 1;
      break;
    case 30:                     /* cutout */
      drvui->El_Cutout = 1;
      (void)sscanf (input, "%s %39c", string, t_color);
      trim_string(t_color,40);
      if (!strlen(t_color)) strcpy(t_color,"White");
      strcpy(drvui->Cutout_color,t_color);
      break;
    case 31:                     /* depthcue - thickness of edges scaled by Z coordinate */
      (void)sscanf (input, "%s %f", string, &DepthCue);
      break;
    case 32:                     /* vrml1 - turn on VRML1 output */
      Vrml2 = 0;
      break;
    case 33:                     /* polylimit (formerly polyfudge) */
      (void)sscanf (input, "%s %f", string, &drvui->polylimit);
      if (!Quick)
        fprintf (drvui->flout,
             "Tolerance factor for non-planarity of polyhedra faces set to %f. This option should be used with care\n",
             drvui->polylimit);
      break;
    case 34:                     /* clip command */
      (void)sscanf (input, "%s %f %f %f %f %f %f", string, &drvui->clip_lim[tmp_frame_no][0],
          &drvui->clip_lim[tmp_frame_no][3], &drvui->clip_lim[tmp_frame_no][1], 
          &drvui->clip_lim[tmp_frame_no][4], &drvui->clip_lim[tmp_frame_no][2], 
          &drvui->clip_lim[tmp_frame_no][5]);
      clipflag = 1;
      break;
    case 35:                     /* list - limit for printed distance table */
      (void)sscanf (input, "%s %f", string, &printdist);
      if (!Quick)
        fprintf (drvui->flout,
            "Interatomic distances up to %6.3f input units will be tabulated in the logfile\n",
            printdist);
      break;
    case 36:                     /* polyedge - enables individual coloring of polyhedral edges */
      get_label (input, &t_name[0], &t_name[1], &t_name[2], &t_name[3], 1);
      (void)sscanf (input, "%f %39c", &t_rad, t_color);
      trim_string(t_color,40);
      if (!strlen(t_color))strcpy(t_color,"White");
      for (i = 1; i < drvui->npoly; i++) {         /* find this atom in poly list */
        if ((t_name[0] == drvui->poly_l[i][0]) &&
            (t_name[1] == drvui->poly_l[i][1]) &&
            (t_name[2] == drvui->poly_l[i][2]) && (t_name[3] == drvui->poly_l[i][3])) {
          edges = 1;
          drvui->poly_rad_edge[i] = t_rad;
          strcpy (drvui->poly_col_edge[i], t_color);
        }
      }
      break;
    case 37:                     /* frame command */
      tmp_frame_no++;
      break;
    case 38:                     /* axislines */
      (void)sscanf (input, "%s %f %39c", string, &drvui->Ellipaxis_width,t_color);
      trim_string(t_color,40);
      strcpy(drvui->Ellipaxis_color,t_color);
      if (strlen(drvui->Ellipaxis_color) == 0)
        strcpy (drvui->Ellipaxis_color, "Gray20");
      break;
    case 40:                     /* nolabels command */
      Labels = 0;                 /* set value false */
      break;
    case 41:                     /* lonepair command */
      get_label (input, &drvui->cone_l1[drvui->ncone][0], &drvui->cone_l1[drvui->ncone][1]
         , &drvui->cone_l1[drvui->ncone][2], &drvui->cone_l1[drvui->ncone][3], 1);
      (void)sscanf (input, "%d %f %f %f %39c",
          &drvui->numlonepairs[drvui->ncone], &drvui->cone_height[drvui->ncone],
          &drvui->cone_min[drvui->ncone], &drvui->cone_max[drvui->ncone], t_color);
      trim_string(t_color,40);
      strcpy(drvui->col_cone[drvui->ncone],t_color);
      if (strlen (drvui->col_cone[drvui->ncone]) == 0)
          strcpy (drvui->col_cone[drvui->ncone], "Grey");
      drvui->cone_fn[drvui->ncone] = tmp_frame_no;
      drvui->ncone++;
      if (drvui->ncone >= MAX_CONE) {
          Error_Box("Too many lp bond types specified - increase MAX_CONE.");
          return;
      }
      break;
    case 42:                     /* labeltext command */
      memset (drvui->label_label[drvui->nlabel], 0, 64);
      Token_Strip (input, 1);
      (void)sscanf (input, "%f %f %f %63c",
          &(drvui->label_x[drvui->nlabel][0]), &(drvui->label_x[drvui->nlabel][1]),
          &(drvui->label_x[drvui->nlabel][2]), drvui->label_label[drvui->nlabel]);
      trim_string(drvui->label_label[drvui->nlabel],64);
      drvui->label_fn[drvui->nlabel] = tmp_frame_no;
      intype = 0;
      if(!strcmp(drvui->label_label[drvui->nlabel],"a") && curframe != 1) break;
      if(!strcmp(drvui->label_label[drvui->nlabel],"b") && curframe != 1) break;
      if(!strcmp(drvui->label_label[drvui->nlabel],"c") && curframe != 1) break;
      if(!strcmp(drvui->label_label[drvui->nlabel],"o") && curframe != 1) break;
      intype = 42;
      drvui->nlabel++;
      if (drvui->nlabel >= MAX_LABELS) {
          Error_Box("Too many labels specified - increase MAX_LABELS.");
          return;
      }
      break;
    case 44:                     // background (color) command
      Token_Strip (input, 1);
      strcpy (drvui->col_bg, input);
      trim_string(drvui->col_bg,40);
      if (strlen(drvui->col_bg) < 2)
          strcpy(drvui->col_bg,"White");
      strcpy (input, drvui->col_bg);
      Transform_VRML_Color(input);
      sscanf(input,"%f %f %f",&drvui->glback[0],&drvui->glback[1],&drvui->glback[2]);
      break;
    case 45:
      (void)sscanf (input,"%*s %f %f %f %f %f %f %f %f %f %f %f %f %d",
                    &drvui->slab_con[0],&drvui->slab_con[1],&drvui->slab_con[2],
                    &drvui->slab_con[3],&drvui->slab_con[4],&drvui->slab_con[5],
                    &drvui->slab_off[0],&drvui->slab_off[1],&drvui->slab_off[2],
                    &drvui->slab_rot[0],&drvui->slab_rot[1],&drvui->slab_rot[2],
                    &slabmode);

      break;
    case 46:                     // mag_trans command
      Token_Strip (input, 1);
      (void) sscanf(input,"%f %f %f %f %f %f %f %f %f",&drvui->mag_matrix[0][0],
                    &drvui->mag_matrix[0][1],&drvui->mag_matrix[0][2],
                    &drvui->mag_matrix[1][0],&drvui->mag_matrix[1][1],
                    &drvui->mag_matrix[1][2],&drvui->mag_matrix[2][0],
                    &drvui->mag_matrix[2][1],&drvui->mag_matrix[2][2]);
      break;
    case 47:                     // special command
      if (!Quick) break;
      (void) sscanf (input, "%*7c %d %d", &Omit->omit1[Omit->nomits],
          &Omit->omit2[Omit->nomits]);
      Omit->nomits++;
      break;
    case 48:                    // noshadow command
      drvui->noshadow = 1;
      break;
    case 49:			// finish command
      (void) sscanf (input, "%*6c %f %f %f %f",&drvui->ambient,
            &drvui->diffuse, &drvui->specular, &drvui->roughness);	
      break;
    case 50:            // arrow command
      Token_Strip (input, 1);
      (void) sscanf(input,"%f %f %f %f %f %f %f %f %39c",&drvui->mag_xp[drvui->nmag][0],
                         &drvui->mag_xp[drvui->nmag][1],&drvui->mag_xp[drvui->nmag][2],
                         &drvui->mag_xc[drvui->nmag][0],&drvui->mag_xc[drvui->nmag][1],
                         &drvui->mag_xc[drvui->nmag][2],&drvui->arrow_length[drvui->nmag],
                         &drvui->arrow_diam[drvui->nmag],t_color);
      trim_string(t_color,40);
      if (!strlen(t_color))strcpy(t_color,"White");
      strcpy(drvui->col_arrow[drvui->nmag],t_color);
      drvui->arrow_fn[drvui->nmag] = tmp_frame_no;
      drvui->nmag++;
      if (drvui->nmag >= MAX_ARROW) {
          Error_Box("Too many arrow types specified - increase MAX_ARROW.");
          return;
      }
      break;
    case 52:            /* mapcontour command */
      numOfFourierContours++;
      if (numOfFourierContours > MAX_MAP_CONTOUR) {
          Error_Box("Too many contours specified - increase MAX_MAP_CONTOUR.");
          exit (0);
      }
      Token_Strip (input, 1);
      (void) sscanf(input,"%f %s %39c",&FourierContourLevel[numOfFourierContours],
          string,t_color);
      trim_string(t_color,40);
      if (!strlen(t_color))strcpy(t_color,"White");
      strcpy(FourierContourColor[numOfFourierContours],t_color);
      if(strncmp(string, "mesh", 4) == 0)
          FourierContourSolid[numOfFourierContours] = 0;
      else
          FourierContourSolid[numOfFourierContours] = 1;
      break;
    case 53:              /* mapread command */
      char *lastblank;
      Token_Strip (input, 1);
      if (!Quick) break;
      memset(filename,0,sizeof(filename));
      t_name[0]='\0';
      (void)sscanf(input, "%s", string);
      Token_Strip(input,1);
      trim_string(input,100);
      strcpy(filename,input);
      if (strncmp(string, "grd", 3) == 0) {
          strcpy(FourierFileName,filename);
          FourierMapType=1;
      } else if (strncmp(string, "stf", 3) == 0) {
          strcpy(FourierFileName,filename);
          FourierMapType=2;
      } else if (strncmp(string, "w2k", 3) == 0) {
          strcpy(FourierFileName,filename);
          FourierMapType=3;
      } else if (strncmp(string, "vsp", 3) == 0) {
          strcpy(FourierFileName,filename);
          FourierMapType=4;
      } else if (strncmp(string, "flp", 3) == 0) {
          strcpy(FourierFileName,filename);
          FourierMapType=5;
      } else if (strncmp(string, "fcf", 3) == 0) {
          lastblank=strrchr(input,' ');
          if (!lastblank)
            strcpy(t_name,"Fo");
          else {
            while (isspace(*(lastblank-1))) {
              lastblank--;
            }
            *lastblank = '\0';
            lastblank++;
            while(isspace(*lastblank)) lastblank++;
            strcpy(t_name,lastblank);
          }         
          strcpy(filename,input);
          strcpy(FourierFileName,filename);
          FourierMapType=6;
          Map_Info.map_type = 0;
          if (!strcmp(t_name,"Fc")) Map_Info.map_type = 1;
          else if (!strcmp(t_name,"Fo-Fc")) Map_Info.map_type = 2;
          else if (!strcmp(t_name,"2Fo-Fc")) Map_Info.map_type = 3;
          else if (!strcmp(t_name,"Fo2")) Map_Info.map_type = 4;
      } else if (strncmp(string, "dn6", 3) == 0) {
          strcpy(FourierFileName,filename);
          FourierMapType=7;
      } else if (strncmp(string, "m80", 3) == 0) {
          lastblank=strrchr(input,' ');
          if (!lastblank) 
            strcpy(t_name,"Fo");
          else {
            while (isspace(*(lastblank-1))) {
              lastblank--;
            }
            *lastblank = '\0';
            lastblank++;
            while(isspace(*lastblank)) lastblank++;
            strcpy(t_name,lastblank);
          } 
          strcpy(filename,input);
          strcpy(FourierFileName,filename);
          FourierMapType=8;
          Map_Info.map_type = 0;
          if (!strcmp(t_name,"Fc")) Map_Info.map_type = 1;
          else if (!strcmp(t_name,"Fo-Fc")) Map_Info.map_type = 2;
          else if (!strcmp(t_name,"2Fo-Fc")) Map_Info.map_type = 3;
          else if (!strcmp(t_name,"Fo2")) Map_Info.map_type = 4;
      } else if (strncmp(string, "exc", 3) == 0) {
          strcpy(FourierFileName,filename);
          FourierMapType=9;
      } else {
          Error_Box("mapread command does not specify a legal file type.");
      }
      break;
    case 54:              /* mapregion command */
      Token_Strip(input, 1);
      (void) sscanf(input, "%f %f %f %f %f %f",&xMin, &xMax, &yMin,
          &yMax, &zMin, &zMax);
	  MapRegionRead = 1;
      break;
    case 55:              /* mapcalclimits command */
      Token_Strip(input, 1);
      (void) sscanf(input, "%f %f %f %f %f %f",&Map_Info.xlim[0], &Map_Info.xlim[1],
          &Map_Info.ylim[0], &Map_Info.ylim[1], &Map_Info.zlim[0], &Map_Info.zlim[1]);
      break;
    case 56:              /* labelscale command */
      Token_Strip(input, 1);
      (void) sscanf(input, "%f", &drvui->label_scale);
      break;
    case 57: /* bestplane command */
      char bplane[15][5];
      sscanf(input,"%*s %d",&drvui->nbatoms[drvui->nbplane]);
      if (drvui->nbatoms[drvui->nbplane] > 15) {
          Error_Box("Cannot handle more than 15 atoms in a best plane.");
          return;
      }
      Token_Strip(input, 2);
      for (i=0;i<drvui->nbatoms[drvui->nbplane];i++) {
	sscanf(input,"%s",bplane[i]);
        Token_Strip(input,1);
      }
      memset(drvui->bplane_col[drvui->nbplane],0,40);
      sscanf(input,"%f %f %39c",&drvui->bplane_d1[drvui->nbplane],
        &drvui->bplane_d2[drvui->nbplane],drvui->bplane_col[drvui->nbplane]);
      for (i=0;i<drvui->nbatoms[drvui->nbplane];i++) {
        memset(drvui->bplane_t[drvui->nbplane][i],0,4);
        strcpy(drvui->bplane_t[drvui->nbplane][i],bplane[i]);
        char *p = strstr(bplane[i],"*");
        if (p!=NULL) {
          *p='\0';
          drvui->bplane_n[drvui->nbplane][i]=-1;
        } else {
          for (j=1;j<4;j++)
            if (bplane[i][j]<= '9') {
              char *q=drvui->bplane_t[drvui->nbplane][i]+j;
              drvui->bplane_n[drvui->nbplane][i]=strtol(q,NULL,10);
              for (k=j;k<4;k++)
                drvui->bplane_t[drvui->nbplane][i][k]=' ';
              break;
            }
          }
        }
      drvui->nbplane++;
      if (drvui->nbplane >= MAX_BPLANE) {
          Error_Box("Too many best planes specified - increase MAX_BPLANE.");
          return;
      }
      break;
    case 58:
	average=1;
      break;
    case 59:
      Token_Strip (input, 1);
      memset(modl,0,4);
      sscanf (input,"%s %d %f %f",modl,&modnum, &avg_occ, &min_occ);
      if (modl[1]=='\0') 
        modl[1]=modl[2]=modl[3]=' ';
      if (modl[2]=='\0') 
        modl[2]=modl[3]=' ';
      if (modl[3]==0) 
        modl[3]=' ';
      modl[4]='\0';
          for (j=0;j< natom;j++) {
            if (check_atom_name(modl,drvui->atom_l[j]) && modnum == drvui->atom_n[j] ) {
              drvui->occupancy[j]=avg_occ;
              drvui->min_occ[j]=min_occ;
              break;
            }
          }
      break;
    case 60:
          sscanf(input,"%*s %f %f %f",&drvui->phaseshift[0], &drvui->phaseshift[1],
                                      &drvui->phaseshift[2]);
          for (j=0; j<3; j++)
              drvui->phaseshift[j] = (float)fmod (drvui->phaseshift[j], 1.);
          break;
    default:
      if (!Quick)
        fprintf (drvui->flout,
             "********** Next line does not start with a recognizable keyword - ignored\n");
      break;
    }
    if (!Quick && intype)
        fprintf (drvui->flout, "*         %s\n", input2);     // echo to listing file
  }
  if (drvui->n_ellips > 1)
    drvui->do_ellipsoids = 1;
  if (packflag != 0) {
    boxlim[0] = boxlim[1] = boxlim[2] = 500.0f;
  }
}                         /* end of read_inp */

/* ************************************************************** */
/* ************************************************************** */

void set_tf_status(void)
{
// set the TF_status flags for the atoms in the list

    int i, j;

    for (i=0; i<natom; i++) {
        if (drvui->atom_fn[i] != drvui->frame_no) continue;
        drvui->TF_status[i] = -1;
        for (j=1; j<drvui->n_ellips; j++) {
            if (check_atom_name(drvui->atom_l[i], drvui->ellips_l[j])) {
              if (drvui->ellips_n[j] == -1
                    || drvui->ellips_n[j] == drvui->sv_atom_n[i]) {
                if(drvui->ell_type[j] > 1000) {
                    drvui->TF_status[i] = 1;
                    j = drvui->n_ellips;
                    break;
                } else {
                    if(drvui->ell_type[j] > 0) {
                        drvui->TF_status[i] = 0;
                        j = drvui->n_ellips;
                        break;
                    }
                }
              }
            }
        }
        j = drvui->TF_status[i];
    }
}


/* ************************************************************** */
/* ************************************************************** */

void skip_blocks(int i, FILE * in)
{
// routine to skip data blocks in a CIF
    char string[100];
    int j;

    memset(string, 0, 100);
    for (j=0; j<=i; j++) {
        while (strncmp (string, "data_", 5) != 0) {
            if (!fgets (string, 100, in)) {         /* search for data_ keyword */
                Error_Box("Error skipping data blocks in CIF Import File, Run aborted.");
                (void)fclose(in);
                return;
            }
        }
        memset(string, 0, 100);
  }
}


/* ************************************************************** */
/* ************************************************************** */

void Token_Strip (char string[100], int no)

/*  string[100] - input output string */
/*  no - number of tokens to strip */
/* routine to strip tokens from the leading part of a string */
{
  int i, j;

  for (i = 0, j = 0; i < no; i++) {         /* loop through tokens to be removed */
    while (j<99 && string[j++] != ' ')
      string[j - 1] = ' ';             /* characters to space */
    while (j<100 && string[j] == ' ')
      j++;                     /* skip all spaces */
  }
  Blank_Strip (string);                 /* move stuff to left */
}

/* ************************************************************** */
/* ************************************************************** */

void Transform_POV_Color(char * color)
{
// Transform a POV color is specified in the form f1 f2 f2 {filter f} format,
// where {filter} is optional. If a color name is specified, return with no
// action.
    float red, green, blue, filter;
    char working[120], string[20];
    char *p;

    if (!strtod(color,&p) && p == color)
        return;                          /* we have the color name form */

    strcpy(working, color);              /* make a working copy */
    (void)sscanf(working, "%f %f %f", &red, &green, &blue);

    if ((p = strstr(working,"filter"))) { /* if "filter" given, return rgbf<...> form */
        (void)sscanf(p, "%s %f", string,&filter);
        sprintf(working,"rgbf <%.4f,%.4f,%.4f,%.4f>",red,green,blue,filter);
    } else {                             /* return rgb <...> form */
        sprintf(working,"rgb <%.4f,%.4f,%.4f>",red,green,blue);
    }
    strcpy(color, working);              /* reformatted color to caller */
}
    
/* ************************************************************** */
/* ************************************************************** */

void Transform_VRML_Color (char *input)
{
  char *transp_start;
  char transp[20];
  float fval;
  int i;
  char *a,b[30],line[81];
  char thecolor[50];
  char keyword[80];
  float thered,thegreen,theblue;
  char POV_incpath[255]="\0";
  FILE *colinc;

/*
   VRML routine to transform VRML (and openGL) colors into RGB triples
   derived from 'colors.inc' of POV

   If the color information is "float1 float2 float3", then extract the rgb
   values from the input.   LWF 9/30/05
*/

  i = strlen(input);
  if (i == 0) return;
  if (input[i-1] < ' ') input[i-1] = 0;        // get rid of any ^J character
  transp[0] = '\0';
  transp_start = strstr (input, "filter");
  if (transp_start) {
    fval = (float) atof (transp_start + 6);
    sprintf (transp, " transparency %5.2f", fval / 2.);
  }
  if (strtod(input,&a) || a != input) {             // rgb form of color
      sscanf(input,"%f %f %f", &thered, &thegreen, &theblue);
      sprintf(input,"%.6f %.6f %.6f",thered,thegreen,theblue);
      strcat(input, transp);
      return;
  }
  if (!strncmp (input, "Black", 5)) {
    strncpy (input, "0 0 0\0", 6);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "Brass", 5)) {
    strncpy (input, ".71 .65 .26\0", 12);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "Bronze2", 7)) {
    strncpy (input, ".65 .49 .24\0", 12);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "Bronze", 6)) {
    strncpy (input, ".55 .47 .14\0", 12);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "BlueViolet", 10)) {
    strncpy (input, ".62352 .372549 .623529\0", 24);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "Blue", 3)) {
    strncpy (input, "0 0 1\0", 6);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "Brown", 5)) {
    strncpy (input, ".647059 .164706 .164706\0", 24);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "Copper", 6)) {
    strncpy (input, ".72 .45 .20\0", 12);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "Cyan", 3)) {
    strncpy (input, "0 1 1\0", 6);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "DimGray", 7)) {
    strncpy (input, ".329412 .329412 .329412\0", 24);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "DimGrey", 7)) {
    strncpy (input, ".329412 .329412 .329412\0", 24);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "Aquamarine", 10)) {
    strncpy (input, ".439216 .858824 .576471\0", 24);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "CadetBlue", 9)) {
    strncpy (input, ".372549 .623529 .623529\0", 24);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "Coral", 5)) {
    strncpy (input, "1.0 .498039 .0\0", 24);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "CornflowerBlue", 14)) {
    strncpy (input, ".258824 .258824 .435294\0", 24);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "DarkGreen", 9)) {
    strncpy (input, ".184314 .309804 .184314\0", 24);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "DarkOliveGreen", 14)) {
    strncpy (input, ".309804 .309804 .184314\0", 24);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "DarkOrchid", 10)) {
    strncpy (input, ".6 .196078 .8\0", 24);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "DarkSlateBlue", 13)) {
    strncpy (input, ".119608 .137255 .556863\0", 24);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "DarkSlateGray", 13)) {
    strncpy (input, ".184314 .309804 .309804\0", 24);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "DarkSlateGrey", 13)) {
    strncpy (input, ".184314 .309804 .309804\0", 24);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "DarkTurquoise", 13)) {
    strncpy (input, ".439216 .576471 .858824\0", 24);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "Firebrick", 9)) {
    strncpy (input, ".556863 .137255 .137255\0", 24);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "ForestGreen", 11)) {
    strncpy (input, ".137255 .556863 .137255\0", 24);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "Goldenrod", 9)) {
    strncpy (input, ".858824 .858824 .439216\0", 24);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "Gold", 4)) {
    strncpy (input, ".8 .498039 .196078\0", 24);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "Gray05", 6)) {
    strncpy (input, ".05 .05 .05\0", 12);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "Gray10", 6)) {
    strncpy (input, ".10 .10 .10\0", 12);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "Gray15", 6)) {
    strncpy (input, ".15 .15 .15\0", 12);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "Gray20", 6)) {
    strncpy (input, ".20 .20 .20\0", 12);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "Gray25", 6)) {
    strncpy (input, ".25 .25 .25\0", 12);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "Gray30", 6)) {
    strncpy (input, ".30 .30 .30\0", 12);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "Gray35", 6)) {
    strncpy (input, ".35 .35 .35\0", 12);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "Gray40", 6)) {
    strncpy (input, ".40 .40 .40\0", 12);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "Gray45", 6)) {
    strncpy (input, ".45 .45 .45\0", 12);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "Gray50", 6)) {
    strncpy (input, ".50 .50 .50\0", 12);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "Gray55", 6)) {
    strncpy (input, ".55 .55 .55\0", 12);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "Gray60", 6)) {
    strncpy (input, ".60 .60 .60\0", 12);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "Gray65", 6)) {
    strncpy (input, ".65 .65 .65\0", 12);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "Gray70", 6)) {
    strncpy (input, ".70 .70 .70\0", 12);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "Gray75", 6)) {
    strncpy (input, ".75 .75 .75\0", 12);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "Gray80", 6)) {
    strncpy (input, ".80 .80 .80\0", 12);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "Gray85", 6)) {
    strncpy (input, ".85 .85 .85\0", 12);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "Gray90", 6)) {
    strncpy (input, ".90 .90 .90\0", 12);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "Gray95", 6)) {
    strncpy (input, ".95 .95 .95\0", 12);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "Gray", 4)) {
    strncpy (input, ".752941 .752941 .752941\0", 24);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "Grey", 4)) {
    strncpy (input, ".752941 .752941 .752941\0", 24);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "GreenYellow", 11)) {
    strncpy (input, ".576471 .858824 .439216\0", 24);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "Green", 5)) {
    strncpy (input, "0 1 0\0", 6);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "IndianRed", 9)) {
    strncpy (input, ".309804 .184314 .184314\0", 24);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "Khaki", 5)) {
    strncpy (input, ".623529 .623529 .372549\0", 24);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "LightBlue", 9)) {
    strncpy (input, ".74902 .847059 .847059\0", 24);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "LightGray", 9)) {
    strncpy (input, ".658824 .658824 .658824\0", 24);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "LightGrey", 9)) {
    strncpy (input, ".658824 .658824 .658824\0", 24);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "Light_Purple", 12)) {
    strncpy (input, ".87 .58 .98\0", 12);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "LightSteelBlue", 14)) {
    strncpy (input, ".560784 .560784 .737255\0", 24);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "LimeGreen", 9)) {
    strncpy (input, ".196078 .8 .196078\0", 24);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "Magenta", 7)) {
    strncpy (input, "1 0 1\0", 6);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "Maroon", 6)) {
    strncpy (input, ".556863 .137255 .419608\0", 24);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "MediumAquamarine", 16)) {
    strncpy (input, ".196078 .8 .6\0", 24);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "MediumBlue", 10)) {
    strncpy (input, ".196078 .196078 .8\0", 24);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "MediumForestGreen", 17)) {
    strncpy (input, ".419608 .556863 .137255\0", 24);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "MediumGoldenrod", 15)) {
    strncpy (input, ".917647 .917647 .678431\0", 24);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "MediumOrchid", 12)) {
    strncpy (input, ".576471 .439216 .858824\0", 24);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "MediumSeaGreen", 14)) {
    strncpy (input, ".258824 .435294 .258824\0", 24);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "MediumSlateBlue", 15)) {
    strncpy (input, ".498039 0.  1.0\0", 24);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "MediumSpringGreen", 17)) {
    strncpy (input, ".498039 1.0 0\0", 24);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "MediumTurquoise", 15)) {
    strncpy (input, ".439216 .858824 .858824\0", 24);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "MediumVioletRed", 15)) {
    strncpy (input, ".858824 .439216 .576471\0", 24);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "MidnightBlue", 12)) {
    strncpy (input, ".184314 .184314 .309804\0", 24);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "NavyBlue", 8)) {
    strncpy (input, ".137255 .137255 .556863\0", 24);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "Navy", 4)) {
    strncpy (input, ".137255 .137255 .556863\0", 24);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "OrangeRed", 9)) {
    strncpy (input, "1.0 .498039 0.0\0", 24);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "Orange", 6)) {
    strncpy (input, "1 .5 .0\0", 24);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "Orchid", 6)) {
    strncpy (input, ".858824 .439216 .858824\0", 24);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "PaleGreen", 9)) {
    strncpy (input, ".560784 .737255 .560784\0", 24);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "Pink", 4)) {
    strncpy (input, ".737255 .560784 .560784\0", 24);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "Plum", 4)) {
    strncpy (input, ".917647 .678431 .917647\0", 24);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "Red", 3)) {
    strncpy (input, "1 0 0\0", 6);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "RichBlue", 8)) {
    strncpy (input, ".35 .35 .67\0", 12);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "Salmon", 6)) {
    strncpy (input, ".435294 .258824 .258824\0", 24);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "SeaGreen", 8)) {
    strncpy (input, ".137255 .556863 .419608\0", 24);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "Sienna", 6)) {
    strncpy (input, ".556863 .419608 .137255\0", 24);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "SkyBlue", 7)) {
    strncpy (input, ".196078 .6 .8\0", 14);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "SlateBlue", 9)) {
    strncpy (input, "0 .498039 1\0", 12);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "SpringGreen", 11)) {
    strncpy (input, "0 1 .498039\0", 12);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "SteelBlue", 9)) {
    strncpy (input, ".137255 .419608 .556863\0", 24);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "SummerSky", 9)) {
    strncpy (input, ".22 .69 .87\0", 12);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "Tan", 3)) {
    strncpy (input, ".858824 .576471 .439216\0", 24);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "Thistle", 7)) {
    strncpy (input, ".847059 .74902 .847059\0", 23);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "Turquoise", 9)) {
    strncpy (input, ".678431 .917647 .917647\0", 24);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "VioletRed", 9)) {
    strncpy (input, ".8 .196078 .6\0", 14);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "Violet", 6)) {
    strncpy (input, ".309804 .184314 .309804\0", 24);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "VLightGrey", 10)) {
    strncpy (input, ".8 .8 .8\0", 9);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "Wheat", 5)) {
    strncpy (input, ".847059 .847059 .74902\0", 23);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "Silver", 6)) {
    strncpy (input, ".90 .91 .98\0", 12);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "BrightGold", 10)) {
    strncpy (input, ".85 .85 .10\0", 12);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "OldGold", 7)) {
    strncpy (input, ".81 .71 .23\0", 12);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "Feldspar", 8)) {
    strncpy (input, ".82 .57 .46\0", 12);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "Quartz", 6)) {
    strncpy (input, ".85 .85 .95\0", 12);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "Mica", 4)) {
    strncpy (input, "0 0 0\0", 6);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "NeonPink", 8)) {
    strncpy (input, "1 .43 .78\0", 10);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "DarkPurple", 10)) {
    strncpy (input, ".53 .12 .47\0", 12);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "NeonBlue", 8)) {
    strncpy (input, ".3 .3 1\0", 8);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "CoolCopper", 10)) {
    strncpy (input, ".85 .53 .10\0", 12);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "MandarinOrange", 14)) {
    strncpy (input, ".89 .47 .20\0", 12);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "LightWood", 9)) {
    strncpy (input, ".91 .76 .65\0", 12);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "MediumWood", 10)) {
    strncpy (input, ".65 .50 .39\0", 12);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "DarkWood", 8)) {
    strncpy (input, ".52 .37 .26\0", 12);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "SpicyPink", 9)) {
    strncpy (input, "1 .11 .68\0", 10);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "SemiSweetChoc", 13)) {
    strncpy (input, ".42 .26 .15\0", 12);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "BakersChoc", 10)) {
    strncpy (input, ".36 .20 .09\0", 12);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "Flesh", 5)) {
    strncpy (input, ".96 .80 .69\0", 12);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "NewTan", 6)) {
    strncpy (input, ".92 .78 .62\0", 12);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "NewMidnightBlue", 15)) {
    strncpy (input, "0 0 .61\0", 8);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "VeryDarkBrown", 13)) {
    strncpy (input, ".35 .16 .14\0", 12);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "DarkBrown", 9)) {
    strncpy (input, ".36 .25 .20\0", 12);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "DarkTan", 7)) {
    strncpy (input, ".59 .41 .31\0", 12);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "GreenCopper", 11)) {
    strncpy (input, ".32 .49 .46\0", 12);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "DkGreenCopper", 13)) {
    strncpy (input, ".29 .46 .43\0", 12);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "DustyRose", 9)) {
    strncpy (input, ".52 .39 .39\0", 12);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "HuntersGreen", 12)) {
    strncpy (input, ".13 .37 .31\0", 12);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "Scarlet", 7)) {
    strncpy (input, ".55 .09 .09\0", 12);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "Med_Purple", 10)) {
    strncpy (input, ".73 .16 .96\0", 12);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "White", 5)) {
    strncpy (input, "1 1 1\0", 6);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "Very_Light_Purple", 17)) {
    strncpy (input, ".94 .81 .99\0", 12);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "YellowGreen", 11)) {
    strncpy (input, ".6 .8 .196078\0", 14);
    strcat (input, transp);
    return;
  }
  if (!strncmp (input, "Yellow", 6)) {
    strncpy (input, "1 1 0\0", 6);
    strcat (input, transp);
    return;
  }

/* try to find it in colors.inc, if povray options include +L path or if POV_Include is non-blank */
  colinc = NULL;
  if (strlen(drvui->POV_Include) > 10) {
      colinc = fopen(drvui->POV_Include,"r");
  }
  if (!colinc) {
      strncpy(b,drvui->POV_Options,29);
      b[29]='\0';
      a=strstr(b,"+L");
      if (a!=NULL) {
        sscanf(a+2,"%s",POV_incpath);
        if (POV_incpath!=NULL) {
          strcat(POV_incpath,"/colors.inc");
          colinc=fopen(POV_incpath,"r");
          if (colinc) {
            strcpy(drvui->POV_Include,POV_incpath);
            WriteConfig();
          }
        }
      }
  }
  if (colinc!=NULL){
      while (!feof(colinc)){
        if (fgets(line,80,colinc)==NULL)continue;
        sscanf(line,"%s %s = color red %f green %f blue %f",keyword,thecolor,
              &thered,&thegreen,&theblue); 
        if (strcmp(keyword,"#declare"))continue;
        if (!strcmp(thecolor,input)) {
          snprintf(input,30, "%3.2f %3.2f %3.2f%c",thered,thegreen,theblue,0);
          if (transp_start == NULL) {
            transp[0] = '\0';
            transp_start = strstr (line, "filter");
            if (transp_start != NULL) {
              fval = (float) atof (transp_start + 6);
              sprintf (transp, " transparency %5.2f", fval / 2.);
            }
          }
          strcat (input,transp);
          fclose(colinc);
          return;
        }
      }
      fclose(colinc);
  }
  
  if (Color_Warning <= 5) {
      char string[256];
      sprintf(string," **** Warning - Non-standard color \"%s\" encountered. ****",
         input);
      Error_Box(string);
      Color_Warning++;
  }

  if (sscanf(input," %f %f %f",&thered,&thegreen,&theblue) <3) {
    strcpy(input,"1. 1. 1.");    // substitute White if no color triplet
  } else {
    snprintf(input,30, "%3.2f %3.2f %3.2f%c",thered,thegreen,theblue,0);
    strcat (input, transp);
  }
}

/* ************************************************************** */
/* ************************************************************** */

void trim_string(char string[], int len)
{
// trim string - remove trailing ^J (Windows) and spaces. 'len' is maximum length
    int i;
    string[len-1] = 0;
    if (strlen(string) == 0) {
//        strcpy(string,"White");
        return;
    }
    for (i=0;i<(int)strlen(string);i++) {
        if ((unsigned char)string[i] < ' ') {
            string[i] = 0;
            break;
        }
    }
    for (i=strlen(string);i > 0; --i) {
        if (string[i-1] <= ' ') {
            string[i-1] = 0;
        } else
            break;
    }
}

/* ************************************************************** */
/* ************************************************************** */

int Unique_Atom (void)

// routine to check if atom at position 'natom' is unique. Returns 1 if it
//   is. If the position is a duplicate, returns 0.
{
  int j, k, test;

  for (k = 0; k < natom - 1; k++) {         /* loop through atoms */
    if (drvui->atom_fn[k] != drvui->frame_no) continue;
    if (drvui->atom_n[k] == drvui->atom_n[natom]) {
      test = 0;
      for (j = 0; j < 4; j++) {
    if (drvui->atom_l[k][j] != drvui->atom_l[natom][j])
      test = 1;
      }
      if (!test) {                     /* name and number the same - check coordinates */
    expand_atom (natom);             /* find all of test atoms in unit cell */
    for (j = 0; j < ncell; j++) {
      if ((fabs (drvui->atom_xyz[k][1] - drvui->cell_xyz[j][1]) < 0.0001) &&
          (fabs (drvui->atom_xyz[k][2] - drvui->cell_xyz[j][2]) < 0.0001) &&
          (fabs (drvui->atom_xyz[k][0] - drvui->cell_xyz[j][0]) < 0.0001))
        return (0);                 /* duplicate */
    }
      }
    }
  }
  return (1);                     /* unique */
}

/* ************************************************************** */
/* ************************************************************** */

int vec_dif(int n1, float v1[3], int n2, float v2[3], int n3, float v3[3], float v[3])
{
/* routine to determine if vector v == n1*v1+n2*v2+n3*v3, returns 1 if equal, 0 if not */
    int i;
    float d[3];

    for (i=0; i<3; i++)
        d[i] = v[i] - n1 * v1[i] - n2 * v2[i] - n3 * v3[i];
    return d[0] * d[0] + d[1] * d[1] + d[2] * d[2] < 1.0e-4 ? 1 : 0;
}


